@@ -1,10 +1,10 @@
 import os
 import shutil
 
-tag = os.environ['TAG'].split("refs/tags/")[1]
+tag = os.environ['TAG_VAL']
 scan_dir = os.environ['SCAN_DIR']
 output_dir = os.environ['OUT_DIR']
-accepted_exts = ["AppImage", "tar.gz", "7z"]
+accepted_exts = ["AppImage", "flatpak", "tar.xz", "7z"]
 
 
 for dir_name in os.listdir(scan_dir):
@@ -12,9 +12,12 @@ for dir_name in os.listdir(scan_dir):
   if "macos" in dir_name.lower():
     asset_name += "-macos"
   elif "linux" in dir_name.lower():
-    asset_name += "-linux-AppImage-64bit"
+    if "flatpak" in dir_name.lower():
+      asset_name += "-linux-flatpak-x64"
+    else:
+      asset_name += "-linux-appimage-x64"
   elif "windows" in dir_name.lower():
-    asset_name += "-windows-64bit"
+    asset_name += "-windows-x64"
   else:
     continue;
 
