@@ -1,97 +1,8 @@
-/*  PCSX2 - PS2 Emulator for PCs
- *  Copyright (C) 2002-2010  PCSX2 Dev Team
- *
- *  PCSX2 is free software: you can redistribute it and/or modify it under the terms
- *  of the GNU Lesser General Public License as published by the Free Software Found-
- *  ation, either version 3 of the License, or (at your option) any later version.
- *
- *  PCSX2 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
- *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
- *  PURPOSE.  See the GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License along with PCSX2.
- *  If not, see <http://www.gnu.org/licenses/>.
- */
+// SPDX-FileCopyrightText: 2002-2025 PCSX2 Dev Team
+// SPDX-License-Identifier: GPL-3.0+
 
 #include "common/emitter/internal.h"
-#include "common/emitter/tools.h"
-
-// Mask of valid bit fields for the target CPU.  Typically this is either 0xFFFF (SSE2
-// or better) or 0xFFBF (SSE1 and earlier).  Code can ensure a safe/valid MXCSR by
-// AND'ing this mask against an MXCSR prior to LDMXCSR.
-SSE_MXCSR MXCSR_Mask;
-
-const char* EnumToString(SSE_RoundMode sse)
-{
-	switch (sse)
-	{
-		case SSEround_Nearest:
-			return "Nearest";
-		case SSEround_NegInf:
-			return "NegativeInfinity";
-		case SSEround_PosInf:
-			return "PositiveInfinity";
-		case SSEround_Chop:
-			return "Chop";
-		default:
-			return "Invalid";
-	}
-}
-
-SSE_MXCSR SSE_MXCSR::GetCurrent()
-{
-	SSE_MXCSR ret;
-	ret.bitmask = _mm_getcsr();
-	return ret;
-}
-
-void SSE_MXCSR::SetCurrent(const SSE_MXCSR& value)
-{
-	_mm_setcsr(value.bitmask);
-}
-
-SSE_RoundMode SSE_MXCSR::GetRoundMode() const
-{
-	return (SSE_RoundMode)RoundingControl;
-}
-
-SSE_MXCSR& SSE_MXCSR::SetRoundMode(SSE_RoundMode mode)
-{
-	pxAssert((uint)mode < 4);
-	RoundingControl = (u32)mode;
-	return *this;
-}
-
-SSE_MXCSR& SSE_MXCSR::ClearExceptionFlags()
-{
-	bitmask &= ~0x3f;
-	return *this;
-}
-
-SSE_MXCSR& SSE_MXCSR::EnableExceptions()
-{
-	bitmask &= ~(0x3f << 7);
-	return *this;
-}
-
-SSE_MXCSR& SSE_MXCSR::DisableExceptions()
-{
-	bitmask |= 0x3f << 7;
-	return *this;
-}
-
-// Applies the reserve bits mask for the current running cpu, as fetched from the CPU
-// during CPU init/detection.
-SSE_MXCSR& SSE_MXCSR::ApplyReserveMask()
-{
-	bitmask &= MXCSR_Mask.bitmask;
-	return *this;
-}
-
-SSE_MXCSR::operator x86Emitter::xIndirect32() const
-{
-	return x86Emitter::ptr32[&bitmask];
-}
+#include "common/VectorIntrin.h"
 
 namespace x86Emitter
 {
@@ -645,12 +556,18 @@ namespace x86Emitter
 	const xImplSimd_MovHL_RtoR xMOVLH = {0x16};
 	const xImplSimd_MovHL_RtoR xMOVHL = {0x12};
 
+	const xImplSimd_PBlend xPBLEND =
+	{
+		{0x66, 0x0e3a}, // W
+		{0x66, 0x1038}, // VB
+	};
+
 	const xImplSimd_Blend xBLEND =
-		{
-			{0x66, 0x0c3a}, // PS
-			{0x66, 0x0d3a}, // PD
-			{0x66, 0x1438}, // VPS
-			{0x66, 0x1538}, // VPD
+	{
+		{0x66, 0x0c3a}, // PS
+		{0x66, 0x0d3a}, // PD
+		{0x66, 0x1438}, // VPS
+		{0x66, 0x1538}, // VPD
 	};
 
 	const xImplSimd_PMove xPMOVSX = {0x2038};
