@@ -1,19 +1,5 @@
-/*  PCSX2 - PS2 Emulator for PCs
- *  Copyright (C) 2002-2022  PCSX2 Dev Team
- *
- *  PCSX2 is free software: you can redistribute it and/or modify it under the terms
- *  of the GNU Lesser General Public License as published by the Free Software Found-
- *  ation, either version 3 of the License, or (at your option) any later version.
- *
- *  PCSX2 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
- *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
- *  PURPOSE.  See the GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License along with PCSX2.
- *  If not, see <http://www.gnu.org/licenses/>.
- */
-
-#include "PrecompiledHeader.h"
+// SPDX-FileCopyrightText: 2002-2025 PCSX2 Dev Team
+// SPDX-License-Identifier: GPL-3.0+
 
 #include "GameListModel.h"
 #include "QtHost.h"
@@ -39,17 +25,17 @@ static constexpr int COVER_ART_HEIGHT = 512;
 static constexpr int COVER_ART_SPACING = 32;
 static constexpr int MIN_COVER_CACHE_SIZE = 256;
 
-static int DPRScale(int size, float dpr)
+static int DPRScale(int size, qreal dpr)
 {
-	return static_cast<int>(static_cast<float>(size) * dpr);
+	return static_cast<int>(static_cast<qreal>(size) * dpr);
 }
 
-static int DPRUnscale(int size, float dpr)
+static int DPRUnscale(int size, qreal dpr)
 {
-	return static_cast<int>(static_cast<float>(size) / dpr);
+	return static_cast<int>(static_cast<qreal>(size) / dpr);
 }
 
-static void resizeAndPadPixmap(QPixmap* pm, int expected_width, int expected_height, float dpr)
+static void resizeAndPadPixmap(QPixmap* pm, int expected_width, int expected_height, qreal dpr)
 {
 	const int dpr_expected_width = DPRScale(expected_width, dpr);
 	const int dpr_expected_height = DPRScale(expected_height, dpr);
@@ -85,9 +71,8 @@ static void resizeAndPadPixmap(QPixmap* pm, int expected_width, int expected_hei
 }
 
 static QPixmap createPlaceholderImage(const QPixmap& placeholder_pixmap, int width, int height, float scale,
-	const std::string& title)
+	qreal dpr, const std::string& title)
 {
-	const float dpr = qApp->devicePixelRatio();
 	QPixmap pm(placeholder_pixmap.copy());
 	pm.setDevicePixelRatio(dpr);
 	if (pm.isNull())
@@ -127,18 +112,21 @@ const char* GameListModel::getColumnName(Column col)
 	return s_column_names[static_cast<int>(col)];
 }
 
-GameListModel::GameListModel(QObject* parent /* = nullptr */)
+GameListModel::GameListModel(float cover_scale, bool show_cover_titles, qreal dpr, QObject* parent /* = nullptr */)
 	: QAbstractTableModel(parent)
-	, m_cover_pixmap_cache(MIN_COVER_CACHE_SIZE)
+	, m_show_titles_for_covers(show_cover_titles)
+	, m_dpr{dpr}
 {
+	loadSettings();
 	loadCommonImages();
+	setCoverScale(cover_scale);
 	setColumnDisplayNames();
 }
 GameListModel::~GameListModel() = default;
 
-void GameListModel::refreshImages()
+void GameListModel::reloadThemeSpecificImages()
 {
-	loadCommonImages();
+	loadThemeSpecificImages();
 	refresh();
 }
 
@@ -152,6 +140,8 @@ void GameListModel::setCoverScale(float scale)
 	m_cover_scale_counter.fetch_add(1, std::memory_order_release);
 	m_loading_pixmap = QPixmap(getCoverArtWidth(), getCoverArtHeight());
 	m_loading_pixmap.fill(QColor(0, 0, 0, 0));
+
+	emit coverScaleChanged();
 }
 
 void GameListModel::refreshCovers()
@@ -170,31 +160,39 @@ void GameListModel::updateCacheSize(int width, int height)
 	m_cover_pixmap_cache.SetMaxCapacity(static_cast<int>(std::max(num_columns * num_rows, MIN_COVER_CACHE_SIZE)));
 }
 
+void GameListModel::setDevicePixelRatio(qreal dpr)
+{
+	m_dpr = dpr;
+	loadCommonImages();
+	refreshCovers();
+}
+
 void GameListModel::loadOrGenerateCover(const GameList::Entry* ge)
 {
 	// Why this counter: Every time we change the cover scale, we increment the counter variable. This way if the scale is changed
 	// while there's outstanding jobs, the old jobs won't proceed (at the wrong size), or get added into the grid.
 	const u32 counter = m_cover_scale_counter.load(std::memory_order_acquire);
 
-	QFuture<QPixmap> future = QtConcurrent::run([this, path = ge->path, title = ge->title, serial = ge->serial, counter]() -> QPixmap {
+	QFuture<QPixmap> future = QtConcurrent::run([this, entry = *ge, counter]() -> QPixmap {
 		QPixmap image;
 		if (m_cover_scale_counter.load(std::memory_order_acquire) == counter)
 		{
-			const std::string cover_path(GameList::GetCoverImagePath(path, serial, title));
+			const std::string cover_path(GameList::GetCoverImagePathForEntry(&entry));
 			if (!cover_path.empty())
 			{
-				const float dpr = qApp->devicePixelRatio();
 				image = QPixmap(QString::fromStdString(cover_path));
 				if (!image.isNull())
 				{
-					image.setDevicePixelRatio(dpr);
-					resizeAndPadPixmap(&image, getCoverArtWidth(), getCoverArtHeight(), dpr);
+					image.setDevicePixelRatio(m_dpr);
+					resizeAndPadPixmap(&image, getCoverArtWidth(), getCoverArtHeight(), m_dpr);
 				}
 			}
 		}
 
+		const std::string& title = entry.GetTitle(m_prefer_english_titles);
+
 		if (image.isNull())
-			image = createPlaceholderImage(m_placeholder_pixmap, getCoverArtWidth(), getCoverArtHeight(), m_cover_scale, title);
+			image = createPlaceholderImage(m_placeholder_pixmap, getCoverArtWidth(), getCoverArtHeight(), m_cover_scale, m_dpr, title);
 
 		if (m_cover_scale_counter.load(std::memory_order_acquire) != counter)
 			image = {};
@@ -267,6 +265,17 @@ int GameListModel::columnCount(const QModelIndex& parent) const
 	return Column_Count;
 }
 
+QString GameListModel::formatTimespan(time_t timespan)
+{
+	// avoid an extra string conversion
+	const u32 hours = static_cast<u32>(timespan / 3600);
+	const u32 minutes = static_cast<u32>((timespan % 3600) / 60);
+	if (hours > 0)
+		return qApp->translate("GameList", "%n hours", "", hours);
+	else
+		return qApp->translate("GameList", "%n minutes", "", minutes);
+}
+
 QVariant GameListModel::data(const QModelIndex& index, int role) const
 {
 	if (!index.isValid())
@@ -291,7 +300,7 @@ QVariant GameListModel::data(const QModelIndex& index, int role) const
 					return QString::fromStdString(ge->serial);
 
 				case Column_Title:
-					return QString::fromStdString(ge->title);
+					return QString::fromStdString(ge->GetTitle(m_prefer_english_titles));
 
 				case Column_FileTitle:
 					return QtUtils::StringViewToQString(Path::GetFileTitle(ge->path));
@@ -304,7 +313,7 @@ QVariant GameListModel::data(const QModelIndex& index, int role) const
 					if (ge->total_played_time == 0)
 						return {};
 					else
-						return QString::fromStdString(GameList::FormatTimespan(ge->total_played_time, true));
+						return formatTimespan(ge->total_played_time);
 				}
 
 				case Column_LastPlayed:
@@ -316,7 +325,7 @@ QVariant GameListModel::data(const QModelIndex& index, int role) const
 				case Column_Cover:
 				{
 					if (m_show_titles_for_covers)
-						return QString::fromStdString(ge->title);
+						return QString::fromStdString(ge->GetTitle(m_prefer_english_titles));
 					else
 						return {};
 				}
@@ -338,7 +347,7 @@ QVariant GameListModel::data(const QModelIndex& index, int role) const
 
 				case Column_Title:
 				case Column_Cover:
-					return QString::fromStdString(ge->title);
+					return QString::fromStdString(ge->GetTitleSort(m_prefer_english_titles));
 
 				case Column_FileTitle:
 					return QtUtils::StringViewToQString(Path::GetFileTitle(ge->path));
@@ -385,7 +394,7 @@ QVariant GameListModel::data(const QModelIndex& index, int role) const
 					return m_compatibility_pixmaps[static_cast<u32>(
 						(static_cast<u32>(ge->compatibility_rating) >= GameList::CompatibilityRatingCount) ?
 							GameList::CompatibilityRating::Unknown :
-                            ge->compatibility_rating)];
+							ge->compatibility_rating)];
 				}
 
 				case Column_Cover:
@@ -422,6 +431,7 @@ QVariant GameListModel::headerData(int section, Qt::Orientation orientation, int
 void GameListModel::refresh()
 {
 	beginResetModel();
+	loadSettings();
 	endResetModel();
 }
 
@@ -435,7 +445,8 @@ bool GameListModel::titlesLessThan(int left_row, int right_row) const
 
 	const GameList::Entry* left = GameList::GetEntryByIndex(left_row);
 	const GameList::Entry* right = GameList::GetEntryByIndex(right_row);
-	return (StringUtil::Strcasecmp(left->title.c_str(), right->title.c_str()) < 0);
+	return QtHost::LocaleSensitiveCompare(QString::fromStdString(left->GetTitleSort(m_prefer_english_titles)),
+			   QString::fromStdString(right->GetTitleSort(m_prefer_english_titles))) < 0;
 }
 
 bool GameListModel::lessThan(const QModelIndex& left_index, const QModelIndex& right_index, int column) const
@@ -542,40 +553,49 @@ bool GameListModel::lessThan(const QModelIndex& left_index, const QModelIndex& r
 	}
 }
 
+void GameListModel::loadSettings()
+{
+	m_prefer_english_titles = Host::GetBaseBoolSettingValue("UI", "PreferEnglishGameList", false);
+}
+
 QIcon GameListModel::getIconForType(GameList::EntryType type)
 {
 	switch (type)
 	{
 		case GameList::EntryType::PS2Disc:
 		case GameList::EntryType::PS1Disc:
-			return QIcon(QStringLiteral(":/icons/media-optical-24.png"));
+			return QIcon::fromTheme("disc-2-line");
 
 		case GameList::EntryType::ELF:
 		default:
-			return QIcon(QStringLiteral(":/icons/applications-system-24.png"));
+			return QIcon::fromTheme("file-settings-line");
 	}
 }
 
 QIcon GameListModel::getIconForRegion(GameList::Region region)
 {
 	return QIcon(
-		QStringLiteral("%1/icons/flags/%2.png").arg(QtHost::GetResourcesBasePath()).arg(GameList::RegionToString(region)));
+		QStringLiteral("%1/icons/flags/%2.svg").arg(QtHost::GetResourcesBasePath()).arg(GameList::RegionToString(region, false)));
+}
+
+void GameListModel::loadThemeSpecificImages()
+{
+	for (u32 type = 0; type < static_cast<u32>(GameList::EntryType::Count); type++)
+		m_type_pixmaps[type] = getIconForType(static_cast<GameList::EntryType>(type)).pixmap(QSize(24, 24), m_dpr);
+
+	for (u32 i = 0; i < static_cast<u32>(GameList::Region::Count); i++)
+		m_region_pixmaps[i] = getIconForRegion(static_cast<GameList::Region>(i)).pixmap(QSize(36, 26), m_dpr);
 }
 
 void GameListModel::loadCommonImages()
 {
-	for (u32 type = 0; type < static_cast<u32>(GameList::EntryType::Count); type++)
-		m_type_pixmaps[type] = getIconForType(static_cast<GameList::EntryType>(type)).pixmap(QSize(24, 24));
-
-	for (u32 i = 0; i < static_cast<u32>(GameList::Region::Count); i++)
-		m_region_pixmaps[i] = getIconForRegion(static_cast<GameList::Region>(i)).pixmap(QSize(42, 30));
+	loadThemeSpecificImages();
 
 	const QString base_path(QtHost::GetResourcesBasePath());
 	for (u32 i = 1; i < GameList::CompatibilityRatingCount; i++)
-		m_compatibility_pixmaps[i].load(QStringLiteral("%1/icons/star-%2.png").arg(base_path).arg(i - 1));
+		m_compatibility_pixmaps[i] = QIcon((QStringLiteral("%1/icons/star-%2.svg").arg(base_path).arg(i - 1))).pixmap(QSize(88, 16), m_dpr);
 
 	m_placeholder_pixmap.load(QStringLiteral("%1/cover-placeholder.png").arg(base_path));
-	setCoverScale(1.0f);
 }
 
 void GameListModel::setColumnDisplayNames()
