@@ -1,33 +1,5 @@
-/*  PCSX2 - PS2 Emulator for PCs
- *  Copyright (C) 2002-2022  PCSX2 Dev Team
- *
- *  PCSX2 is free software: you can redistribute it and/or modify it under the terms
- *  of the GNU Lesser General Public License as published by the Free Software Found-
- *  ation, either version 3 of the License, or (at your option) any later version.
- *
- *  PCSX2 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
- *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
- *  PURPOSE.  See the GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License along with PCSX2.
- *  If not, see <http://www.gnu.org/licenses/>.
- */
-
-#include "PrecompiledHeader.h"
-
-#include "common/Assertions.h"
-#include "common/StringUtil.h"
-
-#include "pcsx2/Frontend/GameList.h"
-#include "pcsx2/HostSettings.h"
-
-#include <QtCore/QSortFilterProxyModel>
-#include <QtGui/QPixmap>
-#include <QtGui/QWheelEvent>
-#include <QtWidgets/QApplication>
-#include <QtWidgets/QHeaderView>
-#include <QtWidgets/QMenu>
-#include <QtWidgets/QScrollBar>
+// SPDX-FileCopyrightText: 2002-2025 PCSX2 Dev Team
+// SPDX-License-Identifier: GPL-3.0+
 
 #include "GameListModel.h"
 #include "GameListRefreshThread.h"
@@ -35,12 +7,32 @@
 #include "QtHost.h"
 #include "QtUtils.h"
 
+#include "pcsx2/GameList.h"
+#include "pcsx2/Host.h"
+
+#include "common/Assertions.h"
+#include "common/Console.h"
+#include "common/StringUtil.h"
+
 #include "fmt/format.h"
 
+#include <QtCore/QSortFilterProxyModel>
+#include <QtGui/QPainter>
+#include <QtGui/QPixmap>
+#include <QtGui/QPixmapCache>
+#include <QtGui/QWheelEvent>
+#include <QtWidgets/QApplication>
+#include <QtWidgets/QHeaderView>
+#include <QtWidgets/QMenu>
+#include <QtWidgets/QScrollBar>
+#include <QtWidgets/QStyledItemDelegate>
+
 static const char* SUPPORTED_FORMATS_STRING = QT_TRANSLATE_NOOP(GameListWidget,
 	".bin/.iso (ISO Disc Images)\n"
+	".mdf (Media Descriptor File)\n"
 	".chd (Compressed Hunks of Data)\n"
 	".cso (Compressed ISO)\n"
+	".zso (Compressed ISO)\n"
 	".gz (Gzip Compressed ISO)");
 
 static constexpr float MIN_SCALE = 0.1f;
@@ -86,7 +78,8 @@ public:
 			if (!m_filter_name.isEmpty() &&
 				!QString::fromStdString(entry->path).contains(m_filter_name, Qt::CaseInsensitive) &&
 				!QString::fromStdString(entry->serial).contains(m_filter_name, Qt::CaseInsensitive) &&
-				!QString::fromStdString(entry->title).contains(m_filter_name, Qt::CaseInsensitive))
+				!QString::fromStdString(entry->title).contains(m_filter_name, Qt::CaseInsensitive) &&
+				!QString::fromStdString(entry->title_en).contains(m_filter_name, Qt::CaseInsensitive))
 				return false;
 		}
 
@@ -105,6 +98,77 @@ private:
 	QString m_filter_name;
 };
 
+namespace
+{
+	class GameListIconStyleDelegate final : public QStyledItemDelegate
+	{
+	public:
+		GameListIconStyleDelegate(QWidget* parent)
+			: QStyledItemDelegate(parent)
+		{
+		}
+		~GameListIconStyleDelegate() = default;
+
+		void paint(QPainter* painter, const QStyleOptionViewItem& option, const QModelIndex& index) const override
+		{
+			// https://stackoverflow.com/questions/32216568/how-to-set-icon-center-in-qtableview
+			Q_ASSERT(index.isValid());
+
+			// Draw the base item, with a blank icon
+			QStyleOptionViewItem opt = option;
+			initStyleOption(&opt, index);
+			opt.icon = QIcon();
+			// Based on QStyledItemDelegate::paint()
+			const QStyle* style = option.widget ? option.widget->style() : QApplication::style();
+			style->drawControl(QStyle::CE_ItemViewItem, &opt, painter, option.widget);
+
+			// Fetch icon pixmap
+			const QRect r = option.rect;
+			const QPixmap pix = qvariant_cast<QPixmap>(index.data(Qt::DecorationRole));
+			const int pix_width = static_cast<int>(pix.width() / pix.devicePixelRatio());
+			const int pix_height = static_cast<int>(pix.height() / pix.devicePixelRatio());
+
+			// Clip the pixmaps so they don't extend outside the column
+			painter->save();
+			painter->setClipRect(option.rect);
+
+			// Draw the icon, using code derived from QItemDelegate::drawDecoration()
+			const bool enabled = option.state & QStyle::State_Enabled;
+			const QPoint p = QPoint((r.width() - pix_width) / 2, (r.height() - pix_height) / 2);
+			if (option.state & QStyle::State_Selected)
+			{
+				// See QItemDelegate::selectedPixmap()
+				QColor color = option.palette.color(enabled ? QPalette::Normal : QPalette::Disabled, QPalette::Highlight);
+				color.setAlphaF(0.3f);
+
+				QString key = QString::fromStdString(fmt::format("{:016X}-{:d}-{:08X}", pix.cacheKey(), enabled, color.rgba()));
+				QPixmap pm;
+				if (!QPixmapCache::find(key, &pm))
+				{
+					QImage img = pix.toImage().convertToFormat(QImage::Format_ARGB32_Premultiplied);
+
+					QPainter tinted_painter(&img);
+					tinted_painter.setCompositionMode(QPainter::CompositionMode_SourceAtop);
+					tinted_painter.fillRect(0, 0, img.width(), img.height(), color);
+					tinted_painter.end();
+
+					pm = QPixmap(QPixmap::fromImage(img));
+					QPixmapCache::insert(key, pm);
+				}
+
+				painter->drawPixmap(r.topLeft() + p, pm);
+			}
+			else
+			{
+				painter->drawPixmap(r.topLeft() + p, pix);
+			}
+
+			// Restore the old clip path.
+			painter->restore();
+		}
+	};
+} // namespace
+
 GameListWidget::GameListWidget(QWidget* parent /* = nullptr */)
 	: QWidget(parent)
 {
@@ -114,24 +178,29 @@ GameListWidget::~GameListWidget() = default;
 
 void GameListWidget::initialize()
 {
-	m_model = new GameListModel(this);
-	m_model->setCoverScale(Host::GetBaseFloatSettingValue("UI", "GameListCoverArtScale", 0.45f));
-	m_model->setShowCoverTitles(Host::GetBaseBoolSettingValue("UI", "GameListShowCoverTitles", true));
+	const float cover_scale = Host::GetBaseFloatSettingValue("UI", "GameListCoverArtScale", 0.45f);
+	const bool show_cover_titles = Host::GetBaseBoolSettingValue("UI", "GameListShowCoverTitles", true);
+	m_model = new GameListModel(cover_scale, show_cover_titles, devicePixelRatioF(), this);
 	m_model->updateCacheSize(width(), height());
 
 	m_sort_model = new GameListSortModel(m_model);
 	m_sort_model->setSourceModel(m_model);
 
 	m_ui.setupUi(this);
+
 	for (u32 type = 0; type < static_cast<u32>(GameList::EntryType::Count); type++)
 	{
-		m_ui.filterType->addItem(GameListModel::getIconForType(static_cast<GameList::EntryType>(type)),
-			qApp->translate("GameList", GameList::EntryTypeToDisplayString(static_cast<GameList::EntryType>(type))));
+		if (type != static_cast<u32>(GameList::EntryType::Invalid))
+		{
+			m_ui.filterType->addItem(GameListModel::getIconForType(static_cast<GameList::EntryType>(type)),
+				GameList::EntryTypeToString(static_cast<GameList::EntryType>(type), true));
+		}
 	}
+
 	for (u32 region = 0; region < static_cast<u32>(GameList::Region::Count); region++)
 	{
 		m_ui.filterRegion->addItem(GameListModel::getIconForRegion(static_cast<GameList::Region>(region)),
-			qApp->translate("GameList", GameList::RegionToString(static_cast<GameList::Region>(region))));
+			GameList::RegionToString(static_cast<GameList::Region>(region), true));
 	}
 
 	connect(m_ui.viewGameList, &QPushButton::clicked, this, &GameListWidget::showGameList);
@@ -155,6 +224,7 @@ void GameListWidget::initialize()
 	m_table_view->setSelectionBehavior(QAbstractItemView::SelectRows);
 	m_table_view->setContextMenuPolicy(Qt::CustomContextMenu);
 	m_table_view->setAlternatingRowColors(true);
+	m_table_view->setMouseTracking(true);
 	m_table_view->setShowGrid(false);
 	m_table_view->setCurrentIndex({});
 	m_table_view->horizontalHeader()->setHighlightSections(false);
@@ -162,6 +232,9 @@ void GameListWidget::initialize()
 	m_table_view->verticalHeader()->hide();
 	m_table_view->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
 	m_table_view->setVerticalScrollMode(QAbstractItemView::ScrollMode::ScrollPerPixel);
+	m_table_view->setItemDelegateForColumn(0, new GameListIconStyleDelegate(this));
+	m_table_view->setItemDelegateForColumn(8, new GameListIconStyleDelegate(this));
+	m_table_view->setItemDelegateForColumn(9, new GameListIconStyleDelegate(this));
 
 	loadTableViewColumnVisibilitySettings();
 	loadTableViewColumnSortSettings();
@@ -181,18 +254,16 @@ void GameListWidget::initialize()
 	m_list_view = new GameListGridListView(m_ui.stack);
 	m_list_view->setModel(m_sort_model);
 	m_list_view->setModelColumn(GameListModel::Column_Cover);
-	m_list_view->setSelectionMode(QAbstractItemView::ExtendedSelection);
+	m_list_view->setSelectionMode(QAbstractItemView::SingleSelection);
 	m_list_view->setViewMode(QListView::IconMode);
 	m_list_view->setResizeMode(QListView::Adjust);
 	m_list_view->setUniformItemSizes(true);
 	m_list_view->setItemAlignment(Qt::AlignHCenter);
 	m_list_view->setContextMenuPolicy(Qt::CustomContextMenu);
 	m_list_view->setFrameStyle(QFrame::NoFrame);
-	m_list_view->setSpacing(m_model->getCoverArtSpacing());
 	m_list_view->setVerticalScrollMode(QAbstractItemView::ScrollMode::ScrollPerPixel);
 	m_list_view->verticalScrollBar()->setSingleStep(15);
-
-	updateListFont();
+	onCoverScaleChanged();
 
 	connect(m_list_view->selectionModel(), &QItemSelectionModel::currentChanged, this,
 		&GameListWidget::onSelectionModelCurrentChanged);
@@ -200,6 +271,7 @@ void GameListWidget::initialize()
 	connect(m_list_view, &GameListGridListView::zoomOut, this, &GameListWidget::gridZoomOut);
 	connect(m_list_view, &QListView::activated, this, &GameListWidget::onListViewItemActivated);
 	connect(m_list_view, &QListView::customContextMenuRequested, this, &GameListWidget::onListViewContextMenuRequested);
+	connect(m_model, &GameListModel::coverScaleChanged, this, &GameListWidget::onCoverScaleChanged);
 
 	m_ui.stack->insertWidget(1, m_list_view);
 
@@ -215,6 +287,8 @@ void GameListWidget::initialize()
 	else
 		m_ui.stack->setCurrentIndex(0);
 
+	setFocusProxy(m_ui.stack->currentWidget());
+
 	updateToolbar();
 	resizeTableViewColumnsToFit();
 }
@@ -259,16 +333,19 @@ void GameListWidget::cancelRefresh()
 		QApplication::processEvents(QEventLoop::ExcludeUserInputEvents, 1);
 }
 
-void GameListWidget::refreshImages()
+void GameListWidget::reloadThemeSpecificImages()
 {
-	m_model->refreshImages();
+	m_model->reloadThemeSpecificImages();
 }
 
 void GameListWidget::onRefreshProgress(const QString& status, int current, int total)
 {
 	// switch away from the placeholder while we scan, in case we find anything
 	if (m_ui.stack->currentIndex() == 2)
+	{
 		m_ui.stack->setCurrentIndex(Host::GetBaseBoolSettingValue("UI", "GameListGridView", false) ? 1 : 0);
+		setFocusProxy(m_ui.stack->currentWidget());
+	}
 
 	m_model->refresh();
 	emit refreshProgress(status, current, total);
@@ -286,7 +363,10 @@ void GameListWidget::onRefreshComplete()
 
 	// if we still had no games, switch to the helper widget
 	if (m_model->rowCount() == 0)
+	{
 		m_ui.stack->setCurrentIndex(2);
+		setFocusProxy(nullptr);
+	}
 }
 
 void GameListWidget::onSelectionModelCurrentChanged(const QModelIndex& current, const QModelIndex& previous)
@@ -353,14 +433,23 @@ void GameListWidget::onTableViewHeaderSortIndicatorChanged(int, Qt::SortOrder)
 	saveTableViewColumnSortSettings();
 }
 
+void GameListWidget::onCoverScaleChanged()
+{
+	m_model->updateCacheSize(width(), height());
+
+	m_list_view->setSpacing(m_model->getCoverArtSpacing());
+
+	QFont font;
+	font.setPointSizeF(20.0f * m_model->getCoverScale());
+	m_list_view->setFont(font);
+}
+
 void GameListWidget::listZoom(float delta)
 {
 	const float new_scale = std::clamp(m_model->getCoverScale() + delta, MIN_SCALE, MAX_SCALE);
 	Host::SetBaseFloatSettingValue("UI", "GameListCoverArtScale", new_scale);
 	Host::CommitBaseSettingChanges();
 	m_model->setCoverScale(new_scale);
-	m_model->updateCacheSize(width(), height());
-	updateListFont();
 	updateToolbar();
 }
 
@@ -381,8 +470,6 @@ void GameListWidget::gridIntScale(int int_scale)
 	Host::SetBaseFloatSettingValue("UI", "GameListCoverArtScale", new_scale);
 	Host::CommitBaseSettingChanges();
 	m_model->setCoverScale(new_scale);
-	m_model->updateCacheSize(width(), height());
-	updateListFont();
 	updateToolbar();
 }
 
@@ -403,6 +490,7 @@ void GameListWidget::showGameList()
 	Host::SetBaseBoolSettingValue("UI", "GameListGridView", false);
 	Host::CommitBaseSettingChanges();
 	m_ui.stack->setCurrentIndex(0);
+	setFocusProxy(m_ui.stack->currentWidget());
 	resizeTableViewColumnsToFit();
 	updateToolbar();
 	emit layoutChange();
@@ -420,6 +508,7 @@ void GameListWidget::showGameGrid()
 	Host::SetBaseBoolSettingValue("UI", "GameListGridView", true);
 	Host::CommitBaseSettingChanges();
 	m_ui.stack->setCurrentIndex(1);
+	setFocusProxy(m_ui.stack->currentWidget());
 	updateToolbar();
 	emit layoutChange();
 }
@@ -438,13 +527,6 @@ void GameListWidget::setShowCoverTitles(bool enabled)
 	emit layoutChange();
 }
 
-void GameListWidget::updateListFont()
-{
-	QFont font;
-	font.setPointSizeF(16.0f * m_model->getCoverScale());
-	m_list_view->setFont(font);
-}
-
 void GameListWidget::updateToolbar()
 {
 	const bool grid_view = isShowingGameGrid();
@@ -476,6 +558,18 @@ void GameListWidget::resizeEvent(QResizeEvent* event)
 	m_model->updateCacheSize(width(), height());
 }
 
+bool GameListWidget::event(QEvent* event)
+{
+	if (event->type() == QEvent::DevicePixelRatioChange)
+	{
+		m_model->setDevicePixelRatio(devicePixelRatioF());
+		QWidget::event(event);
+		return true;
+	}
+
+	return QWidget::event(event);
+}
+
 void GameListWidget::resizeTableViewColumnsToFit()
 {
 	QtUtils::ResizeColumnsForTableView(m_table_view, {
@@ -488,7 +582,7 @@ void GameListWidget::resizeTableViewColumnsToFit()
 														 80, // last played
 														 80, // size
 														 60, // region
-														 100 // compatibility
+														 120 // compatibility
 													 });
 }
 
@@ -548,7 +642,10 @@ void GameListWidget::loadTableViewColumnSortSettings()
 			.value_or(DEFAULT_SORT_COLUMN);
 	const bool sort_descending =
 		Host::GetBaseBoolSettingValue("GameListTableView", "SortDescending", DEFAULT_SORT_DESCENDING);
-	m_table_view->sortByColumn(sort_column, sort_descending ? Qt::DescendingOrder : Qt::AscendingOrder);
+	const Qt::SortOrder sort_order = sort_descending ? Qt::DescendingOrder : Qt::AscendingOrder;
+	m_sort_model->sort(sort_column, sort_order);
+	if (QHeaderView* hv = m_table_view->horizontalHeader())
+		hv->setSortIndicator(sort_column, sort_order);
 }
 
 void GameListWidget::saveTableViewColumnSortSettings()
