@@ -1,3 +1,5 @@
+// SPDX-FileCopyrightText: 2002-2025 PCSX2 Dev Team
+// SPDX-License-Identifier: GPL-3.0+
 
 #pragma once
 
@@ -7,8 +9,8 @@
 
 #include "QtProgressCallback.h"
 
-#include "pcsx2/MemoryCardFile.h"
-#include "pcsx2/MemoryCardFolder.h"
+#include "pcsx2/SIO/Memcard/MemoryCardFile.h"
+#include "pcsx2/SIO/Memcard/MemoryCardFolder.h"
 
 class MemoryCardConvertWorker : public QtAsyncProgressThread
 {
