@@ -1,24 +1,12 @@
-/*  PCSX2 - PS2 Emulator for PCs
- *  Copyright (C) 2002-2020  PCSX2 Dev Team
- *
- *  PCSX2 is free software: you can redistribute it and/or modify it under the terms
- *  of the GNU Lesser General Public License as published by the Free Software Found-
- *  ation, either version 3 of the License, or (at your option) any later version.
- *
- *  PCSX2 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
- *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
- *  PURPOSE.  See the GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License along with PCSX2.
- *  If not, see <http://www.gnu.org/licenses/>.
- */
-
-#include "PrecompiledHeader.h"
+// SPDX-FileCopyrightText: 2002-2025 PCSX2 Dev Team
+// SPDX-License-Identifier: GPL-3.0+
 
 #include "common/Assertions.h"
 #include "common/Path.h"
 #include "common/StringUtil.h"
 
+#include "IopDma.h"
+
 #ifdef _WIN32
 #include "common/RedtapeWindows.h"
 #include <winioctl.h>
@@ -171,9 +159,7 @@ s32 DEV9init()
 	}
 #endif
 
-	int rxbi;
-
-	for (rxbi = 0; rxbi < (SMAP_BD_SIZE / 8); rxbi++)
+	for (int rxbi = 0; rxbi < (SMAP_BD_SIZE / 8); rxbi++)
 	{
 		smap_bd_t* pbd = (smap_bd_t*)&dev9.dev9R[SMAP_BD_RX_BASE & 0xffff];
 		pbd = &pbd[rxbi];
@@ -196,7 +182,6 @@ void DEV9shutdown()
 s32 DEV9open()
 {
 	DevCon.WriteLn("DEV9: DEV9open");
-	DevCon.WriteLn("DEV9: open r+: %s", EmuConfig.DEV9.HddFile.c_str());
 
 	std::string hddPath(GetHDDPath());
 
@@ -217,6 +202,7 @@ void DEV9close()
 {
 	DevCon.WriteLn("DEV9: DEV9close");
 
+	dev9.dma_iop_ptr = nullptr;
 	dev9.ata->Close();
 	TermNet();
 	isRunning = false;
@@ -243,49 +229,124 @@ void _DEV9irq(int cause, int cycles)
 		dev9Irq(cycles);
 }
 
-//Fakes SPEED FIFO
+// SPEED <-> HDD FIFO
 void HDDWriteFIFO()
 {
-	if (dev9.ata->dmaReady && (dev9.if_ctrl & SPD_IF_ATA_DMAEN))
-	{
-		const int unread = (dev9.fifo_bytes_write - dev9.fifo_bytes_read);
-		const int spaceSectors = (SPD_DBUF_AVAIL_MAX * 512 - unread) / 512;
-		if (spaceSectors < 0)
-		{
-			Console.Error("DEV9: No Space on SPEED FIFO");
-			pxAssert(false);
-			abort();
-		}
+	pxAssert(dev9.ata->dmaReady && (dev9.if_ctrl & SPD_IF_ATA_DMAEN));
+	pxAssert((dev9.if_ctrl & SPD_IF_READ));
 
-		const int readSectors = dev9.ata->nsectorLeft < spaceSectors ? dev9.ata->nsectorLeft : spaceSectors;
-		dev9.fifo_bytes_write += readSectors * 512;
-		dev9.ata->nsectorLeft -= readSectors;
+	const int unread = (dev9.fifo_bytes_write - dev9.fifo_bytes_read);
+	const int space = (SPD_DBUF_AVAIL_MAX * 512 - unread);
+	const int base = dev9.fifo_bytes_write % (SPD_DBUF_AVAIL_MAX * 512);
+
+	pxAssert(unread <= SPD_DBUF_AVAIL_MAX * 512);
+
+	int read;
+	if (base + space > SPD_DBUF_AVAIL_MAX * 512)
+	{
+		const int was = SPD_DBUF_AVAIL_MAX * 512 - base;
+		read = dev9.ata->ReadDMAToFIFO(dev9.fifo + base, was);
+		if (read == was)
+			read += dev9.ata->ReadDMAToFIFO(dev9.fifo, space - was);
 	}
-	//FIFOIntr();
+	else
+	{
+		read = dev9.ata->ReadDMAToFIFO(dev9.fifo + base, space);
+	}
+
+	dev9.fifo_bytes_write += read;
 }
 void HDDReadFIFO()
 {
-	if (dev9.ata->dmaReady && (dev9.if_ctrl & SPD_IF_ATA_DMAEN))
+	pxAssert(dev9.ata->dmaReady && (dev9.if_ctrl & SPD_IF_ATA_DMAEN));
+	pxAssert((dev9.if_ctrl & SPD_IF_READ) == 0);
+
+	const int unread = (dev9.fifo_bytes_write - dev9.fifo_bytes_read);
+	const int base = dev9.fifo_bytes_read % (SPD_DBUF_AVAIL_MAX * 512);
+
+	pxAssert(unread <= SPD_DBUF_AVAIL_MAX * 512);
+
+	int write;
+	if (base + unread > SPD_DBUF_AVAIL_MAX * 512)
 	{
-		const int writeSectors = (dev9.fifo_bytes_write - dev9.fifo_bytes_read) / 512;
-		dev9.fifo_bytes_read += writeSectors * 512;
-		dev9.ata->nsectorLeft -= writeSectors;
+		const int was = SPD_DBUF_AVAIL_MAX * 512 - base;
+		write = dev9.ata->WriteDMAFromFIFO(dev9.fifo + base, was);
+		if (write == was)
+			write += dev9.ata->WriteDMAFromFIFO(dev9.fifo, unread - was);
 	}
-	//FIFOIntr();
+	else
+	{
+		write = dev9.ata->WriteDMAFromFIFO(dev9.fifo + base, unread);
+	}
+
+	dev9.fifo_bytes_read += write;
 }
-void IOPReadFIFO(int bytes)
+void IOPReadFIFO()
 {
-	dev9.fifo_bytes_read += bytes;
+	pxAssert((dev9.dma_iop_ptr != nullptr) && (dev9.xfr_ctrl & SPD_XFR_DMAEN));
+	pxAssert((dev9.xfr_ctrl & SPD_XFR_WRITE) == 0);
+
+	const int unread = (dev9.fifo_bytes_write - dev9.fifo_bytes_read);
+	const int base = dev9.fifo_bytes_read % (SPD_DBUF_AVAIL_MAX * 512);
+
+	const int remain = dev9.dma_iop_size - dev9.dma_iop_transfered;
+	const int read = std::min(remain, unread);
+
+	pxAssert(unread <= SPD_DBUF_AVAIL_MAX * 512);
+
+	if (read == 0)
+		return;
+
+	if (base + read > SPD_DBUF_AVAIL_MAX * 512)
+	{
+		const int was = SPD_DBUF_AVAIL_MAX * 512 - base;
+
+		std::memcpy(dev9.dma_iop_ptr + dev9.dma_iop_transfered, dev9.fifo + base, was);
+		std::memcpy(dev9.dma_iop_ptr + dev9.dma_iop_transfered + was, dev9.fifo, read - was);
+	}
+	else
+	{
+		std::memcpy(dev9.dma_iop_ptr + dev9.dma_iop_transfered, dev9.fifo + base, read);
+	}
+
+	dev9.dma_iop_transfered += read;
+	dev9.fifo_bytes_read += read;
 	if (dev9.fifo_bytes_read > dev9.fifo_bytes_write)
 		Console.Error("DEV9: UNDERFLOW BY IOP");
-	//FIFOIntr();
 }
-void IOPWriteFIFO(int bytes)
+void IOPWriteFIFO()
 {
-	dev9.fifo_bytes_write += bytes;
-	if (dev9.fifo_bytes_write - SPD_DBUF_AVAIL_MAX * 512 > dev9.fifo_bytes_read)
+	pxAssert((dev9.dma_iop_ptr != nullptr) && (dev9.xfr_ctrl & SPD_XFR_DMAEN));
+	pxAssert(dev9.xfr_ctrl & SPD_XFR_WRITE);
+
+	const int unread = (dev9.fifo_bytes_write - dev9.fifo_bytes_read);
+	const int space = (SPD_DBUF_AVAIL_MAX * 512 - unread);
+	const int base = dev9.fifo_bytes_write % (SPD_DBUF_AVAIL_MAX * 512);
+
+	const int remain = dev9.dma_iop_size - dev9.dma_iop_transfered;
+	const int write = std::min(remain, space);
+
+	pxAssert(unread <= SPD_DBUF_AVAIL_MAX * 512);
+
+	if (write == 0)
+		return;
+
+	if (base + write > SPD_DBUF_AVAIL_MAX * 512)
+	{
+		const int was = SPD_DBUF_AVAIL_MAX * 512 - base;
+
+		std::memcpy(dev9.fifo + base, dev9.dma_iop_ptr + dev9.dma_iop_transfered, was);
+		std::memcpy(dev9.fifo + base, dev9.dma_iop_ptr + dev9.dma_iop_transfered + was, write - was);
+	}
+	else
+	{
+		std::memcpy(dev9.fifo + base, dev9.dma_iop_ptr + dev9.dma_iop_transfered, write);
+	}
+
+	dev9.dma_iop_transfered += write;
+	dev9.fifo_bytes_write += write;
+	if ((dev9.fifo_bytes_write - dev9.fifo_bytes_read) > SPD_DBUF_AVAIL_MAX * 512)
 		Console.Error("DEV9: OVERFLOW BY IOP");
-	//FIFOIntr();
 }
 void FIFOIntr()
 {
@@ -294,110 +355,99 @@ void FIFOIntr()
 
 	if (unread == 0)
 	{
+		dev9.irqcause &= ~SPD_INTR_ATA_FIFO_DATA;
 		if ((dev9.irqcause & SPD_INTR_ATA_FIFO_EMPTY) == 0)
 			_DEV9irq(SPD_INTR_ATA_FIFO_EMPTY, 1);
 	}
+	else
+	{
+		dev9.irqcause &= ~SPD_INTR_ATA_FIFO_EMPTY;
+		if ((dev9.irqcause & SPD_INTR_ATA_FIFO_DATA) == 0)
+			_DEV9irq(SPD_INTR_ATA_FIFO_DATA, 1);
+	}
+
 	if (unread == SPD_DBUF_AVAIL_MAX * 512)
 	{
-		//Log_Error("FIFO Full");
-		//INTR Full?
+		if ((dev9.irqcause & SPD_INTR_ATA_FIFO_FULL) == 0)
+			_DEV9irq(SPD_INTR_ATA_FIFO_FULL, 1);
+	}
+	else
+	{
+		dev9.irqcause &= ~SPD_INTR_ATA_FIFO_FULL;
+	}
+
+	// is DMA finished
+	if ((dev9.dma_iop_ptr != nullptr) &&
+		(dev9.dma_iop_transfered == dev9.dma_iop_size))
+	{
+		dev9.dma_iop_ptr = nullptr;
+		psxDMA8Interrupt();
 	}
 }
 
-u8 DEV9read8(u32 addr)
+// FIFO counters operate based on the direction set in SPD_R_XFR_CTRL
+// Both might have to set to the same direction for (SPEED <-> HDD) to work
+void DEV9runFIFO()
 {
-	if (!EmuConfig.DEV9.EthEnable && !EmuConfig.DEV9.HddEnable)
-		return 0;
+	const bool iopWrite = dev9.xfr_ctrl & SPD_XFR_WRITE; // IOP writes to FIFO
+	const bool hddRead = dev9.if_ctrl & SPD_IF_READ; // HDD writes to FIFO
 
-	u8 hard;
-	if (addr >= ATA_DEV9_HDD_BASE && addr < ATA_DEV9_HDD_END)
-	{
-		Console.Error("DEV9: ATA does not support 8bit reads %lx", addr);
-		return 0;
-	}
-	if (addr >= SMAP_REGBASE && addr < FLASH_REGBASE)
-	{
-		//smap
-		return smap_read8(addr);
-	}
-	if ((addr >= FLASH_REGBASE) && (addr < (FLASH_REGBASE + FLASH_REGSIZE)))
-	{
-		return (u8)FLASHread32(addr, 1);
-	}
+	const bool iopXfer = (dev9.dma_iop_ptr != nullptr) && (dev9.xfr_ctrl & SPD_XFR_DMAEN);
+	const bool hddXfer = dev9.ata->dmaReady && (dev9.if_ctrl & SPD_IF_ATA_DMAEN);
 
-	switch (addr)
+	// Order operations based on iopWrite to ensure DMA has data/space to work with.
+	if (iopWrite)
 	{
-		case SPD_R_PIO_DATA:
+		// Perform DMA from IOP.
+		if (iopXfer)
+			IOPWriteFIFO();
 
-			/*if(dev9.eeprom_dir!=1)
+		// Drain the FIFO
+		if (hddXfer && !hddRead)
+		{
+			HDDReadFIFO();
+		}
+	}
+	else
+	{
+		// Ensure FIFO has data.
+		if (hddXfer && hddRead)
+		{
+			HDDWriteFIFO();
+		}
+
+		if (iopXfer)
+		{
+			// Perform DMA to IOP.
+			IOPReadFIFO();
+
+			// Refill FIFO after DMA.
+			// Need to recheck dmaReady incase prior
+			// HDDWriteFIFO competed the transfer from HDD
+			if (hddXfer && hddRead && dev9.ata->dmaReady)
 			{
-				hard=0;
-				break;
-			}*/
-
-			if (dev9.eeprom_state == EEPROM_TDATA)
-			{
-				if (dev9.eeprom_command == 2) //read
-				{
-					if (dev9.eeprom_bit == 0xFF)
-						hard = 0;
-					else
-						hard = ((dev9.eeprom[dev9.eeprom_address] << dev9.eeprom_bit) & 0x8000) >> 11;
-					dev9.eeprom_bit++;
-					if (dev9.eeprom_bit == 16)
-					{
-						dev9.eeprom_address++;
-						dev9.eeprom_bit = 0;
-					}
-				}
-				else
-					hard = 0;
+				HDDWriteFIFO();
 			}
-			else
-				hard = 0;
-			//DevCon.WriteLn("DEV9: SPD_R_PIO_DATA 8bit read %x", hard);
-			return hard;
-
-		case DEV9_R_REV:
-			hard = 0x32; // expansion bay
-			//DevCon.WriteLn("DEV9: DEV9_R_REV 8bit read %x", hard);
-			return hard;
-
-		default:
-			hard = dev9Ru8(addr);
-			Console.Error("DEV9: Unknown 8bit read at address %lx value %x", addr, hard);
-			return hard;
+		}
 	}
+
+	FIFOIntr();
 }
 
-u16 DEV9read16(u32 addr)
+u16 SpeedRead(u32 addr, int width)
 {
-	if (!EmuConfig.DEV9.EthEnable && !EmuConfig.DEV9.HddEnable)
-		return 0;
-
-	u16 hard;
-	if (addr >= ATA_DEV9_HDD_BASE && addr < ATA_DEV9_HDD_END)
-	{
-		return dev9.ata->Read16(addr);
-	}
-	if (addr >= SMAP_REGBASE && addr < FLASH_REGBASE)
-	{
-		//smap
-		return smap_read16(addr);
-	}
-	if ((addr >= FLASH_REGBASE) && (addr < (FLASH_REGBASE + FLASH_REGSIZE)))
-	{
-		return (u16)FLASHread32(addr, 2);
-	}
-
+	u16 hard = 0;
 	switch (addr)
 	{
+		case 0x10000020:
+			//DevCon.WriteLn("DEV9: SPD_R_20 %dbit read %x", width, 1);
+			return 1;
 		case SPD_R_INTR_STAT:
-			//DevCon.WriteLn("DEV9: SPD_R_INTR_STAT 16bit read %x", dev9.irqcause);
+			//DevCon.WriteLn("DEV9: SPD_R_INTR_STAT %dbit read %x", width, dev9.irqcause);
 			return dev9.irqcause;
 
 		case SPD_R_INTR_MASK:
-			//DevCon.WriteLn("DEV9: SPD_R_INTR_MASK 16bit read %x", dev9.irqmask);
+			//DevCon.WriteLn("DEV9: SPD_R_INTR_MASK %dbit read %x", width, dev9.irqmask);
 			return dev9.irqmask;
 
 		case SPD_R_PIO_DATA:
@@ -410,11 +460,9 @@ u16 DEV9read16(u32 addr)
 
 			if (dev9.eeprom_state == EEPROM_TDATA)
 			{
-				if (dev9.eeprom_command == 2) //read
+				if (dev9.eeprom_command == 2) // read
 				{
-					if (dev9.eeprom_bit == 0xFF)
-						hard = 0;
-					else
+					if (dev9.eeprom_bit != 0xFF)
 						hard = ((dev9.eeprom[dev9.eeprom_address] << dev9.eeprom_bit) & 0x8000) >> 11;
 					dev9.eeprom_bit++;
 					if (dev9.eeprom_bit == 16)
@@ -423,73 +471,53 @@ u16 DEV9read16(u32 addr)
 						dev9.eeprom_bit = 0;
 					}
 				}
-				else
-					hard = 0;
 			}
-			else
-				hard = 0;
-			//DevCon.WriteLn("DEV9: SPD_R_PIO_DATA 16bit read %x", hard);
-			return hard;
-
-		case DEV9_R_REV:
-			//hard = 0x0030; // expansion bay
-			//DevCon.WriteLn("DEV9: DEV9_R_REV 16bit read %x", dev9.irqmask);
-			hard = 0x0032;
+			//DevCon.WriteLn("DEV9: SPD_R_PIO_DATA %dbit read %x", width, hard);
 			return hard;
 
 		case SPD_R_REV_1:
-			//DevCon.WriteLn("DEV9: SPD_R_REV_1 16bit read %x", 0);
+			//DevCon.WriteLn("DEV9: SPD_R_REV_1 %dbit read %x", width, 0);
 			return 0;
 
 		case SPD_R_REV_2:
-			hard = 0x0011;
-			//DevCon.WriteLn("DEV9: STD_R_REV_2 16bit read %x", hard);
+			hard = 0x11;
+			//DevCon.WriteLn("DEV9: STD_R_REV_2 %dbit read %x", width, hard);
 			return hard;
 
 		case SPD_R_REV_3:
-			hard = 0;
-			if (EmuConfig.DEV9.HddEnable)
-				hard |= SPD_CAPS_ATA;
+			// The Expansion bay always says HDD and Ethernet are supported, we need to keep HDD enabled and we handle it elsewhere.
+			// Ethernet we will turn off as not sure on what that would do right now, but no known game cares if it's off.
 			if (EmuConfig.DEV9.EthEnable)
 				hard |= SPD_CAPS_SMAP;
-			hard |= SPD_CAPS_FLASH;
-			//DevCon.WriteLn("DEV9: SPD_R_REV_3 16bit read %x", hard);
+
+			// TODO: Do we need flash? my 50003 model doesn't report this, but it does report DVR capable aka (1<<4), was that intended?
+			hard |= SPD_CAPS_ATA | SPD_CAPS_FLASH;
+			//DevCon.WriteLn("DEV9: SPD_R_REV_3 %dbit read %x", width, hard);
 			return hard;
 
 		case SPD_R_0e:
-			hard = 0x0002; //Have HDD inserted
-			DevCon.WriteLn("DEV9: SPD_R_0e 16bit read %x", hard);
+			hard = 0x0002; // Have HDD module inserted
+			DevCon.WriteLn("DEV9: SPD_R_0e %dbit read %x", width, hard);
 			return hard;
 		case SPD_R_XFR_CTRL:
-			DevCon.WriteLn("DEV9: SPD_R_XFR_CTRL 16bit read %x", dev9.xfr_ctrl);
+			DevCon.WriteLn("DEV9: SPD_R_XFR_CTRL %dbit read %x", width, dev9.xfr_ctrl);
 			return dev9.xfr_ctrl;
 		case SPD_R_DBUF_STAT:
 		{
-			hard = 0;
-			if (dev9.if_ctrl & SPD_IF_READ) //Semi async
+			const u8 count = static_cast<u8>((dev9.fifo_bytes_write - dev9.fifo_bytes_read) / 512);
+			if (dev9.xfr_ctrl & SPD_XFR_WRITE)
 			{
-				HDDWriteFIFO(); //Yes this is not a typo
-			}
-			else
-			{
-				HDDReadFIFO();
-			}
-			FIFOIntr();
-
-			const u8 count = (u8)((dev9.fifo_bytes_write - dev9.fifo_bytes_read) / 512);
-			if (dev9.xfr_ctrl & SPD_XFR_WRITE) //or ifRead?
-			{
-				hard = (u8)(SPD_DBUF_AVAIL_MAX - count);
-				hard |= (count == 0) ? SPD_DBUF_STAT_1 : (u16)0;
-				hard |= (count > 0) ? SPD_DBUF_STAT_2 : (u16)0;
+				hard = static_cast<u8>(SPD_DBUF_AVAIL_MAX - count);
+				hard |= (count == 0) ? SPD_DBUF_STAT_1 : static_cast<u16>(0);
+				hard |= (count > 0) ? SPD_DBUF_STAT_2 : static_cast<u16>(0);
 			}
 			else
 			{
 				hard = count;
-				hard |= (count < SPD_DBUF_AVAIL_MAX) ? SPD_DBUF_STAT_1 : (u16)0;
-				hard |= (count == 0) ? SPD_DBUF_STAT_2 : (u16)0;
-				//If overflow (HDD->SPEED), set both SPD_DBUF_STAT_2 & SPD_DBUF_STAT_FULL
-				//and overflow INTR set
+				hard |= (count < SPD_DBUF_AVAIL_MAX) ? SPD_DBUF_STAT_1 : static_cast<u16>(0);
+				hard |= (count == 0) ? SPD_DBUF_STAT_2 : static_cast<u16>(0);
+				// If overflow (HDD->SPEED), set both SPD_DBUF_STAT_2 & SPD_DBUF_STAT_FULL
+				// and overflow INTR set
 			}
 
 			if (count == SPD_DBUF_AVAIL_MAX)
@@ -497,193 +525,42 @@ u16 DEV9read16(u32 addr)
 				hard |= SPD_DBUF_STAT_FULL;
 			}
 
-			//DevCon.WriteLn("DEV9: SPD_R_DBUF_STAT 16bit read %x", hard);
+			//DevCon.WriteLn("DEV9: SPD_R_DBUF_STAT %dbit read %x", width, hard);
 			return hard;
 		}
 		case SPD_R_IF_CTRL:
-			//DevCon.WriteLn("DEV9: SPD_R_IF_CTRL 16bit read %x", dev9.if_ctrl);
+			//DevCon.WriteLn("DEV9: SPD_R_IF_CTRL %dbit read %x", width,, dev9.if_ctrl);
 			return dev9.if_ctrl;
 		default:
 			hard = dev9Ru16(addr);
-			Console.Error("DEV9: Unknown 16bit read at address %lx value %x", addr, hard);
+			Console.Error("DEV9: Unknown %dbit read at address %lx value %x", width, addr, hard);
 			return hard;
 	}
 }
 
-u32 DEV9read32(u32 addr)
+void SpeedWrite(u32 addr, u16 value, int width)
 {
-	if (!EmuConfig.DEV9.EthEnable && !EmuConfig.DEV9.HddEnable)
-		return 0;
-
-	u32 hard;
-	if (addr >= ATA_DEV9_HDD_BASE && addr < ATA_DEV9_HDD_END)
-	{
-		Console.Error("DEV9: ATA does not support 32bit reads %lx", addr);
-		return 0;
-	}
-	if (addr >= SMAP_REGBASE && addr < FLASH_REGBASE)
-	{
-		//smap
-		return smap_read32(addr);
-	}
-	if ((addr >= FLASH_REGBASE) && (addr < (FLASH_REGBASE + FLASH_REGSIZE)))
-	{
-		return (u32)FLASHread32(addr, 4);
-	}
-
-	hard = dev9Ru32(addr);
-	Console.Error("DEV9: Unknown 32bit read at address %lx value %x", addr, hard);
-	return hard;
-}
-
-void DEV9write8(u32 addr, u8 value)
-{
-	if (!EmuConfig.DEV9.EthEnable && !EmuConfig.DEV9.HddEnable)
-		return;
-
-	if (addr >= ATA_DEV9_HDD_BASE && addr < ATA_DEV9_HDD_END)
-	{
-#ifdef ENABLE_ATA
-		ata_write<1>(addr, value);
-#endif
-		return;
-	}
-	if (addr >= SMAP_REGBASE && addr < FLASH_REGBASE)
-	{
-		//smap
-		smap_write8(addr, value);
-		return;
-	}
-	if ((addr >= FLASH_REGBASE) && (addr < (FLASH_REGBASE + FLASH_REGSIZE)))
-	{
-		FLASHwrite32(addr, (u32)value, 1);
-		return;
-	}
-
 	switch (addr)
 	{
 		case 0x10000020:
-			Console.Error("DEV9: SPD_R_INTR_CAUSE, WTFH ?");
-			dev9.irqcause = 0xff;
-			break;
+			// DevCon.WriteLn("DEV9: SPD_R_20 %dbit write %x", wisth, value);
+			return;
 		case SPD_R_INTR_STAT:
-			Console.Error("DEV9: SPD_R_INTR_STAT,  WTFH ?");
+			Console.Error("DEV9: SPD_R_INTR_STAT %dbit write, WTF? %x", width, value);
 			dev9.irqcause = value;
 			return;
-		case SPD_R_INTR_MASK:
-			Console.Error("DEV9: SPD_R_INTR_MASK8, WTFH ?");
-			break;
-
-		case SPD_R_PIO_DIR:
-			//DevCon.WriteLn("DEV9: SPD_R_PIO_DIR 8bit write %x", value);
-
-			if ((value & 0xc0) != 0xc0)
-				return;
-
-			if ((value & 0x30) == 0x20)
-			{
-				dev9.eeprom_state = 0;
-			}
-			dev9.eeprom_dir = (value >> 4) & 3;
-
-			return;
-
-		case SPD_R_PIO_DATA:
-			//DevCon.WriteLn("DEV9: SPD_R_PIO_DATA 8bit write %x", value);
-
-			if ((value & 0xc0) != 0xc0)
-				return;
-
-			switch (dev9.eeprom_state)
-			{
-				case EEPROM_READY:
-					dev9.eeprom_command = 0;
-					dev9.eeprom_state++;
-					break;
-				case EEPROM_OPCD0:
-					dev9.eeprom_command = (value >> 4) & 2;
-					dev9.eeprom_state++;
-					dev9.eeprom_bit = 0xFF;
-					break;
-				case EEPROM_OPCD1:
-					dev9.eeprom_command |= (value >> 5) & 1;
-					dev9.eeprom_state++;
-					break;
-				case EEPROM_ADDR0:
-				case EEPROM_ADDR1:
-				case EEPROM_ADDR2:
-				case EEPROM_ADDR3:
-				case EEPROM_ADDR4:
-				case EEPROM_ADDR5:
-					dev9.eeprom_address =
-						(dev9.eeprom_address & (63 ^ (1 << (dev9.eeprom_state - EEPROM_ADDR0)))) |
-						((value >> (dev9.eeprom_state - EEPROM_ADDR0)) & (0x20 >> (dev9.eeprom_state - EEPROM_ADDR0)));
-					dev9.eeprom_state++;
-					break;
-				case EEPROM_TDATA:
-				{
-					if (dev9.eeprom_command == 1) //write
-					{
-						dev9.eeprom[dev9.eeprom_address] =
-							(dev9.eeprom[dev9.eeprom_address] & (63 ^ (1 << dev9.eeprom_bit))) |
-							((value >> dev9.eeprom_bit) & (0x8000 >> dev9.eeprom_bit));
-						dev9.eeprom_bit++;
-						if (dev9.eeprom_bit == 16)
-						{
-							dev9.eeprom_address++;
-							dev9.eeprom_bit = 0;
-						}
-					}
-				}
-				break;
-				default:
-					Console.Error("DEV9: Unknown EEPROM COMMAND");
-					break;
-			}
-			return;
-		default:
-			dev9Ru8(addr) = value;
-			Console.Error("DEV9: Unknown 8bit write at address %lx value %x", addr, value);
-			return;
-	}
-}
-
-void DEV9write16(u32 addr, u16 value)
-{
-	if (!EmuConfig.DEV9.EthEnable && !EmuConfig.DEV9.HddEnable)
-		return;
-
-	if (addr >= ATA_DEV9_HDD_BASE && addr < ATA_DEV9_HDD_END)
-	{
-		dev9.ata->Write16(addr, value);
-		return;
-	}
-	if (addr >= SMAP_REGBASE && addr < FLASH_REGBASE)
-	{
-		//smap
-		smap_write16(addr, value);
-		return;
-	}
-	if ((addr >= FLASH_REGBASE) && (addr < (FLASH_REGBASE + FLASH_REGSIZE)))
-	{
-		FLASHwrite32(addr, (u32)value, 2);
-		return;
-	}
-
-	switch (addr)
-	{
-		case SPD_R_INTR_MASK:
-			//DevCon.WriteLn("DEV9: SPD_R_INTR_MASK 16bit write %x	, checking for masked/unmasked interrupts", value);
+		case SPD_R_INTR_MASK: // 8bit writes affect whole reg
+			//DevCon.WriteLn("DEV9: SPD_R_INTR_MASK %dbit write %x", checking for masked/unmasked interrupts", width, value);
 			if ((dev9.irqmask != value) && ((dev9.irqmask | value) & dev9.irqcause))
 			{
-				//DevCon.WriteLn("DEV9: SPD_R_INTR_MASK16 firing unmasked interrupts");
+				//DevCon.WriteLn("DEV9: SPD_R_INTR_MASK firing unmasked interrupts");
 				dev9Irq(1);
 			}
 			dev9.irqmask = value;
-			break;
+			return;
 
 		case SPD_R_PIO_DIR:
-			//DevCon.WriteLn("DEV9: SPD_R_PIO_DIR 16bit write %x", value);
+			DevCon.WriteLn("DEV9: SPD_R_PIO_DIR %dbit write %x", width, value);
 
 			if ((value & 0xc0) != 0xc0)
 				return;
@@ -693,11 +570,10 @@ void DEV9write16(u32 addr, u16 value)
 				dev9.eeprom_state = 0;
 			}
 			dev9.eeprom_dir = (value >> 4) & 3;
-
 			return;
 
 		case SPD_R_PIO_DATA:
-			//DevCon.WriteLn("DEV9: SPD_R_PIO_DATA 16bit write %x", value);
+			//DevCon.WriteLn("DEV9: SPD_R_PIO_DATA %dbit write %x", width, value);
 
 			if ((value & 0xc0) != 0xc0)
 				return;
@@ -730,7 +606,7 @@ void DEV9write16(u32 addr, u16 value)
 					break;
 				case EEPROM_TDATA:
 				{
-					if (dev9.eeprom_command == 1) //write
+					if (dev9.eeprom_command == 1) // write
 					{
 						dev9.eeprom[dev9.eeprom_address] =
 							(dev9.eeprom[dev9.eeprom_address] & (63 ^ (1 << dev9.eeprom_bit))) |
@@ -742,8 +618,8 @@ void DEV9write16(u32 addr, u16 value)
 							dev9.eeprom_bit = 0;
 						}
 					}
+					break;
 				}
-				break;
 				default:
 					Console.Error("DEV9: Unknown EEPROM COMMAND");
 					break;
@@ -751,7 +627,7 @@ void DEV9write16(u32 addr, u16 value)
 			return;
 
 		case SPD_R_DMA_CTRL:
-			//DevCon.WriteLn("DEV9: SPD_R_IF_CTRL 16bit write %x", value);
+			//DevCon.WriteLn("DEV9: SPD_R_IF_CTRL %dbit write %x", width, value);
 			dev9.dma_ctrl = value;
 
 			//if (value & SPD_DMA_TO_SMAP)
@@ -777,9 +653,14 @@ void DEV9write16(u32 addr, u16 value)
 
 			break;
 		case SPD_R_XFR_CTRL:
-			//DevCon.WriteLn("DEV9: SPD_R_XFR_CTRL 16bit write %x", value);
+		{
+			//DevCon.WriteLn("DEV9: SPD_R_XFR_CTRL %dbit write %x", width, value);
+			const u16 oldValue = dev9.xfr_ctrl;
 			dev9.xfr_ctrl = value;
 
+			if ((value & SPD_XFR_WRITE) != (oldValue & SPD_XFR_WRITE))
+				DEV9runFIFO();
+
 			//if (value & SPD_XFR_WRITE)
 			//	DevCon.WriteLn("DEV9: SPD_R_XFR_CTRL Set Write");
 			//else
@@ -791,8 +672,11 @@ void DEV9write16(u32 addr, u16 value)
 			//if ((value & (1 << 2)) != 0)
 			//	DevCon.WriteLn("DEV9: SPD_R_XFR_CTRL Unknown Bit 2");
 
-			//if (value & SPD_XFR_DMAEN)
-			//	DevCon.WriteLn("DEV9: SPD_R_XFR_CTRL For DMA Enabled");
+			if (value & SPD_XFR_DMAEN)
+			{
+				//DevCon.WriteLn("DEV9: SPD_R_XFR_CTRL For DMA Enabled");
+				DEV9runFIFO();
+			}
 			//else
 			//	DevCon.WriteLn("DEV9: SPD_R_XFR_CTRL For DMA Disabled");
 
@@ -800,32 +684,42 @@ void DEV9write16(u32 addr, u16 value)
 				Console.Error("DEV9: SPD_R_XFR_CTRL Unknown value written %x", value);
 
 			break;
+		}
 		case SPD_R_DBUF_STAT:
-			//DevCon.WriteLn("DEV9: SPD_R_DBUF_STAT 16bit write %x", value);
+			//DevCon.WriteLn("DEV9: SPD_R_DBUF_STAT %dbit write %x", width, value);
 
-			if ((value & SPD_DBUF_RESET_FIFO) != 0)
+			if ((value & SPD_DBUF_RESET_READ_CNT) != 0)
 			{
-				//DevCon.WriteLn("DEV9: SPD_R_DBUF_STAT Reset FIFO");
-				dev9.fifo_bytes_write = 0;
+				//DevCon.WriteLn("DEV9: SPD_R_DBUF_STAT Reset read counter");
 				dev9.fifo_bytes_read = 0;
-				dev9.xfr_ctrl &= ~SPD_XFR_WRITE; //?
-				dev9.if_ctrl |= SPD_IF_READ; //?
-
-				FIFOIntr();
 			}
+			if ((value & SPD_DBUF_RESET_WRITE_CNT) != 0)
+			{
+				//DevCon.WriteLn("DEV9: SPD_R_DBUF_STAT Reset write counter");
+				dev9.fifo_bytes_write = 0;
+			}
+
+			if (value != 0)
+				FIFOIntr();
 
 			if (value != 3)
 				Console.Error("DEV9: SPD_R_DBUF_STAT 16bit write %x Which != 3!!!", value);
 			break;
 
 		case SPD_R_IF_CTRL:
-			//DevCon.WriteLn("DEV9: SPD_R_IF_CTRL 16bit write %x", value);
+		{
+			//DevCon.WriteLn("DEV9: SPD_R_IF_CTRL %dbit write %x", width, value);
+			const u16 oldValue = dev9.if_ctrl;
 			dev9.if_ctrl = value;
 
 			//if (value & SPD_IF_UDMA)
 			//	DevCon.WriteLn("DEV9: IF_CTRL UDMA Enabled");
 			//else
 			//	DevCon.WriteLn("DEV9: IF_CTRL UDMA Disabled");
+
+			if ((value & SPD_IF_READ) != (oldValue & SPD_IF_READ))
+				DEV9runFIFO();
+
 			//if (value & SPD_IF_READ)
 			//	DevCon.WriteLn("DEV9: IF_CTRL DMA Is ATA Read");
 			//else
@@ -834,21 +728,22 @@ void DEV9write16(u32 addr, u16 value)
 			if (value & SPD_IF_ATA_DMAEN)
 			{
 				//DevCon.WriteLn("DEV9: IF_CTRL ATA DMA Enabled");
-				if (value & SPD_IF_READ) //Semi async
-				{
-					HDDWriteFIFO(); //Yes this is not a typo
-				}
-				else
-				{
-					HDDReadFIFO();
-				}
-				FIFOIntr();
+				DEV9runFIFO();
 			}
 			//else
 			//	DevCon.WriteLn("DEV9: IF_CTRL ATA DMA Disabled");
 
-			if (value & (1 << 3))
-				DevCon.WriteLn("DEV9: IF_CTRL Unknown Bit 3 Set");
+			/* During a HDD DMA transfer, the ATA regs are inacessable.
+			 * The SPEED will cache register writes and wait until the end of the DMA block to write them.
+			 * Bit 3 controls what happens when a read is performed while the ATA regs are inacessable.
+			 * When set, the SPEED will asserts /WAIT to the IOP and wait for the HDD to end the DMA block, before reading the reg.
+			 * When cleared, the read will fail if mid DMA. bit 1 in reg 0x62 indicates if the last read failed.
+			 * Our DMA transfers are instant, so we can ignore this bit.
+			 */
+			//if (value & (1 << 3))
+			//	DevCon.WriteLn("DEV9: IF_CTRL Wait for ATA register read Enabled");
+			//else
+			//	DevCon.WriteLn("DEV9: IF_CTRL Wait for ATA register read Disabled");
 
 			if (value & (1 << 4))
 				Console.Error("DEV9: IF_CTRL Unknown Bit 4 Set");
@@ -876,8 +771,9 @@ void DEV9write16(u32 addr, u16 value)
 				Console.Error("DEV9: IF_CTRL Unknown Bit(s) %x", (value & 0xFF00));
 
 			break;
+		}
 		case SPD_R_PIO_MODE: //ATA only? or includes EEPROM?
-			//DevCon.WriteLn("DEV9: SPD_R_PIO_MODE 16bit write %x", value);
+			//DevCon.WriteLn("DEV9: SPD_R_PIO_MODE 16bit %dbit write %x", width, value);
 			dev9.pio_mode = value;
 
 			switch (value)
@@ -903,8 +799,8 @@ void DEV9write16(u32 addr, u16 value)
 					break;
 			}
 			break;
-		case SPD_R_MDMA_MODE: //ATA only? or includes EEPROM?
-			DevCon.WriteLn("DEV9: SPD_R_MDMA_MODE 16bit write %x", value);
+		case SPD_R_MDMA_MODE:
+			DevCon.WriteLn("DEV9: SPD_R_MDMA_MODE 16bit write %dbit write %x", width, value);
 			dev9.mdma_mode = value;
 
 			switch (value)
@@ -924,8 +820,8 @@ void DEV9write16(u32 addr, u16 value)
 			}
 
 			break;
-		case SPD_R_UDMA_MODE: //ATA only?
-			DevCon.WriteLn("DEV9: SPD_R_UDMA_MODE 16bit write %x", value);
+		case SPD_R_UDMA_MODE:
+			DevCon.WriteLn("DEV9: SPD_R_UDMA_MODE 16bit write %dbit write %x", width, value);
 			dev9.udma_mode = value;
 
 			switch (value)
@@ -952,12 +848,185 @@ void DEV9write16(u32 addr, u16 value)
 			break;
 
 		default:
-			dev9Ru16(addr) = value;
-			Console.Error("DEV9: *Unknown 16bit write at address %lx value %x", addr, value);
+			dev9Ru8(addr) = value;
+			Console.Error("DEV9: Unknown %dbit write at address %lx value %x", width, addr, value);
 			return;
 	}
 }
 
+u8 DEV9read8(u32 addr)
+{
+	if (!EmuConfig.DEV9.EthEnable && !EmuConfig.DEV9.HddEnable)
+		return 0;
+
+	if (addr >= ATA_DEV9_HDD_BASE && addr < ATA_DEV9_HDD_END)
+	{
+		return dev9.ata->Read(addr, 8);
+	}
+	// Note, ATA regs within range of addresses used by Speed
+	if (addr >= SPD_REGBASE && addr < SMAP_REGBASE)
+	{
+		// speed
+		return SpeedRead(addr, 8);
+	}
+	if (addr >= SMAP_REGBASE && addr < FLASH_REGBASE)
+	{
+		// smap
+		return smap_read8(addr);
+	}
+	if ((addr >= FLASH_REGBASE) && (addr < (FLASH_REGBASE + FLASH_REGSIZE)))
+	{
+		return static_cast<u8>(FLASHread32(addr, 1));
+	}
+
+	u8 hard = 0;
+	switch (addr)
+	{
+		case DEV9_R_REV:
+			hard = 0x32; // expansion bay
+			//DevCon.WriteLn("DEV9: DEV9_R_REV 8bit read %x", hard);
+			return hard;
+
+		default:
+			hard = dev9Ru8(addr);
+			Console.Error("DEV9: Unknown 8bit read at address %lx value %x", addr, hard);
+			return hard;
+	}
+}
+
+u16 DEV9read16(u32 addr)
+{
+	if (!EmuConfig.DEV9.EthEnable && !EmuConfig.DEV9.HddEnable)
+		return 0;
+
+	if (addr >= ATA_DEV9_HDD_BASE && addr < ATA_DEV9_HDD_END)
+	{
+		return dev9.ata->Read(addr, 16);
+	}
+	// Note, ATA regs within range of addresses used by Speed
+	if (addr >= SPD_REGBASE && addr < SMAP_REGBASE)
+	{
+		// speed
+		return SpeedRead(addr, 16);
+	}
+	if (addr >= SMAP_REGBASE && addr < FLASH_REGBASE)
+	{
+		// smap
+		return smap_read16(addr);
+	}
+	if ((addr >= FLASH_REGBASE) && (addr < (FLASH_REGBASE + FLASH_REGSIZE)))
+	{
+		return static_cast<u16>(FLASHread32(addr, 2));
+	}
+
+	u16 hard = 0;
+	switch (addr)
+	{
+		case DEV9_R_REV:
+			//hard = 0x0030; // expansion bay
+			//DevCon.WriteLn("DEV9: DEV9_R_REV 16bit read %x", dev9.irqmask);
+			hard = 0x0032;
+			return hard;
+
+		default:
+			hard = dev9Ru16(addr);
+			Console.Error("DEV9: Unknown 16bit read at address %lx value %x", addr, hard);
+			return hard;
+	}
+}
+
+u32 DEV9read32(u32 addr)
+{
+	if (!EmuConfig.DEV9.EthEnable && !EmuConfig.DEV9.HddEnable)
+		return 0;
+
+	if (addr >= ATA_DEV9_HDD_BASE && addr < ATA_DEV9_HDD_END)
+	{
+		Console.Error("DEV9: ATA does not support 32bit reads %lx", addr);
+		return 0;
+	}
+	if (addr >= SMAP_REGBASE && addr < FLASH_REGBASE)
+	{
+		//smap
+		return smap_read32(addr);
+	}
+	if ((addr >= FLASH_REGBASE) && (addr < (FLASH_REGBASE + FLASH_REGSIZE)))
+	{
+		return static_cast<u32>(FLASHread32(addr, 4));
+	}
+
+	const u32 hard = dev9Ru32(addr);
+	Console.Error("DEV9: Unknown 32bit read at address %lx value %x", addr, hard);
+	return hard;
+}
+
+void DEV9write8(u32 addr, u8 value)
+{
+	if (!EmuConfig.DEV9.EthEnable && !EmuConfig.DEV9.HddEnable)
+		return;
+
+	if (addr >= ATA_DEV9_HDD_BASE && addr < ATA_DEV9_HDD_END)
+	{
+		dev9.ata->Write(addr, value, 8);
+		return;
+	}
+	// Note, ATA regs within range of addresses used by Speed
+	if (addr >= SPD_REGBASE && addr < SMAP_REGBASE)
+	{
+		// speed
+		SpeedWrite(addr, value, 8);
+		return;
+	}
+	if (addr >= SMAP_REGBASE && addr < FLASH_REGBASE)
+	{
+		// smap
+		smap_write8(addr, value);
+		return;
+	}
+	if ((addr >= FLASH_REGBASE) && (addr < (FLASH_REGBASE + FLASH_REGSIZE)))
+	{
+		FLASHwrite32(addr, static_cast<u32>(value), 1);
+		return;
+	}
+
+	Console.Error("DEV9: Unknown 8bit write at address %lx value %x", addr, value);
+	return;
+}
+
+void DEV9write16(u32 addr, u16 value)
+{
+	if (!EmuConfig.DEV9.EthEnable && !EmuConfig.DEV9.HddEnable)
+		return;
+
+	if (addr >= ATA_DEV9_HDD_BASE && addr < ATA_DEV9_HDD_END)
+	{
+		dev9.ata->Write(addr, value, 16);
+		return;
+	}
+	// Note, ATA regs within range of addresses used by Speed
+	if (addr >= SPD_REGBASE && addr < SMAP_REGBASE)
+	{
+		// speed
+		SpeedWrite(addr, value, 16);
+		return;
+	}
+	if (addr >= SMAP_REGBASE && addr < FLASH_REGBASE)
+	{
+		// smap
+		smap_write16(addr, value);
+		return;
+	}
+	if ((addr >= FLASH_REGBASE) && (addr < (FLASH_REGBASE + FLASH_REGSIZE)))
+	{
+		FLASHwrite32(addr, static_cast<u32>(value), 2);
+		return;
+	}
+
+	dev9Ru16(addr) = value;
+	Console.Error("DEV9: *Unknown 16bit write at address %lx value %x", addr, value);
+	return;
+}
+
 void DEV9write32(u32 addr, u32 value)
 {
 	if (!EmuConfig.DEV9.EthEnable && !EmuConfig.DEV9.HddEnable)
@@ -978,7 +1047,7 @@ void DEV9write32(u32 addr, u32 value)
 	}
 	if ((addr >= FLASH_REGBASE) && (addr < (FLASH_REGBASE + FLASH_REGSIZE)))
 	{
-		FLASHwrite32(addr, (u32)value, 4);
+		FLASHwrite32(addr, static_cast<u32>(value), 4);
 		return;
 	}
 
@@ -1004,16 +1073,20 @@ void DEV9readDMA8Mem(u32* pMem, int size)
 	DevCon.WriteLn("DEV9: *DEV9readDMA8Mem: size %x", size);
 
 	if (dev9.dma_ctrl & SPD_DMA_TO_SMAP)
+	{
 		smap_readDMA8Mem(pMem, size);
+		psxDMA8Interrupt();
+	}
 	else
 	{
-		if (dev9.xfr_ctrl & SPD_XFR_DMAEN &&
-			!(dev9.xfr_ctrl & SPD_XFR_WRITE))
+		if (!(dev9.xfr_ctrl & SPD_XFR_WRITE))
 		{
-			HDDWriteFIFO();
-			IOPReadFIFO(size);
-			dev9.ata->ATAreadDMA8Mem((u8*)pMem, size);
-			FIFOIntr();
+			pxAssert(size <= SPD_DBUF_AVAIL_MAX * 512);
+			dev9.dma_iop_ptr = reinterpret_cast<u8*>(pMem);
+			dev9.dma_iop_size = size;
+			dev9.dma_iop_transfered = 0;
+
+			DEV9runFIFO();
 		}
 	}
 
@@ -1030,20 +1103,22 @@ void DEV9writeDMA8Mem(u32* pMem, int size)
 	DevCon.WriteLn("DEV9: *DEV9writeDMA8Mem: size %x", size);
 
 	if (dev9.dma_ctrl & SPD_DMA_TO_SMAP)
+	{
 		smap_writeDMA8Mem(pMem, size);
+		psxDMA8Interrupt();
+	}
 	else
 	{
-		if (dev9.xfr_ctrl & SPD_XFR_DMAEN &&
-			dev9.xfr_ctrl & SPD_XFR_WRITE)
+		if (dev9.xfr_ctrl & SPD_XFR_WRITE)
 		{
-			IOPWriteFIFO(size);
-			HDDReadFIFO();
-			dev9.ata->ATAwriteDMA8Mem((u8*)pMem, size);
-			FIFOIntr();
+			pxAssert(size <= SPD_DBUF_AVAIL_MAX * 512);
+			dev9.dma_iop_ptr = reinterpret_cast<u8*>(pMem);
+			dev9.dma_iop_size = size;
+			dev9.dma_iop_transfered = 0;
+
+			DEV9runFIFO();
 		}
 	}
-
-	//TODO, track if write was successful
 }
 
 void DEV9async(u32 cycles)
@@ -1071,8 +1146,7 @@ void DEV9CheckChanges(const Pcsx2Config& old_config)
 		{
 			//ATA::Open/Close dosn't set any regs
 			//So we can close/open to apply settings
-			if (EmuConfig.DEV9.HddFile != old_config.DEV9.HddFile ||
-				EmuConfig.DEV9.HddSizeSectors != old_config.DEV9.HddSizeSectors)
+			if (EmuConfig.DEV9.HddFile != old_config.DEV9.HddFile)
 			{
 				dev9.ata->Close();
 				if (dev9.ata->Open(hddPath) != 0)
