@@ -1,17 +1,5 @@
-/*  PCSX2 - PS2 Emulator for PCs
- *  Copyright (C) 2002-2021  PCSX2 Dev Team
- *
- *  PCSX2 is free software: you can redistribute it and/or modify it under the terms
- *  of the GNU Lesser General Public License as published by the Free Software Found-
- *  ation, either version 3 of the License, or (at your option) any later version.
- *
- *  PCSX2 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
- *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
- *  PURPOSE.  See the GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License along with PCSX2.
- *  If not, see <http://www.gnu.org/licenses/>.
- */
+// SPDX-FileCopyrightText: 2002-2025 PCSX2 Dev Team
+// SPDX-License-Identifier: GPL-3.0+
 
 #pragma once
 #include "DEV9/PacketReader/IP/IP_Packet.h"
@@ -39,16 +27,16 @@ namespace PacketReader::IP::ICMP
 	public:
 		//Takes ownership of payload
 		ICMP_Packet(Payload* data);
-		ICMP_Packet(u8* buffer, int bufferSize);
+		ICMP_Packet(const u8* buffer, int bufferSize);
 		ICMP_Packet(const ICMP_Packet&);
 
-		Payload* GetPayload();
+		Payload* GetPayload() const;
 
 		virtual int GetLength();
 		virtual void WriteBytes(u8* buffer, int* offset);
 		virtual ICMP_Packet* Clone() const;
 
-		virtual u8 GetProtocol();
+		virtual u8 GetProtocol() const;
 
 		virtual bool VerifyChecksum(IP_Address srcIP, IP_Address dstIP);
 		virtual void CalculateChecksum(IP_Address srcIP, IP_Address dstIP);
