@@ -1,25 +1,16 @@
-/*  PCSX2 - PS2 Emulator for PCs
- *  Copyright (C) 2002-2021  PCSX2 Dev Team
- *
- *  PCSX2 is free software: you can redistribute it and/or modify it under the terms
- *  of the GNU Lesser General Public License as published by the Free Software Found-
- *  ation, either version 3 of the License, or (at your option) any later version.
- *
- *  PCSX2 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
- *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
- *  PURPOSE.  See the GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License along with PCSX2.
- *  If not, see <http://www.gnu.org/licenses/>.
- */
+// SPDX-FileCopyrightText: 2002-2025 PCSX2 Dev Team
+// SPDX-License-Identifier: GPL-3.0+
 
 #pragma once
-#include <vector>
 
 #include "DNS_Classes.h"
 #include "DNS_Enums.h"
 #include "DEV9/PacketReader/Payload.h"
 
+#include "common/Pcsx2Defs.h"
+
+#include <vector>
+
 namespace PacketReader::IP::UDP::DNS
 {
 	class DNS_Packet : public Payload
@@ -42,38 +33,38 @@ namespace PacketReader::IP::UDP::DNS
 		std::vector<DNS_ResponseEntry> authorities;
 		std::vector<DNS_ResponseEntry> additional;
 
-		bool GetQR();
+		bool GetQR() const;
 		void SetQR(bool value);
 
-		u8 GetOpCode();
+		u8 GetOpCode() const;
 		void SetOpCode(u8 value);
 
-		bool GetAA();
+		bool GetAA() const;
 		void SetAA(bool value);
 
-		bool GetTC();
+		bool GetTC() const;
 		void SetTC(bool value);
 
-		bool GetRD();
+		bool GetRD() const;
 		void SetRD(bool value);
 
-		bool GetRA();
+		bool GetRA() const;
 		void SetRA(bool value);
 
-		u8 GetZ0();
+		u8 GetZ0() const;
 		void SetZ0(u8 value);
 
-		bool GetAD();
+		bool GetAD() const;
 		void SetAD(bool value);
 
-		bool GetCD();
+		bool GetCD() const;
 		void SetCD(bool value);
 
-		u8 GetRCode();
+		u8 GetRCode() const;
 		void SetRCode(u8 value);
 
 		DNS_Packet() {}
-		DNS_Packet(u8* buffer, int bufferSize);
+		DNS_Packet(const u8* buffer, int bufferSize);
 
 		virtual int GetLength();
 		virtual void WriteBytes(u8* buffer, int* offset);
