@@ -1,19 +1,5 @@
-/*  PCSX2 - PS2 Emulator for PCs
- *  Copyright (C) 2002-2021  PCSX2 Dev Team
- *
- *  PCSX2 is free software: you can redistribute it and/or modify it under the terms
- *  of the GNU Lesser General Public License as published by the Free Software Found-
- *  ation, either version 3 of the License, or (at your option) any later version.
- *
- *  PCSX2 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
- *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
- *  PURPOSE.  See the GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License along with PCSX2.
- *  If not, see <http://www.gnu.org/licenses/>.
- */
-
-#include "PrecompiledHeader.h"
+// SPDX-FileCopyrightText: 2002-2025 PCSX2 Dev Team
+// SPDX-License-Identifier: GPL-3.0+
 
 #ifdef _WIN32
 #include "common/RedtapeWindows.h"
@@ -29,8 +15,11 @@
 #endif
 
 #include <algorithm>
+#include <bit>
+#include <thread>
 #include <errno.h>
 #include <sys/types.h>
+#include <sys/ioctl.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <netinet/ip.h>
@@ -49,36 +38,38 @@ using namespace PacketReader::IP::ICMP;
 
 using namespace std::chrono_literals;
 
-/*	Ping is kindof annoying to do crossplatform
-	All platforms restrict raw sockets
-
-	Windows provides an api for ICMP
-	ICMP_ECHO_REPLY should always be used, ignore ICMP_ECHO_REPLY32
-	IP_OPTION_INFORMATION should always be used, ignore IP_OPTION_INFORMATION32
-
-	Linux
-	We have access to raw sockets via CAP_NET_RAW (for pcap)
-		However we may be missing that cap on some builds
-	Linux has socket(PF_INET, SOCK_DGRAM, IPPROTO_ICMP), used similar to raw sockets but for ICMP only
-		Auto filters responses
-		Requires net.ipv4.ping_group_range sysctl, default off on alot of distros
-	Timeouts reported via sock_extended_err control messages (with IP_RECVERR socket option set)
-
-	Mac
-	Raw sockets restricted
-	Mac has socket(PF_INET, SOCK_DGRAM, IPPROTO_ICMP)
-		No restriction to using it
-		Implementation differs, is more versatile than linux
-		Does not auto filter responses
-	Timeouts reported as a normal packet
-
-	FreeBSD
-	Raw sockets restricted
-	No unprivilaged ICMP sockets
-	Timeouts reported as a normal packet??
-
-	Ping cli
-	Present for all platforms, but command args differ
+/*
+ * Ping is kindof annoying to do crossplatform
+ * All platforms restrict raw sockets
+ * 
+ * Windows provides an api for ICMP
+ * ICMP_ECHO_REPLY should always be used, ignore ICMP_ECHO_REPLY32
+ * IP_OPTION_INFORMATION should always be used, ignore IP_OPTION_INFORMATION32
+ * 
+ * Linux
+ * We have access to raw sockets via CAP_NET_RAW (for pcap)
+ *     However we may be missing that cap on some builds
+ * Also hava socket(PF_INET, SOCK_DGRAM, IPPROTO_ICMP), used similarly to raw sockets, but for ICMP only
+ *     Auto filters responses
+ *     Requires net.ipv4.ping_group_range sysctl, default off on a lot of distros
+ * Timeouts reported via sock_extended_err control messages (with IP_RECVERR socket option set)
+ * 
+ * Mac
+ * Raw sockets restricted
+ * Mac has socket(PF_INET, SOCK_DGRAM, IPPROTO_ICMP)
+ *     No restriction to using it with ICMP_ECHO
+ *     Implementation differs, is more versatile than linux
+ *     Does not auto filter responses
+ * Timeouts reported as a normal packet
+ * 
+ * FreeBSD
+ * Raw sockets restricted
+ * No unprivilaged ICMP sockets
+ * Timeouts reported as a normal packet??
+ * 
+ * Ping cli
+ * Present for all platforms, but command args differ
+ * Not used here
  */
 
 namespace Sessions
@@ -100,8 +91,8 @@ namespace Sessions
 			return;
 		}
 
-		icmpEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
-		if (icmpEvent == NULL)
+		icmpEvent = CreateEvent(nullptr, FALSE, FALSE, nullptr);
+		if (icmpEvent == nullptr)
 		{
 			Console.Error("DEV9: ICMP: Failed to Create Event");
 			IcmpCloseHandle(icmpFile);
@@ -109,29 +100,36 @@ namespace Sessions
 			return;
 		}
 
-		//Allocate return buffer
-		//Documentation says + 8 to allow for an ICMP error message
-		//In testing, ICMP_ECHO_REPLY structure itself was returned with data set to null
+		/* 
+		 * Allocate response buffer
+		 * Documentation says + 8 to allow for an ICMP error message
+		 * In testing, ICMP_ECHO_REPLY structure itself was returned with data set to null
+		 */
 		icmpResponseBufferLen = sizeof(ICMP_ECHO_REPLY) + requestSize + 8;
-		icmpResponseBuffer = std::make_unique<u8[]>(icmpResponseBufferLen);
+		icmpResponseBuffer = std::make_unique<std::byte[]>(icmpResponseBufferLen);
 #elif defined(__POSIX__)
 	{
+		/* 
+		 * Allocate response buffer
+		 * Size needed depends on which socket protocol (ICMP or raw) we use aswell as os
+		 */
 		switch (icmpConnectionKind)
 		{
-			//Two different methods for raw/icmp sockets bettween the unix OSes
-			//Play it safe and only enable when we know which of the two methods we use
+			// Two different methods for raw/icmp sockets between the Unix OSes
+			// Play it safe and only enable when we know which of the two methods we use
 #if defined(ICMP_SOCKETS_LINUX) || defined(ICMP_SOCKETS_BSD)
 			case (PingType::ICMP):
 				icmpSocket = socket(AF_INET, SOCK_DGRAM, IPPROTO_ICMP);
 				if (icmpSocket != -1)
 				{
 #if defined(ICMP_SOCKETS_LINUX)
-					//Space for ICMP header, as MSG_ERRQUEUE returns what we sent
-					//An extra +8 required sometimes, for some reason?
+					// Only need space for ICMP header, as MSG_ERRQUEUE returns data we sent
+					// Testing found an extra + 8 was required sometimes, for some reason?
 					icmpResponseBufferLen = 8 + requestSize + 8;
 #elif defined(ICMP_SOCKETS_BSD)
-					//Returned IP Header, ICMP Header & either data or failed ICMP packet
-					icmpResponseBufferLen = 20 + 8 + std::max(20 + 8, requestSize);
+					// Returned IP Header, ICMP Header & either data or failed ICMP packet
+					// Sometimes get full packet in ICMP error response
+					icmpResponseBufferLen = 20 + 8 + (20 + 8 + requestSize);
 #endif
 					break;
 				}
@@ -146,16 +144,17 @@ namespace Sessions
 				if (icmpSocket != -1)
 				{
 #if defined(ICMP_SOCKETS_LINUX)
-					//We get packet + header
+					// We get IP packet + ICMP header
 					icmpResponseBufferLen = 20 + 8 + requestSize;
 #elif defined(ICMP_SOCKETS_BSD)
-					//As above, but we will also directly recive error ICMP messages
+					// As above, but we will also directly receive error ICMP messages
 					icmpResponseBufferLen = 20 + 8 + std::max(20 + 8, requestSize);
 #endif
 					break;
 				}
 
 				DevCon.WriteLn("DEV9: ICMP: Failed To Open RAW Socket");
+
 				[[fallthrough]];
 #endif
 			default:
@@ -163,11 +162,11 @@ namespace Sessions
 				return;
 		}
 
-		icmpResponseBuffer = std::make_unique<u8[]>(icmpResponseBufferLen);
+		icmpResponseBuffer = std::make_unique<std::byte[]>(icmpResponseBufferLen);
 #endif
 	}
 
-	bool ICMP_Session::Ping::IsInitialised()
+	bool ICMP_Session::Ping::IsInitialised() const
 	{
 #ifdef _WIN32
 		return icmpFile != INVALID_HANDLE_VALUE;
@@ -183,19 +182,29 @@ namespace Sessions
 #endif
 	}
 
-	//Note, we can finish reading but have no data
+	// Returned PingResult.data is only valid when PingResult.type is 0
 	ICMP_Session::PingResult* ICMP_Session::Ping::Recv()
 	{
 #ifdef _WIN32
 		if (WaitForSingleObject(icmpEvent, 0) == WAIT_OBJECT_0)
 		{
 			ResetEvent(icmpEvent);
-			//Prep buffer for reasing
-			int count = IcmpParseReplies(icmpResponseBuffer.get(), icmpResponseBufferLen);
-			pxAssert(count == 1);
-			ICMP_ECHO_REPLY* pingRet = (ICMP_ECHO_REPLY*)icmpResponseBuffer.get();
 
-			//Map status to ICMP type/code
+			const int count = IcmpParseReplies(icmpResponseBuffer.get(), icmpResponseBufferLen);
+			pxAssert(count <= 1);
+
+			// Timeout
+			if (count == 0)
+			{
+				result.type = -2;
+				result.code = 0;
+				return &result;
+			}
+
+			// Rely on implicit object creation
+			const ICMP_ECHO_REPLY* pingRet = reinterpret_cast<ICMP_ECHO_REPLY*>(icmpResponseBuffer.get());
+
+			// Map status to ICMP type/code
 			switch (pingRet->Status)
 			{
 				case (IP_SUCCESS):
@@ -222,21 +231,23 @@ namespace Sessions
 					result.type = 3;
 					result.code = 4;
 					break;
-				case (IP_BAD_ROUTE): //Bad source route
+				case (IP_BAD_ROUTE): // Bad source route
 					result.type = 3;
 					result.code = 5;
 					break;
 				case (IP_BAD_DESTINATION):
-					//I think this could be either
-					//Destination network unknown
-					//or
-					//Destination host unknown
-					//Use host unkown
+					/*
+					 * I think this could be mapped to either
+					 * Destination network unknown
+					 * or
+					 * Destination host unknown
+					 * Lets map to host unknown
+					 */
 					result.type = 3;
 					result.code = 7;
 					break;
 				case (IP_REQ_TIMED_OUT):
-					//Return nothing
+					// Return nothing
 					result.type = -2;
 					result.code = 0;
 					break;
@@ -253,7 +264,7 @@ namespace Sessions
 					result.code = 0;
 					break;
 
-					//Unexpected Errors
+					// Unexpected errors
 				case (IP_BUF_TOO_SMALL):
 				case (IP_NO_RESOURCES):
 				case (IP_BAD_OPTION):
@@ -269,7 +280,7 @@ namespace Sessions
 			}
 
 			result.dataLength = pingRet->DataSize;
-			result.data = (u8*)pingRet->Data;
+			result.data = static_cast<u8*>(pingRet->Data);
 			result.address.integer = pingRet->Address;
 
 			return &result;
@@ -282,82 +293,30 @@ namespace Sessions
 			case (PingType::ICMP):
 			case (PingType::RAW):
 			{
-				int ret;
-
-#if defined(ICMP_SOCKETS_BSD)
-				fd_set sReady;
-				fd_set sExcept;
-
-				timeval nowait{0};
-				FD_ZERO(&sReady);
-				FD_ZERO(&sExcept);
-				FD_SET(icmpSocket, &sReady);
-				FD_SET(icmpSocket, &sExcept);
-				ret = select(icmpSocket + 1, &sReady, nullptr, &sExcept, &nowait);
-
-				bool hasData;
-				if (ret == -1)
-				{
-					hasData = false;
-					Console.WriteLn("DEV9: ICMP: select failed. Error Code: %d", errno);
-				}
-				else if (FD_ISSET(icmpSocket, &sExcept))
-				{
-					hasData = false;
-
-					int error = 0;
-
-					socklen_t len = sizeof(error);
-					if (getsockopt(icmpSocket, SOL_SOCKET, SO_ERROR, (char*)&error, &len) < 0)
-						Console.Error("DEV9: ICMP: Unkown ICMP Connection Error (getsockopt Error: %d)", errno);
-					else
-						Console.Error("DEV9: ICMP: Recv Error: %d", error);
-				}
-				else
-					hasData = FD_ISSET(icmpSocket, &sReady);
-
-				if (hasData == false)
-				{
-					if (std::chrono::steady_clock::now() - icmpDeathClockStart > ICMP_TIMEOUT)
-					{
-						result.type = -2;
-						result.code = 0;
-						return &result;
-					}
-					else
-						return nullptr;
-				}
-#endif
-
-				sockaddr endpoint{0};
+				sockaddr_in endpoint{};
 
 				iovec iov;
 				iov.iov_base = icmpResponseBuffer.get();
 				iov.iov_len = icmpResponseBufferLen;
 
 #if defined(ICMP_SOCKETS_LINUX)
-				//Needs to hold cmsghdr + sock_extended_err + sockaddr_in
-				//for ICMP error responses (total 44 bytes)
-				//Unkown for other types of error
-				u8 cbuff[64];
+				// Needs to hold cmsghdr + sock_extended_err + sockaddr_in
+				// for ICMP error responses, this is a total of 44 bytes
+				// Unknown size needed for other error types
+				std::byte cbuff[64]{};
 #endif
 
-				msghdr msg{0};
+				msghdr msg{};
 				msg.msg_name = &endpoint;
 				msg.msg_namelen = sizeof(endpoint);
 				msg.msg_iov = &iov;
 				msg.msg_iovlen = 1;
 
-#if defined(ICMP_SOCKETS_LINUX)
-				ret = recvmsg(icmpSocket, &msg, MSG_DONTWAIT);
-#elif defined(ICMP_SOCKETS_BSD)
-				ret = recvmsg(icmpSocket, &msg, 0);
-#endif
+				int ret = recvmsg(icmpSocket, &msg, 0);
 
 				if (ret == -1)
 				{
 					int err = errno;
-#if defined(ICMP_SOCKETS_LINUX)
 					if (err == EAGAIN || err == EWOULDBLOCK)
 					{
 						if (std::chrono::steady_clock::now() - icmpDeathClockStart > ICMP_TIMEOUT)
@@ -369,14 +328,16 @@ namespace Sessions
 						else
 							return nullptr;
 					}
+#if defined(ICMP_SOCKETS_LINUX)
 					else
 					{
 						msg.msg_control = &cbuff;
 						msg.msg_controllen = sizeof(cbuff);
-						ret = recvmsg(icmpSocket, &msg, MSG_ERRQUEUE | MSG_DONTWAIT);
+						ret = recvmsg(icmpSocket, &msg, MSG_ERRQUEUE);
 					}
-#endif
+
 					if (ret == -1)
+#endif
 					{
 						Console.Error("DEV9: ICMP: RecvMsg Error: %d", err);
 						result.type = -1;
@@ -387,40 +348,59 @@ namespace Sessions
 
 				if (msg.msg_flags & MSG_TRUNC)
 					Console.Error("DEV9: ICMP: RecvMsg Truncated");
+
 #if defined(ICMP_SOCKETS_LINUX)
 				if (msg.msg_flags & MSG_CTRUNC)
 					Console.Error("DEV9: ICMP: RecvMsg Control Truncated");
 
-				sock_extended_err* ex_err = nullptr;
+				// On Linux, ICMP errors are stored in control messages retrieved using MSG_ERRQUEUE
+				sock_extended_err* exErrorPtr = nullptr;
 				cmsghdr* cmsg;
 
-				/* Receive auxiliary data in msgh */
+				// Search though control messages, taking the latest mesage
+				// We should only have at most 1 message
 				for (cmsg = CMSG_FIRSTHDR(&msg); cmsg != NULL; cmsg = CMSG_NXTHDR(&msg, cmsg))
 				{
 					if (cmsg->cmsg_level == SOL_IP && cmsg->cmsg_type == IP_RECVERR)
 					{
-						ex_err = (sock_extended_err*)CMSG_DATA(cmsg);
+						pxAssert(!exErrorPtr);
+						exErrorPtr = reinterpret_cast<sock_extended_err*>(CMSG_DATA(cmsg));
 						continue;
 					}
 					pxAssert(false);
 				}
 
-				if (ex_err != nullptr)
+				if (exErrorPtr != nullptr)
 				{
-					if (ex_err->ee_origin == SO_EE_ORIGIN_ICMP)
-					{
-						result.type = ex_err->ee_type;
-						result.code = ex_err->ee_code;
+					/* 
+					 * The pointer returned cannot be assumed to be suitably aligned for accessing arbitrary payload data types
+					 * So we would need to memcpy sock_extended_err
+					 */
+					sock_extended_err exError;
+					std::memcpy(&exError, exErrorPtr, sizeof(exError));
 
-						sockaddr_in* sockaddr = (sockaddr_in*)SO_EE_OFFENDER(ex_err);
-						result.address = *(IP_Address*)&sockaddr->sin_addr;
+					// Process the error
+					if (exError.ee_origin == SO_EE_ORIGIN_ICMP)
+					{
+						result.type = exError.ee_type;
+						result.code = exError.ee_code;
+
+						/* 
+						 * SO_EE_OFFENDER reads data relative to, but not necessarily included in struct sock_extended_err
+						 * So we need to pass the original pointer provided to us from CMSG_DATA()
+						 * However, the input pointer needs to be of type sock_extended_err*, hence the reinterpret_cast
+						 * The pointer returned may not be suitably aligned (see CMSG_DATA), so we need to memcpy
+						 */
+						sockaddr_in errorEndpoint;
+						std::memcpy(&errorEndpoint, SO_EE_OFFENDER(exErrorPtr), sizeof(errorEndpoint));
+						result.address = std::bit_cast<IP_Address>(errorEndpoint.sin_addr);
 						return &result;
 					}
 					else
 					{
-						Console.Error("DEV9: ICMP: Recv Error %d", ex_err->ee_errno);
+						Console.Error("DEV9: ICMP: Recv error %d", exError.ee_errno);
 						result.type = -1;
-						result.code = ex_err->ee_errno;
+						result.code = exError.ee_errno;
 						return &result;
 					}
 				}
@@ -438,63 +418,66 @@ namespace Sessions
 					else
 #endif
 					{
-						ip* ipHeader = (ip*)icmpResponseBuffer.get();
-						int headerLength = ipHeader->ip_hl << 2;
+						// Rely on implicit object creation
+						const ip* ipHeader = reinterpret_cast<ip*>(icmpResponseBuffer.get());
+						const int headerLength = ipHeader->ip_hl << 2;
 						pxAssert(headerLength == 20);
 
 						offset = headerLength;
 #ifdef __APPLE__
-						//https://www.unix.com/man-page/mojave/4/ip/
-						//"Note that the ip_off and ip_len fields are in host byte order."
-						//Any other bugs? FreeBSD notes the following
-						//Before FreeBSD 11.0 packets received on raw IP sockets had the ip_len and ip_off fields converted to host byte order.
-						//Before FreeBSD 10.0 packets received on raw IP sockets had the ip_hl sub-tracted from the ip_len field.
-						//TODO, test
-						length = ipHeader->ip_len - headerLength;
+						// Apple (old BSD)'s raw IP sockets implementation converts the ip_len field to host byte order
+						// and additionally subtracts the header length.
+						// https://www.unix.com/man-page/mojave/4/ip/
+						length = ipHeader->ip_len;
 #else
 						length = ntohs(ipHeader->ip_len) - headerLength;
 #endif
 					}
 
-					ICMP_Packet icmp(&icmpResponseBuffer[offset], length);
+					// Rely on implicit object creation for u8
+					ICMP_Packet icmp(reinterpret_cast<u8*>(&icmpResponseBuffer[offset]), length);
 					PayloadPtr* icmpPayload = static_cast<PayloadPtr*>(icmp.GetPayload());
 
 					result.type = icmp.type;
 					result.code = icmp.code;
 
-					sockaddr_in* sockaddr = (sockaddr_in*)&endpoint;
-					result.address = *(IP_Address*)&sockaddr->sin_addr;
+					result.address = std::bit_cast<IP_Address>(endpoint.sin_addr);
 
 					if (icmp.type == 0)
 					{
-						//Check if response is to us
+						// Check if response is for us
 						if (icmpConnectionKind == PingType::RAW)
 						{
-							ICMP_HeaderDataIdentifier headerData(icmp.headerData);
+							const ICMP_HeaderDataIdentifier headerData(icmp.headerData);
 							if (headerData.identifier != icmpId)
 								return nullptr;
 						}
 
-						//While icmp (and its PayloadPtr) will be destroyed when leaving this function
-						//the data pointed to it persist in icmpResponseBuffer
+						// While icmp (and its PayloadPtr) will be destroyed when leaving this function
+						// the data it points to persists in icmpResponseBuffer
 						result.dataLength = icmpPayload->GetLength();
 						result.data = icmpPayload->data;
 						return &result;
 					}
 #if defined(ICMP_SOCKETS_BSD)
+					// On BSD/Mac, ICMP errors are returned as normal packets
 					else if (icmp.type == 3 || icmp.type == 4 || icmp.type == 5 || icmp.type == 11)
 					{
-						//Check if response is to us
-						//We need to extract the sent header
+						// Extract the packet the ICMP message is responding to
 						IP_Packet ipPacket(icmpPayload->data, icmpPayload->GetLength(), true);
-						IP_PayloadPtr* ipPayload = static_cast<IP_PayloadPtr*>(ipPacket.GetPayload());
-						ICMP_Packet retIcmp(ipPayload->data, ipPayload->GetLength());
 
-						ICMP_HeaderDataIdentifier headerData(icmp.headerData);
+						if (ipPacket.protocol != static_cast<u8>(IP_Type::ICMP))
+							return nullptr;
+
+						IP_PayloadPtr* ipPayload = static_cast<IP_PayloadPtr*>(ipPacket.GetPayload());
+						ICMP_Packet icmpInner(ipPayload->data, ipPayload->GetLength());
+
+						// Check if response is for us
+						const ICMP_HeaderDataIdentifier headerData(icmpInner.headerData);
 						if (headerData.identifier != icmpId)
 							return nullptr;
 
-						//This response is for us
+						// This response is for us
 						return &result;
 					}
 #endif
@@ -504,7 +487,7 @@ namespace Sessions
 						Console.Error("DEV9: ICMP: Unexpected packet");
 						pxAssert(false);
 #endif
-						//Assume not for us
+						// Assume not for us
 						return nullptr;
 					}
 				}
@@ -520,25 +503,25 @@ namespace Sessions
 	bool ICMP_Session::Ping::Send(IP_Address parAdapterIP, IP_Address parDestIP, int parTimeToLive, PayloadPtr* parPayload)
 	{
 #ifdef _WIN32
-		//Documentation is incorrect, IP_OPTION_INFORMATION is to be used regardless of platform
-		IP_OPTION_INFORMATION ipInfo{0};
+		// Documentation is incorrect, IP_OPTION_INFORMATION is to be used regardless of platform
+		IP_OPTION_INFORMATION ipInfo{};
 		ipInfo.Ttl = parTimeToLive;
 		DWORD ret;
 		if (parAdapterIP.integer != 0)
-			ret = IcmpSendEcho2Ex(icmpFile, icmpEvent, nullptr, nullptr, parAdapterIP.integer, parDestIP.integer, parPayload->data, parPayload->GetLength(), &ipInfo, icmpResponseBuffer.get(), icmpResponseBufferLen,
-				(DWORD)std::chrono::duration_cast<std::chrono::milliseconds>(ICMP_TIMEOUT).count());
+			ret = IcmpSendEcho2Ex(icmpFile, icmpEvent, nullptr, nullptr, parAdapterIP.integer, parDestIP.integer, const_cast<u8*>(parPayload->data), parPayload->GetLength(), &ipInfo, icmpResponseBuffer.get(), icmpResponseBufferLen,
+				static_cast<DWORD>(std::chrono::duration_cast<std::chrono::milliseconds>(ICMP_TIMEOUT).count()));
 		else
-			ret = IcmpSendEcho2(icmpFile, icmpEvent, nullptr, nullptr, parDestIP.integer, parPayload->data, parPayload->GetLength(), &ipInfo, icmpResponseBuffer.get(), icmpResponseBufferLen,
-				(DWORD)std::chrono::duration_cast<std::chrono::milliseconds>(ICMP_TIMEOUT).count());
+			ret = IcmpSendEcho2(icmpFile, icmpEvent, nullptr, nullptr, parDestIP.integer, const_cast<u8*>(parPayload->data), parPayload->GetLength(), &ipInfo, icmpResponseBuffer.get(), icmpResponseBufferLen,
+				static_cast<DWORD>(std::chrono::duration_cast<std::chrono::milliseconds>(ICMP_TIMEOUT).count()));
 
-		//Documentation states that IcmpSendEcho2 returns ERROR_IO_PENDING
-		//However, it actully returns zero, with the error set to ERROR_IO_PENDING
+		// Documentation states that IcmpSendEcho2 returns ERROR_IO_PENDING
+		// However, it actually returns zero, with the error set to ERROR_IO_PENDING
 		if (ret == 0)
 			ret = GetLastError();
 
 		if (ret != ERROR_IO_PENDING)
 		{
-			Console.Error("DEV9: ICMP: Failed to Send Echo, %d", GetLastError());
+			Console.Error("DEV9: ICMP: Failed to send echo, %d", GetLastError());
 			return false;
 		}
 
@@ -551,55 +534,65 @@ namespace Sessions
 			{
 				icmpDeathClockStart = std::chrono::steady_clock::now();
 
-				//broadcast and multicast mignt need extra setsockopts
-				//I don't think any game will broadcast/multicast ping
+				// Broadcast and multicast might need extra setsockopts calls
+				// I don't think any game will do a broadcast/multicast ping
 
 				if (parAdapterIP.integer != 0)
 				{
-					sockaddr_in endpoint{0};
+					sockaddr_in endpoint{};
 					endpoint.sin_family = AF_INET;
-					*(IP_Address*)&endpoint.sin_addr = parAdapterIP;
+					endpoint.sin_addr = std::bit_cast<in_addr>(parAdapterIP);
 
-					if (bind(icmpSocket, (const sockaddr*)&endpoint, sizeof(endpoint)) == -1)
+					if (bind(icmpSocket, reinterpret_cast<const sockaddr*>(&endpoint), sizeof(endpoint)) == -1)
 					{
 						Console.Error("DEV9: ICMP: Failed to bind socket. Error: %d", errno);
 						::close(icmpSocket);
-						icmpSocket == -1;
+						icmpSocket = -1;
 						return false;
 					}
 				}
 
 #if defined(ICMP_SOCKETS_LINUX)
-				int value = 1;
-				if (setsockopt(icmpSocket, SOL_IP, IP_RECVERR, (char*)&value, sizeof(value)))
+				const int value = 1;
+				if (setsockopt(icmpSocket, SOL_IP, IP_RECVERR, reinterpret_cast<const char*>(&value), sizeof(value)))
 				{
 					Console.Error("DEV9: ICMP: Failed to setsockopt IP_RECVERR. Error: %d", errno);
 					::close(icmpSocket);
-					icmpSocket == -1;
+					icmpSocket = -1;
 					return false;
 				}
 #endif
 
-				// TTL (Note multicast & regular ttl are seperate)
-				if (setsockopt(icmpSocket, IPPROTO_IP, IP_TTL, (const char*)&parTimeToLive, sizeof(parTimeToLive)) == -1)
+				// TTL (Note multicast & regular ttl are separate)
+				if (setsockopt(icmpSocket, IPPROTO_IP, IP_TTL, reinterpret_cast<const char*>(&parTimeToLive), sizeof(parTimeToLive)) == -1)
 				{
 					Console.Error("DEV9: ICMP: Failed to set TTL. Error: %d", errno);
 					::close(icmpSocket);
-					icmpSocket == -1;
+					icmpSocket = -1;
+					return false;
+				}
+
+				// Non-blocking
+				int blocking = 1;
+				if (ioctl(icmpSocket, FIONBIO, &blocking) == -1)
+				{
+					Console.Error("DEV9: ICMP: Failed to set non-blocking. Error: %d", errno);
+					::close(icmpSocket);
+					icmpSocket = -1;
 					return false;
 				}
 
 #if defined(ICMP_SOCKETS_LINUX)
 				if (icmpConnectionKind == PingType::ICMP)
 				{
-					//We get assigned a port
-					sockaddr_in endpoint{0};
+					// We get assigned a port/Id
+					sockaddr_in endpoint{};
 					socklen_t endpointsize = sizeof(endpoint);
-					if (getsockname(icmpSocket, (sockaddr*)&endpoint, &endpointsize) == -1)
+					if (getsockname(icmpSocket, reinterpret_cast<sockaddr*>(&endpoint), &endpointsize) == -1)
 					{
 						Console.Error("DEV9: ICMP: Failed to get id. Error: %d", errno);
 						::close(icmpSocket);
-						icmpSocket == -1;
+						icmpSocket = -1;
 						return false;
 					}
 
@@ -608,15 +601,15 @@ namespace Sessions
 				else
 #endif
 				{
-					//Use time, in ms, as id
-					icmpId = (u16)std::chrono::duration_cast<std::chrono::milliseconds>(std::chrono::system_clock::now().time_since_epoch()).count();
+					// Use time, in ms, as id
+					icmpId = static_cast<u16>(std::chrono::duration_cast<std::chrono::milliseconds>(std::chrono::system_clock::now().time_since_epoch()).count());
 				}
 
 				ICMP_Packet icmp(parPayload->Clone());
 				icmp.type = 8;
 				icmp.code = 0;
 
-				//We only send one icmp packet per identifier
+				// We only send one icmp packet per identifier
 				ICMP_HeaderDataIdentifier headerData(icmpId, 1);
 				headerData.WriteHeaderData(icmp.headerData);
 
@@ -626,17 +619,27 @@ namespace Sessions
 				int offset = 0;
 				icmp.WriteBytes(buffer.get(), &offset);
 
-				sockaddr_in endpoint{0};
+				sockaddr_in endpoint{};
 				endpoint.sin_family = AF_INET;
-				*(IP_Address*)&endpoint.sin_addr.s_addr = parDestIP;
+				endpoint.sin_addr = std::bit_cast<in_addr>(parDestIP);
 
-				const int ret = sendto(icmpSocket, buffer.get(), icmp.GetLength(), 0, (const sockaddr*)&endpoint, sizeof(endpoint));
-				if (ret == -1)
+				int ret = -1;
+				while (ret == -1)
 				{
-					Console.Error("DEV9: ICMP: Send Error %d", errno);
-					::close(icmpSocket);
-					icmpSocket == -1;
-					return false;
+					ret = sendto(icmpSocket, buffer.get(), icmp.GetLength(), 0, reinterpret_cast<const sockaddr*>(&endpoint), sizeof(endpoint));
+					if (ret == -1)
+					{
+						const int err = errno;
+						if (err == EAGAIN || err == EWOULDBLOCK)
+							std::this_thread::yield();
+						else
+						{
+							Console.Error("DEV9: ICMP: Send error %d", errno);
+							::close(icmpSocket);
+							icmpSocket = -1;
+							return false;
+						}
+					}
 				}
 
 				return true;
@@ -676,23 +679,22 @@ namespace Sessions
 		connections = parConnections;
 	}
 
-	IP_Payload* ICMP_Session::Recv()
+	std::optional<ReceivedPayload> ICMP_Session::Recv()
 	{
 		std::unique_lock lock(ping_mutex);
 
 		for (size_t i = 0; i < pings.size(); i++)
 		{
-			ICMP_Session::PingResult* pingRet = nullptr;
-			pingRet = pings[i]->Recv();
+			const ICMP_Session::PingResult* pingRet = pings[i]->Recv();
 			if (pingRet != nullptr)
 			{
-				Ping* ping = pings[i];
-				//Remove ping from list and unlock
+				std::unique_ptr<Ping> ping = std::move(pings[i]);
+				// Remove ping from list and unlock mutex
 				pings.erase(pings.begin() + i);
 				lock.unlock();
 
-				//Create return ICMP packet
-				ICMP_Packet* ret = nullptr;
+				// Create return ICMP packet
+				std::optional<ReceivedPayload> ret;
 				if (pingRet->type >= 0)
 				{
 					PayloadData* data;
@@ -703,53 +705,46 @@ namespace Sessions
 					}
 					else
 					{
-						//We will copy the original packet back here
-						//Allocate fullsize buffer
-						u8* temp = new u8[ping->originalPacket->GetLength()];
-						//Allocate data
-						int responseSize = ping->originalPacket->GetHeaderLength() + 8;
+						// Copy the original packet into the returned ICMP packet
+						// Allocate fullsize buffer
+						std::vector<u8> temp = std::vector<u8>(ping->originalPacket->GetLength());
+						// Allocate returned ICMP payload
+						const int responseSize = ping->originalPacket->GetHeaderLength() + 8;
 						data = new PayloadData(responseSize);
 
-						//Write packet back into bytes
+						// Write packet into buffer
 						int offset = 0;
-						ping->originalPacket->WriteBytes(temp, &offset);
+						ping->originalPacket->WriteBytes(temp.data(), &offset);
 
-						//Copy only needed bytes
-						memcpy(data->data.get(), temp, responseSize);
-
-						//cleanup
-						delete[] temp;
+						// Copy only needed bytes
+						memcpy(data->data.get(), temp.data(), responseSize);
 					}
 
-					ret = new ICMP_Packet(data);
-					ret->type = pingRet->type;
-					ret->code = pingRet->code;
-					memcpy(ret->headerData, ping->headerData, 4);
+					std::unique_ptr<ICMP_Packet> pRet = std::make_unique<ICMP_Packet>(data);
+					pRet->type = pingRet->type;
+					pRet->code = pingRet->code;
+					memcpy(pRet->headerData, ping->headerData, 4);
 
-					if (destIP != pingRet->address)
-						destIP = pingRet->address;
+					ret = {pingRet->address, std::move(pRet)};
 				}
 				else if (pingRet->type == -1)
 					Console.Error("DEV9: ICMP: Unexpected ICMP status %d", pingRet->code);
 				else
 					DevCon.WriteLn("DEV9: ICMP: ICMP timeout");
 
-				//free ping
-				delete ping;
+				if (ret.has_value())
+					DevCon.WriteLn("DEV9: ICMP: Return Ping");
 
 				if (--open == 0)
 					RaiseEventConnectionClosed();
 
-				if (ret != nullptr)
-					DevCon.WriteLn("DEV9: ICMP: Return Ping");
-
-				//Return packet
+				// Return packet
 				return ret;
 			}
 		}
 
 		lock.unlock();
-		return nullptr;
+		return std::nullopt;
 	}
 
 	bool ICMP_Session::Send(PacketReader::IP::IP_Payload* payload)
@@ -759,7 +754,7 @@ namespace Sessions
 		return false;
 	}
 
-	//Note, expects caller to set ipTimeToLive before calling
+	// Expects caller to set ipTimeToLive before calling
 	bool ICMP_Session::Send(PacketReader::IP::IP_Payload* payload, IP_Packet* packet)
 	{
 		IP_PayloadPtr* ipPayload = static_cast<IP_PayloadPtr*>(payload);
@@ -769,16 +764,19 @@ namespace Sessions
 
 		switch (icmp.type)
 		{
-			case 3: //Port Closed
+			case 3: // Port Closed
 				switch (icmp.code)
 				{
 					case 3:
 					{
-						Console.Error("DEV9: ICMP: Recived Packet Rejected, Port Closed");
-
-						//RE:Outbreak Hackfix
-						//TODO, check if still needed
+						Console.Error("DEV9: ICMP: Received Packet Rejected, Port Closed");
 
+						/*
+						 * RE:Outbreak Hackfix
+						 * ICMP port closed messages has an extra 4 bytes of padding before the packet copy
+						 * this can be tested by trying to connect without using the resurrection server DNS
+						 * turbo mode may be needed to trigger the bug, depending on the DNS server's latency
+						 */
 						std::unique_ptr<IP_Packet> retPkt;
 						if ((icmpPayload->data[0] & 0xF0) == (4 << 4))
 							retPkt = std::make_unique<IP_Packet>(icmpPayload->data, icmpPayload->GetLength(), true);
@@ -787,27 +785,44 @@ namespace Sessions
 							Console.Error("DEV9: ICMP: Malformed ICMP Packet");
 							int off = 1;
 							while ((icmpPayload->data[off] & 0xF0) != (4 << 4))
+							{
 								off += 1;
 
+								// Require space for the IP Header and source/dest port of a UDP/TCP packet
+								// We don't generate packets with IP options, so IP header is always 20 bytes
+								if (icmpPayload->GetLength() - off - 24 < 0)
+								{
+									off = -1;
+									break;
+								}
+							}
+
+							if (off == -1)
+							{
+								Console.Error("DEV9: ICMP: Unable To Recover Data");
+								Console.Error("DEV9: ICMP: Failed To Reset Rejected Connection");
+								break;
+							}
+
 							Console.Error("DEV9: ICMP: Payload delayed %d bytes", off);
 
-							retPkt = std::make_unique<IP_Packet>(&icmpPayload->data[off], icmpPayload->GetLength(), true);
+							retPkt = std::make_unique<IP_Packet>(&icmpPayload->data[off], icmpPayload->GetLength() - off, true);
 						}
 
-						IP_Address srvIP = retPkt->sourceIP;
-						u8 prot = retPkt->protocol;
+						const IP_Address srvIP = retPkt->sourceIP;
+						const u8 prot = retPkt->protocol;
 						u16 srvPort = 0;
 						u16 ps2Port = 0;
 						switch (prot)
 						{
-							case (u8)IP_Type::TCP:
-							case (u8)IP_Type::UDP:
-								//Read ports directly from the payload
-								//both UDP and TCP have the same locations for ports
+							case static_cast<u8>(IP_Type::TCP):
+							case static_cast<u8>(IP_Type::UDP):
+								// Read ports directly from the payload
+								// both UDP and TCP have the same locations for ports
 								IP_PayloadPtr* payload = static_cast<IP_PayloadPtr*>(retPkt->GetPayload());
 								int offset = 0;
-								NetLib::ReadUInt16(payload->data, &offset, &srvPort); //src
-								NetLib::ReadUInt16(payload->data, &offset, &ps2Port); //dst
+								NetLib::ReadUInt16(payload->data, &offset, &srvPort); // src
+								NetLib::ReadUInt16(payload->data, &offset, &ps2Port); // dst
 						}
 
 						ConnectionKey Key{};
@@ -816,7 +831,7 @@ namespace Sessions
 						Key.ps2Port = ps2Port;
 						Key.srvPort = srvPort;
 
-						//is from Normal Port?
+						// Is from Normal Port?
 						BaseSession* s = nullptr;
 						connections->TryGetValue(Key, &s);
 
@@ -827,7 +842,7 @@ namespace Sessions
 							break;
 						}
 
-						//Is from Fixed Port?
+						// Is from Fixed Port?
 						Key.ip = {};
 						Key.srvPort = 0;
 						connections->TryGetValue(Key, &s);
@@ -845,18 +860,17 @@ namespace Sessions
 						Console.Error("DEV9: ICMP: Unsupported ICMP Code For Destination Unreachable %d", icmp.code);
 				}
 				break;
-			case 8: //Echo
+			case 8: // Echo
 			{
 				DevCon.WriteLn("DEV9: ICMP: Send Ping");
 				open++;
 
-				Ping* ping = new Ping(icmpPayload->GetLength());
+				std::unique_ptr<Ping> ping = std::make_unique<Ping>(icmpPayload->GetLength());
 
 				if (!ping->IsInitialised())
 				{
 					if (--open == 0)
 						RaiseEventConnectionClosed();
-					delete ping;
 					return false;
 				}
 
@@ -864,18 +878,17 @@ namespace Sessions
 				{
 					if (--open == 0)
 						RaiseEventConnectionClosed();
-					delete ping;
 					return false;
 				}
 
 				memcpy(ping->headerData, icmp.headerData, 4);
 
-				//Need to copy IP_Packet, original is stack allocated
+				// Need to copy IP_Packet, original is stack allocated
 				ping->originalPacket = std::make_unique<IP_Packet>(*packet);
 
 				{
 					std::scoped_lock lock(ping_mutex);
-					pings.push_back(ping);
+					pings.push_back(std::move(ping));
 				}
 
 				break;
@@ -894,10 +907,8 @@ namespace Sessions
 
 	ICMP_Session::~ICMP_Session()
 	{
+		// Cleanup
 		std::scoped_lock lock(ping_mutex);
-
-		//Cleanup
-		for (size_t i = 0; i < pings.size(); i++)
-			delete pings[i];
+		pings.clear();
 	}
 } // namespace Sessions
