@@ -1,21 +1,8 @@
-/*  PCSX2 - PS2 Emulator for PCs
- *  Copyright (C) 2002-2021  PCSX2 Dev Team
- *
- *  PCSX2 is free software: you can redistribute it and/or modify it under the terms
- *  of the GNU Lesser General Public License as published by the Free Software Found-
- *  ation, either version 3 of the License, or (at your option) any later version.
- *
- *  PCSX2 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
- *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
- *  PURPOSE.  See the GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License along with PCSX2.
- *  If not, see <http://www.gnu.org/licenses/>.
- */
-
-#include "PrecompiledHeader.h"
+// SPDX-FileCopyrightText: 2002-2025 PCSX2 Dev Team
+// SPDX-License-Identifier: GPL-3.0+
 
 #include <algorithm>
+#include <bit>
 #include <thread>
 
 #ifdef __POSIX__
@@ -51,18 +38,16 @@ namespace Sessions
 			}
 		}
 
-		//Maybe untested
+		// Maybe untested
 		if (tcp.GetRST() == true)
 		{
 			//DevCon.Writeln("DEV9: TCP: PS2 has reset connection");
-			//PS2 has reset connection;
+			// PS2 has reset connection;
 			if (client != INVALID_SOCKET)
 				CloseSocket();
 			else
-				Console.Error("DEV9: TCP: RESET CLOSED CONNECTION");
-			//PS2 sent RST
-			//clearly not expecting
-			//more data
+				Console.Error("DEV9: TCP: Reset closed connection");
+			// PS2 sent RST, clearly not expecting more data
 			state = TCP_State::CloseCompleted;
 			RaiseEventConnectionClosed();
 			return true;
@@ -75,14 +60,14 @@ namespace Sessions
 			case TCP_State::SendingSYN_ACK:
 				if (CheckRepeatSYNNumbers(&tcp) == NumCheckResult::Bad)
 				{
-					Console.Error("DEV9: TCP: Invalid Repeated SYN (SendingSYN_ACK)");
+					Console.Error("DEV9: TCP: Invalid repeated SYN (SendingSYN_ACK)");
 					return false;
 				}
-				return true; //Ignore reconnect attempts while we are still attempting connection
+				return true; // Ignore reconnect attempts while we are still attempting connection
 			case TCP_State::SentSYN_ACK:
 				return SendConnected(&tcp);
 			case TCP_State::Connected:
-				if (tcp.GetFIN() == true) //Connection Close Part 1, received FIN from PS2
+				if (tcp.GetFIN() == true) // Connection close part 1, received FIN from PS2
 					return CloseByPS2Stage1_2(&tcp);
 				else
 					return SendData(&tcp);
@@ -93,7 +78,7 @@ namespace Sessions
 				return CloseByPS2Stage4(&tcp);
 
 			case TCP_State::Closing_ClosedByRemote:
-				if (tcp.GetFIN() == true) //Connection Close Part 3, received FIN from PS2
+				if (tcp.GetFIN() == true) // Connection close part 3, received FIN from PS2
 					return CloseByRemoteStage3_4(&tcp);
 
 				return SendData(&tcp);
@@ -104,26 +89,26 @@ namespace Sessions
 				return false;
 			default:
 				CloseByRemoteRST();
-				Console.Error("DEV9: TCP: Invalid TCP State");
+				Console.Error("DEV9: TCP: Invalid TCP state");
 				return true;
 		}
 	}
 
-	//PS2 sent SYN
+	// PS2 sent SYN
 	bool TCP_Session::SendConnect(TCP_Packet* tcp)
 	{
-		//Expect SYN Packet
+		// Expects SYN Packet
 		destPort = tcp->destinationPort;
 		srcPort = tcp->sourcePort;
 
 		if (tcp->GetSYN() == false)
 		{
 			CloseByRemoteRST();
-			Console.Error("DEV9: TCP: Attempt To Send Data On Non Connected Connection");
+			Console.Error("DEV9: TCP: Attempt to send data to a non connected connection");
 			return true;
 		}
 		expectedSeqNumber = tcp->sequenceNumber + 1;
-		//Fill out last received numbers
+		// Reset last received numbers
 		receivedPS2SeqNumbers.clear();
 		for (int i = 0; i < receivedPS2SeqNumberCount; i++)
 			receivedPS2SeqNumbers.push_back(tcp->sequenceNumber);
@@ -134,24 +119,24 @@ namespace Sessions
 		{
 			switch (tcp->options[i]->GetCode())
 			{
-				case 0: //End
-				case 1: //Nop
+				case 0: // End
+				case 1: // Nop
 					continue;
-				case 2: //MSS
-					maxSegmentSize = ((TCPopMSS*)(tcp->options[i]))->maxSegmentSize;
+				case 2: // MSS
+					maxSegmentSize = static_cast<TCPopMSS*>(tcp->options[i])->maxSegmentSize;
 					break;
-				case 3: //WindowScale
-					windowScale = ((TCPopWS*)(tcp->options[i]))->windowScale;
+				case 3: // WindowScale
+					windowScale = static_cast<TCPopWS*>(tcp->options[i])->windowScale;
 					if (windowScale > 0)
-						Console.Error("DEV9: TCP: Non-Zero WindowScale Option");
+						Console.Error("DEV9: TCP: Non-zero window scale option");
 					break;
-				case 8: //TimeStamp
-					lastRecivedTimeStamp = ((TCPopTS*)(tcp->options[i]))->senderTimeStamp;
+				case 8: // TimeStamp
+					lastRecivedTimeStamp = static_cast<TCPopTS*>(tcp->options[i])->senderTimeStamp;
 					sendTimeStamps = true;
 					timeStampStart = std::chrono::steady_clock::now();
 					break;
 				default:
-					Console.Error("DEV9: TCP: Got Unknown Option %d", tcp->options[i]->GetCode());
+					Console.Error("DEV9: TCP: Got unknown option %d", tcp->options[i]->GetCode());
 					break;
 			}
 		}
@@ -160,7 +145,6 @@ namespace Sessions
 
 		CloseSocket();
 
-		//client = new Socket(SocketType.Stream, ProtocolType.Tcp);
 		client = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
 		if (client == INVALID_SOCKET)
 		{
@@ -177,11 +161,11 @@ namespace Sessions
 		int ret;
 		if (adapterIP.integer != 0)
 		{
-			sockaddr_in endpoint{0};
+			sockaddr_in endpoint{};
 			endpoint.sin_family = AF_INET;
-			*(IP_Address*)&endpoint.sin_addr = adapterIP;
+			endpoint.sin_addr = std::bit_cast<in_addr>(adapterIP);
 
-			ret = bind(client, (const sockaddr*)&endpoint, sizeof(endpoint));
+			ret = bind(client, reinterpret_cast<const sockaddr*>(&endpoint), sizeof(endpoint));
 
 			if (ret != 0)
 				Console.Error("DEV9: UDP: Failed to bind socket. Error: %d",
@@ -201,7 +185,7 @@ namespace Sessions
 #endif
 
 		if (ret != 0)
-			Console.Error("DEV9: TCP: Failed to set non blocking. Error: %d",
+			Console.Error("DEV9: TCP: Failed to set non-blocking. Error: %d",
 #ifdef _WIN32
 				WSAGetLastError());
 #elif defined(__POSIX__)
@@ -209,8 +193,8 @@ namespace Sessions
 #endif
 
 
-		const int noDelay = true; //BOOL
-		ret = setsockopt(client, IPPROTO_TCP, TCP_NODELAY, (const char*)&noDelay, sizeof(noDelay));
+		constexpr int noDelay = true; // BOOL on Windows
+		ret = setsockopt(client, IPPROTO_TCP, TCP_NODELAY, reinterpret_cast<const char*>(&noDelay), sizeof(noDelay));
 
 		if (ret != 0)
 			Console.Error("DEV9: TCP: Failed to set TCP_NODELAY. Error: %d",
@@ -220,12 +204,12 @@ namespace Sessions
 				errno);
 #endif
 
-		sockaddr_in endpoint{0};
+		sockaddr_in endpoint{};
 		endpoint.sin_family = AF_INET;
-		*(IP_Address*)&endpoint.sin_addr = destIP;
+		endpoint.sin_addr = std::bit_cast<in_addr>(destIP);
 		endpoint.sin_port = htons(destPort);
 
-		ret = connect(client, (const sockaddr*)&endpoint, sizeof(endpoint));
+		ret = connect(client, reinterpret_cast<const sockaddr*>(&endpoint), sizeof(endpoint));
 
 		if (ret != 0)
 		{
@@ -241,14 +225,14 @@ namespace Sessions
 				RaiseEventConnectionClosed();
 				return false;
 			}
-			//Compleation of socket connection checked in recv
+			// Compleation of socket connection checked in recv
 		}
 
 		state = TCP_State::SendingSYN_ACK;
 		return true;
 	}
 
-	//PS2 responding to our SYN-ACK (by sending ACK)
+	// PS2 responding to our SYN-ACK (by sending ACK)
 	bool TCP_Session::SendConnected(TCP_Packet* tcp)
 	{
 		if (tcp->GetSYN() == true)
@@ -256,16 +240,16 @@ namespace Sessions
 			if (CheckRepeatSYNNumbers(tcp) == NumCheckResult::Bad)
 			{
 				CloseByRemoteRST();
-				Console.Error("DEV9: TCP: Invalid Repeated SYN (SentSYN_ACK)");
+				Console.Error("DEV9: TCP: Invalid repeated SYN (SentSYN_ACK)");
 				return true;
 			}
-			return true; //Ignore reconnect attempts while we are still attempting connection
+			return true; // Ignore reconnect attempts while we are still attempting connection
 		}
 		const NumCheckResult Result = CheckNumbers(tcp);
 		if (Result == NumCheckResult::Bad)
 		{
 			CloseByRemoteRST();
-			Console.Error("DEV9: TCP: Bad TCP Numbers Received");
+			Console.Error("DEV9: TCP: Bad TCP numbers received");
 			return true;
 		}
 
@@ -273,18 +257,18 @@ namespace Sessions
 		{
 			switch (tcp->options[i]->GetCode())
 			{
-				case 0: //End
-				case 1: //Nop
+				case 0: // End
+				case 1: // Nop
 					continue;
-				case 8: //Timestamp
-					lastRecivedTimeStamp = ((TCPopTS*)(tcp->options[i]))->senderTimeStamp;
+				case 8: // Timestamp
+					lastRecivedTimeStamp = static_cast<TCPopTS*>(tcp->options[i])->senderTimeStamp;
 					break;
 				default:
-					Console.Error("DEV9: TCP: Got Unknown Option %d", tcp->options[i]->GetCode());
+					Console.Error("DEV9: TCP: Got unknown option %d", tcp->options[i]->GetCode());
 					break;
 			}
 		}
-		//Next packet will be data
+		// Next packet will be data
 		state = TCP_State::Connected;
 		return true;
 	}
@@ -294,27 +278,27 @@ namespace Sessions
 		if (tcp->GetSYN())
 		{
 			CloseByRemoteRST();
-			Console.Error("DEV9: TCP: Attempt to Connect to an open Port");
+			Console.Error("DEV9: TCP: Attempt to connect to an existing connection");
 			return true;
 		}
 		if (tcp->GetURG())
 		{
 			CloseByRemoteRST();
-			Console.Error("DEV9: TCP: Urgent Data Not Supported");
+			Console.Error("DEV9: TCP: Urgent data not supported");
 			return true;
 		}
 		for (size_t i = 0; i < tcp->options.size(); i++)
 		{
 			switch (tcp->options[i]->GetCode())
 			{
-				case 0: //End
-				case 1: //Nop
+				case 0: // End
+				case 1: // Nop
 					continue;
 				case 8:
-					lastRecivedTimeStamp = ((TCPopTS*)(tcp->options[i]))->senderTimeStamp;
+					lastRecivedTimeStamp = static_cast<TCPopTS*>(tcp->options[i])->senderTimeStamp;
 					break;
 				default:
-					Console.Error("DEV9: TCP: Got Unknown Option %d", tcp->options[i]->GetCode());
+					Console.Error("DEV9: TCP: Got unknown option %d", tcp->options[i]->GetCode());
 					break;
 			}
 		}
@@ -322,20 +306,23 @@ namespace Sessions
 		windowSize.store(tcp->windowSize << windowScale);
 
 		const NumCheckResult Result = CheckNumbers(tcp);
-		const uint delta = GetDelta(expectedSeqNumber, tcp->sequenceNumber);
-		//if (Result == NumCheckResult::GotOldData)
-		//{
-		//	DevCon.WriteLn("[PS2] New Data Offset: %d bytes", delta);
-		//	DevCon.WriteLn("[PS2] New Data Length: %d bytes", ((uint)tcp->GetPayload()->GetLength() - delta));
-		//}
+
 		if (Result == NumCheckResult::Bad)
 		{
 			CloseByRemoteRST();
-			Console.Error("DEV9: TCP: Bad TCP Numbers Received");
+			Console.Error("DEV9: TCP: Bad TCP numbers received");
 			return true;
 		}
 		if (tcp->GetPayload()->GetLength() != 0)
 		{
+			// Check if we already have sent some of this data
+			const int delta = GetDelta(expectedSeqNumber, tcp->sequenceNumber);
+			pxAssert(delta >= 0);
+			//if (Result == NumCheckResult::OldSeq)
+			//{
+			//	DevCon.WriteLn("[PS2] New data offset: %d bytes", delta);
+			//	DevCon.WriteLn("[PS2] New data length: %d bytes", tcp->GetPayload()->GetLength() - delta);
+			//}
 			if (tcp->GetPayload()->GetLength() - delta > 0)
 			{
 				DevCon.WriteLn("DEV9: TCP: [PS2] Sending: %d bytes", tcp->GetPayload()->GetLength());
@@ -343,14 +330,14 @@ namespace Sessions
 				receivedPS2SeqNumbers.erase(receivedPS2SeqNumbers.begin());
 				receivedPS2SeqNumbers.push_back(expectedSeqNumber);
 
-				//Send the Data
+				// Send the Data
 				int sent = 0;
 				PayloadPtr* payload = static_cast<PayloadPtr*>(tcp->GetPayload());
 				while (sent != payload->GetLength())
 				{
-					int ret = send(client, (const char*)&payload->data[sent], payload->GetLength() - sent, 0);
+					const int ret = send(client, reinterpret_cast<const char*>(&payload->data[sent]), payload->GetLength() - sent, 0);
 
-					if (sent == SOCKET_ERROR)
+					if (ret == SOCKET_ERROR)
 					{
 #ifdef _WIN32
 						const int err = WSAGetLastError();
@@ -363,7 +350,7 @@ namespace Sessions
 						else
 						{
 							CloseByRemoteRST();
-							Console.Error("DEV9: TCP: Send Error: %d", err);
+							Console.Error("DEV9: TCP: Send error: %d", err);
 							return true;
 						}
 					}
@@ -371,15 +358,15 @@ namespace Sessions
 						sent += ret;
 				}
 
-				expectedSeqNumber += ((uint)tcp->GetPayload()->GetLength() - delta);
-				//Done send
+				expectedSeqNumber += tcp->GetPayload()->GetLength() - delta;
+				// Done send
 			}
-			//ACK data
-			//DevCon.WriteLn("[SRV] ACK Data: %d", expectedSeqNumber);
-			TCP_Packet* ret = CreateBasePacket();
+			// ACK data
+			//DevCon.WriteLn("[SRV] ACK data: %u", expectedSeqNumber);
+			std::unique_ptr<TCP_Packet> ret = CreateBasePacket();
 			ret->SetACK(true);
 
-			PushRecvBuff(ret);
+			PushRecvBuff(ReceivedPayload{destIP, std::move(ret)});
 		}
 		return true;
 	}
@@ -389,18 +376,18 @@ namespace Sessions
 		if (tcp->GetSYN() == true)
 		{
 			CloseByRemoteRST();
-			Console.Error("DEV9: TCP: Attempt to Connect to an open Port");
+			Console.Error("DEV9: TCP: Attempt to connect to an existing connection");
 			return true;
 		}
 		for (size_t i = 0; i < tcp->options.size(); i++)
 		{
 			switch (tcp->options[i]->GetCode())
 			{
-				case 0: //End
-				case 1: //Nop
+				case 0: // End
+				case 1: // Nop
 					continue;
 				case 8:
-					lastRecivedTimeStamp = ((TCPopTS*)(tcp->options[i]))->senderTimeStamp;
+					lastRecivedTimeStamp = static_cast<TCPopTS*>(tcp->options[i])->senderTimeStamp;
 					break;
 				default:
 					Console.Error("DEV9: TCP: Got Unknown Option %d", tcp->options[i]->GetCode());
@@ -408,7 +395,7 @@ namespace Sessions
 			}
 		}
 
-		ErrorOnNonEmptyPacket(tcp);
+		ValidateEmptyPacket(tcp);
 
 		return true;
 	}
@@ -416,61 +403,69 @@ namespace Sessions
 	TCP_Session::NumCheckResult TCP_Session::CheckRepeatSYNNumbers(TCP_Packet* tcp)
 	{
 		//DevCon.WriteLn("DEV9: TCP: CHECK_REPEAT_SYN_NUMBERS");
-		//DevCon.WriteLn("DEV9: TCP: [SRV]CurrAckNumber = %d [PS2]Seq Number = %d", expectedSeqNumber, tcp->sequenceNumber);
+		//DevCon.WriteLn("DEV9: TCP: [SRV] CurrAckNumber = %u [PS2] Seq number = %u", expectedSeqNumber, tcp->sequenceNumber);
 
 		if (tcp->sequenceNumber != expectedSeqNumber - 1)
 		{
-			Console.Error("DEV9: TCP: [PS2] Sent Unexpected Sequence Number From Repeated SYN Packet, Got %d Expected %d", tcp->sequenceNumber, (expectedSeqNumber - 1));
+			Console.Error("DEV9: TCP: [PS2] Sent unexpected sequence number from repeated SYN packet, got %u expected %u", tcp->sequenceNumber, (expectedSeqNumber - 1));
 			return NumCheckResult::Bad;
 		}
 		return NumCheckResult::OK;
 	}
 
-	TCP_Session::NumCheckResult TCP_Session::CheckNumbers(TCP_Packet* tcp)
+	TCP_Session::NumCheckResult TCP_Session::CheckNumbers(TCP_Packet* tcp, bool rejectOldSeq)
 	{
 		u32 seqNum;
 		std::vector<u32> oldSeqNums;
 		std::tie(seqNum, oldSeqNums) = GetAllMyNumbers();
 
 		//DevCon.WriteLn("DEV9: TCP: CHECK_NUMBERS");
-		//DevCon.WriteLn("DEV9: TCP: [SRV]CurrSeqNumber = %d [PS2]Ack Number = %d", seqNum, tcp->acknowledgementNumber);
-		//DevCon.WriteLn("DEV9: TCP: [SRV]CurrAckNumber = %d [PS2]Seq Number = %d", expectedSeqNumber, tcp->sequenceNumber);
-		//DevCon.WriteLn("DEV9: TCP: [PS2]Data Length = %d",  tcp->GetPayload()->GetLength());
+		//DevCon.WriteLn("DEV9: TCP: [SRV] CurrSeqNumber = %u [PS2] Ack number = %u", seqNum, tcp->acknowledgementNumber);
+		//DevCon.WriteLn("DEV9: TCP: [SRV] CurrAckNumber = %u [PS2] Seq number = %u", expectedSeqNumber, tcp->sequenceNumber);
+		//DevCon.WriteLn("DEV9: TCP: [PS2] Data length = %u",  tcp->GetPayload()->GetLength());
 
 		if (tcp->acknowledgementNumber != seqNum)
 		{
-			//DevCon.WriteLn("DEV9: TCP: [PS2]Sent Outdated Acknowledgement Number, Got %d Expected %d", tcp->acknowledgementNumber, seqNum);
+			//DevCon.WriteLn("DEV9: TCP: [PS2] Sent outdated acknowledgement number, got %u expected %u", tcp->acknowledgementNumber, seqNum);
 
-			//Check if oldSeqNums contains tcp->acknowledgementNumber
+			// Check if oldSeqNums contains tcp->acknowledgementNumber
 			if (std::find(oldSeqNums.begin(), oldSeqNums.end(), tcp->acknowledgementNumber) == oldSeqNums.end())
 			{
-				Console.Error("DEV9: TCP: [PS2] Sent Unexpected Acknowledgement Number, did not Match Old Numbers, Got %d Expected %d", tcp->acknowledgementNumber, seqNum);
+				Console.Error("DEV9: TCP: [PS2] Sent unexpected acknowledgement number, did not match old numbers, got %u expected %u", tcp->acknowledgementNumber, seqNum);
 				return NumCheckResult::Bad;
 			}
 		}
 		else
 		{
-			//DevCon.WriteLn("[PS2]CurrSeqNumber Acknowleged By PS2");
+			//DevCon.WriteLn("[PS2] CurrSeqNumber acknowledged by PS2");
 			myNumberACKed.store(true);
 		}
 
+		UpdateReceivedAckNumber(tcp->acknowledgementNumber);
+
 		if (tcp->sequenceNumber != expectedSeqNumber)
 		{
-			if (tcp->GetPayload()->GetLength() == 0)
+			if (rejectOldSeq)
 			{
-				Console.Error("DEV9: TCP: [PS2] Sent Unexpected Sequence Number From ACK Packet, Got %d Expected %d", tcp->sequenceNumber, expectedSeqNumber);
+				Console.Error("DEV9: TCP: [PS2] Sent unexpected sequence number, got %u expected %u", tcp->sequenceNumber, expectedSeqNumber);
+				return NumCheckResult::Bad;
+			} 
+			else if (tcp->GetPayload()->GetLength() == 0)
+			{
+				Console.Error("DEV9: TCP: [PS2] Sent unexpected sequence number in a empty packet, got %u expected %u", tcp->sequenceNumber, expectedSeqNumber);
+				return NumCheckResult::OldSeq;
 			}
 			else
 			{
-				//Check if receivedPS2SeqNumbers contains tcp->sequenceNumber
+				// Check if receivedPS2SeqNumbers contains tcp->sequenceNumber
 				if (std::find(receivedPS2SeqNumbers.begin(), receivedPS2SeqNumbers.end(), tcp->sequenceNumber) == receivedPS2SeqNumbers.end())
 				{
-					Console.Error("DEV9: TCP: [PS2] Sent an Old Seq Number on an Data packet, Got %d Expected %d", tcp->sequenceNumber, expectedSeqNumber);
-					return NumCheckResult::GotOldData;
+					Console.Error("DEV9: TCP: [PS2] Sent outdated sequence number in an data packet, got %u expected %u", tcp->sequenceNumber, expectedSeqNumber);
+					return NumCheckResult::OldSeq;
 				}
 				else
 				{
-					Console.Error("DEV9: TCP: [PS2] Sent Unexpected Sequence Number From Data Packet, Got %d Expected %d", tcp->sequenceNumber, expectedSeqNumber);
+					Console.Error("DEV9: TCP: [PS2] Sent unexpected sequence number in a data packet, got %u expected %u", tcp->sequenceNumber, expectedSeqNumber);
 					return NumCheckResult::Bad;
 				}
 			}
@@ -478,49 +473,36 @@ namespace Sessions
 
 		return NumCheckResult::OK;
 	}
-	u32 TCP_Session::GetDelta(u32 parExpectedSeq, u32 parGotSeq)
+	bool TCP_Session::ValidateEmptyPacket(TCP_Packet* tcp, bool ignoreOld)
 	{
-		u32 delta = parExpectedSeq - parGotSeq;
-		if (delta > 0.5 * UINT_MAX)
-		{
-			delta = UINT_MAX - parExpectedSeq + parGotSeq;
-			Console.Error("DEV9: TCP: [PS2] SequenceNumber Overflow Detected");
-			Console.Error("DEV9: TCP: [PS2] New Data Offset: %d bytes", delta);
-		}
-		return delta;
-	}
-	bool TCP_Session::ErrorOnNonEmptyPacket(TCP_Packet* tcp)
-	{
-		NumCheckResult ResultFIN = CheckNumbers(tcp);
-		if (ResultFIN == NumCheckResult::GotOldData)
-		{
-			return false;
-		}
+		const NumCheckResult ResultFIN = CheckNumbers(tcp, !ignoreOld);
 		if (ResultFIN == NumCheckResult::Bad)
 		{
 			CloseByRemoteRST();
-			Console.Error("DEV9: TCP: Bad TCP Numbers Received");
+			Console.Error("DEV9: TCP: Bad TCP numbers received");
 			return true;
 		}
 		if (tcp->GetPayload()->GetLength() > 0)
 		{
-			uint delta = GetDelta(expectedSeqNumber, tcp->sequenceNumber);
-			if (delta == 0)
+			const int delta = GetDelta(expectedSeqNumber, tcp->sequenceNumber);
+			pxAssert(delta >= 0);
+			// Check if packet contains only old data
+			if (delta >= tcp->GetPayload()->GetLength())
 				return false;
 
 			CloseByRemoteRST();
-			Console.Error("DEV9: TCP: Invalid Packet, Packet Has Data");
+			Console.Error("DEV9: TCP: Invalid packet, packet has data");
 			return true;
 		}
 		return false;
 	}
 
-	//Connection Closing Finished in CloseByPS2Stage4
+	// Connection Closing Finished in CloseByPS2Stage4
 	bool TCP_Session::CloseByPS2Stage1_2(TCP_Packet* tcp)
 	{
 		//Console.WriteLn("DEV9: TCP: PS2 has closed connection");
 
-		if (ErrorOnNonEmptyPacket(tcp)) //Sending FIN with data
+		if (ValidateEmptyPacket(tcp, false)) // Check if valid packet for FIN
 			return true;
 
 		receivedPS2SeqNumbers.erase(receivedPS2SeqNumbers.begin());
@@ -531,37 +513,37 @@ namespace Sessions
 
 		const int result = shutdown(client, SD_SEND);
 		if (result == SOCKET_ERROR)
-			Console.Error("DEV9: TCP: Shutdown SD_SEND Error: %d",
+			Console.Error("DEV9: TCP: Shutdown SD_SEND error: %d",
 #ifdef _WIN32
 				WSAGetLastError());
 #elif defined(__POSIX__)
 				errno);
 #endif
 
-		//Connection Close Part 2, Send ACK to PS2
-		TCP_Packet* ret = CreateBasePacket();
+		// Connection close part 2, send ACK to PS2
+		std::unique_ptr<TCP_Packet> ret = CreateBasePacket();
 
 		ret->SetACK(true);
-		PushRecvBuff(ret);
+		PushRecvBuff(ReceivedPayload{destIP, std::move(ret)});
 
 		return true;
 	}
 
-	//PS2 responding to server response to PS2 Closing connection
+	// PS2 responding to server response to PS2 closing connection
 	bool TCP_Session::CloseByPS2Stage4(TCP_Packet* tcp)
 	{
-		//Close Part 4, Receive ACK from PS2
-		//Console.WriteLn("DEV9: TCP: Completed Close By PS2");
+		// Close part 4, receive ACK from PS2
+		//Console.WriteLn("DEV9: TCP: Completed close by PS2");
 
-		if (ErrorOnNonEmptyPacket(tcp))
+		if (ValidateEmptyPacket(tcp))
 			return true;
 
 		if (myNumberACKed.load())
 		{
 			//Console.WriteLn("DEV9: TCP: ACK was for FIN");
 			CloseSocket();
+			// recv buffer should be empty
 			state = TCP_State::CloseCompleted;
-			//recv buffer should be empty
 			RaiseEventConnectionClosed();
 		}
 
@@ -570,17 +552,17 @@ namespace Sessions
 
 	bool TCP_Session::CloseByRemoteStage2_ButAfter4(TCP_Packet* tcp)
 	{
-		//Console.WriteLn("DEV9: TCP: Completed Close By PS2");
+		//Console.WriteLn("DEV9: TCP: Completed close by PS2");
 
-		if (ErrorOnNonEmptyPacket(tcp))
+		if (ValidateEmptyPacket(tcp))
 			return true;
 
 		if (myNumberACKed.load())
 		{
 			//Console.WriteLn("DEV9: TCP: ACK was for FIN");
 			CloseSocket();
+			// Receive buffer may not be empty
 			state = TCP_State::CloseCompletedFlushBuffer;
-			//Recive buffer may not be empty
 		}
 		return true;
 	}
@@ -589,34 +571,34 @@ namespace Sessions
 	{
 		//Console.WriteLn("DEV9: TCP: PS2 has closed connection after remote");
 
-		if (ErrorOnNonEmptyPacket(tcp))
+		if (ValidateEmptyPacket(tcp, false)) // Check if valid packet for FIN
 			return true;
 
 		receivedPS2SeqNumbers.erase(receivedPS2SeqNumbers.begin());
 		receivedPS2SeqNumbers.push_back(expectedSeqNumber);
 		expectedSeqNumber += 1;
 
-		int result = shutdown(client, SD_SEND);
+		const int result = shutdown(client, SD_SEND);
 		if (result == SOCKET_ERROR)
-			Console.Error("DEV9: TCP: Shutdown SD_SEND Error: %d",
+			Console.Error("DEV9: TCP: Shutdown SD_SEND error: %d",
 #ifdef _WIN32
 				WSAGetLastError());
 #elif defined(__POSIX__)
 				errno);
 #endif
 
-		TCP_Packet* ret = CreateBasePacket();
+		std::unique_ptr<TCP_Packet> ret = CreateBasePacket();
 
 		ret->SetACK(true);
 
-		PushRecvBuff(ret);
+		PushRecvBuff(ReceivedPayload{destIP, std::move(ret)});
 
 		if (myNumberACKed.load())
 		{
 			//Console.WriteLn("DEV9: TCP: ACK was for FIN");
 			CloseSocket();
 			state = TCP_State::CloseCompletedFlushBuffer;
-			//Recive buffer may not be empty
+			// Receive buffer may not be empty
 		}
 		else
 			state = TCP_State::Closing_ClosedByRemoteThenPS2_WaitingForAck;
@@ -624,12 +606,12 @@ namespace Sessions
 		return true;
 	}
 
-	//Error on sending data
+	// Error on sending data
 	void TCP_Session::CloseByRemoteRST()
 	{
-		TCP_Packet* reterr = CreateBasePacket();
+		std::unique_ptr<TCP_Packet> reterr = CreateBasePacket();
 		reterr->SetRST(true);
-		PushRecvBuff(reterr);
+		PushRecvBuff(ReceivedPayload{destIP, std::move(reterr)});
 
 		CloseSocket();
 		state = TCP_State::CloseCompletedFlushBuffer;
