@@ -1,19 +1,5 @@
-/*  PCSX2 - PS2 Emulator for PCs
- *  Copyright (C) 2002-2014  PCSX2 Dev Team
- *
- *  PCSX2 is free software: you can redistribute it and/or modify it under the terms
- *  of the GNU Lesser General Public License as published by the Free Software Found-
- *  ation, either version 3 of the License, or (at your option) any later version.
- *
- *  PCSX2 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
- *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
- *  PURPOSE.  See the GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License along with PCSX2.
- *  If not, see <http://www.gnu.org/licenses/>.
- */
-
-#include "PrecompiledHeader.h"
+// SPDX-FileCopyrightText: 2002-2025 PCSX2 Dev Team
+// SPDX-License-Identifier: GPL-3.0+
 
 #include <string>
 #include <algorithm>
@@ -21,7 +7,7 @@
 
 #include "DisassemblyManager.h"
 #include "Memory.h"
-#include "Debug.h"
+#include "common/StringUtil.h"
 #include "MIPSAnalyst.h"
 
 int DisassemblyManager::maxParamChars = 29;
@@ -37,14 +23,14 @@ static u32 computeHash(u32 address, u32 size)
 	u32 hash = 0xBACD7814;
 	while (address < end)
 	{
-		hash += memRead32(address);
+		hash += r5900Debug.read32(address);
 		address += 4;
 	}
 	return hash;
 }
 
 
-void parseDisasm(SymbolMap& map, const char* disasm, char* opcode, char* arguments, bool insertSymbols)
+static void parseDisasm(SymbolGuardian& guardian, const char* disasm, char* opcode, char* arguments, size_t arguments_size, bool insertSymbols)
 {
 	if (*disasm == '(')
 	{
@@ -67,6 +53,7 @@ void parseDisasm(SymbolMap& map, const char* disasm, char* opcode, char* argumen
 		return;
 	}
 
+	char* arguments_start = arguments;
 	const char* jumpAddress = strstr(disasm,"->$");
 	const char* jumpRegister = strstr(disasm,"->");
 	while (*disasm != 0)
@@ -77,12 +64,12 @@ void parseDisasm(SymbolMap& map, const char* disasm, char* opcode, char* argumen
 			u32 branchTarget;
 			sscanf(disasm+3,"0x%08x",&branchTarget);
 
-			const std::string addressSymbol = map.GetLabelString(branchTarget);
+			const std::string addressSymbol = guardian.SymbolStartingAtAddress(branchTarget).name;
 			if (!addressSymbol.empty() && insertSymbols)
 			{
-				arguments += sprintf(arguments,"%s",addressSymbol.c_str());
+				arguments += std::snprintf(arguments, arguments_size - (arguments - arguments_start), "%s",addressSymbol.c_str());
 			} else {
-				arguments += sprintf(arguments,"0x%08X",branchTarget);
+				arguments += std::snprintf(arguments, arguments_size - (arguments - arguments_start), "0x%08X",branchTarget);
 			}
 
 			disasm += 3+2+8;
@@ -160,19 +147,50 @@ void DisassemblyManager::analyze(u32 address, u32 size = 1024)
 			continue;
 		}
 
-		SymbolInfo info;
-		if (!cpu->GetSymbolMap().GetSymbolInfo(&info,address,ST_ALL))
+		SymbolInfo info = cpu->GetSymbolGuardian().SymbolOverlappingAddress(
+			address, ccc::FUNCTION | ccc::GLOBAL_VARIABLE | ccc::LOCAL_VARIABLE);
+		
+		if (info.descriptor.has_value())
 		{
+			switch (*info.descriptor)
+			{
+				case ccc::SymbolDescriptor::FUNCTION:
+				{
+					DisassemblyFunction* function = new DisassemblyFunction(cpu,info.address.value,info.size);
+					entries[info.address.value] = function;
+					address = info.address.value + info.size;
+					break;
+				}
+				case ccc::SymbolDescriptor::GLOBAL_VARIABLE:
+				{
+					DisassemblyData* data = new DisassemblyData(cpu,info.address.value,info.size,DATATYPE_WORD);
+					entries[info.address.value] = data;
+					address = info.address.value+info.size;
+					break;
+				}
+				case ccc::SymbolDescriptor::LOCAL_VARIABLE:
+				{
+					DisassemblyData* data = new DisassemblyData(cpu,info.address.value,info.size,DATATYPE_WORD);
+					entries[info.address.value] = data;
+					address = info.address.value+info.size;
+					break;
+				}
+				default:
+					break;
+			}
+		} else {
 			if (address % 4)
 			{
-				u32 next = std::min<u32>((address+3) & ~3,cpu->GetSymbolMap().GetNextSymbolAddress(address,ST_ALL));
+				u32 next = std::min<u32>((address+3) & ~3,cpu->GetSymbolGuardian().SymbolAfterAddress(
+					address, ccc::FUNCTION | ccc::GLOBAL_VARIABLE | ccc::LOCAL_VARIABLE).address.value);
 				DisassemblyData* data = new DisassemblyData(cpu,address,next-address,DATATYPE_BYTE);
 				entries[address] = data;
 				address = next;
 				continue;
 			}
 
-			u32 next = cpu->GetSymbolMap().GetNextSymbolAddress(address,ST_ALL);
+			u32 next = cpu->GetSymbolGuardian().SymbolAfterAddress(
+				address, ccc::FUNCTION | ccc::GLOBAL_VARIABLE | ccc::LOCAL_VARIABLE).address.value;
 
 			if ((next % 4) && next != 0xFFFFFFFF)
 			{
@@ -194,26 +212,6 @@ void DisassemblyManager::analyze(u32 address, u32 size = 1024)
 			address = next;
 			continue;
 		}
-
-		switch (info.type)
-		{
-		case ST_FUNCTION:
-			{
-				DisassemblyFunction* function = new DisassemblyFunction(cpu,info.address,info.size);
-				entries[info.address] = function;
-				address = info.address+info.size;
-			}
-			break;
-		case ST_DATA:
-			{
-				DisassemblyData* data = new DisassemblyData(cpu,info.address,info.size,cpu->GetSymbolMap().GetDataType(info.address));
-				entries[info.address] = data;
-				address = info.address+info.size;
-			}
-			break;
-		default:
-			break;
-		}
 	}
 
 }
@@ -408,13 +406,13 @@ u32 DisassemblyFunction::getLineAddress(int line)
 	return lineAddresses[line];
 }
 
-bool DisassemblyFunction::disassemble(u32 address, DisassemblyLineInfo& dest, bool insertSymbols)
+bool DisassemblyFunction::disassemble(u32 address, DisassemblyLineInfo& dest, bool insertSymbols, bool simplify)
 {
 	auto it = findDisassemblyEntry(entries,address,false);
 	if (it == entries.end())
 		return false;
 
-	return it->second->disassemble(address,dest,insertSymbols);
+	return it->second->disassemble(address,dest,insertSymbols,simplify);
 }
 
 void DisassemblyFunction::getBranchLines(u32 start, u32 size, std::vector<BranchLine>& dest)
@@ -542,21 +540,23 @@ void DisassemblyFunction::load()
 	u32 funcPos = address;
 	u32 funcEnd = address+size;
 
-	u32 nextData = cpu->GetSymbolMap().GetNextSymbolAddress(funcPos-1,ST_DATA);
+	SymbolInfo nextData = cpu->GetSymbolGuardian().SymbolAfterAddress(
+		funcPos-1, ccc::GLOBAL_VARIABLE | ccc::LOCAL_VARIABLE);
 	u32 opcodeSequenceStart = funcPos;
 	while (funcPos < funcEnd)
 	{
-		if (funcPos == nextData)
+		if (funcPos == nextData.address.value && nextData.size > 0)
 		{
 			if (opcodeSequenceStart != funcPos)
 				addOpcodeSequence(opcodeSequenceStart,funcPos);
 
-			DisassemblyData* data = new DisassemblyData(cpu,funcPos,cpu->GetSymbolMap().GetDataSize(funcPos),cpu->GetSymbolMap().GetDataType(funcPos));
+			DisassemblyData* data = new DisassemblyData(cpu,funcPos,nextData.size,DATATYPE_WORD);
 			entries[funcPos] = data;
 			lineAddresses.push_back(funcPos);
 			funcPos += data->getTotalSize();
 
-			nextData = cpu->GetSymbolMap().GetNextSymbolAddress(funcPos-1,ST_DATA);
+			nextData = cpu->GetSymbolGuardian().SymbolAfterAddress(funcPos-1,
+				ccc::GLOBAL_VARIABLE | ccc::LOCAL_VARIABLE);
 			opcodeSequenceStart = funcPos;
 			continue;
 		}
@@ -576,7 +576,7 @@ void DisassemblyFunction::load()
 		}
 
 		MIPSAnalyst::MipsOpcodeInfo opInfo = MIPSAnalyst::GetOpcodeInfo(cpu,funcPos);
-		u32 opAddress = funcPos;
+		//u32 opAddress = funcPos;
 		funcPos += 4;
 
 		// skip branches and their delay slots
@@ -586,8 +586,27 @@ void DisassemblyFunction::load()
 			continue;
 		}
 
+/*
+	The QT debugger doesn't follow the same logic as the disassembler
+	It _should_ follow the path of the disassembler, but instead it is naively reading the
+	disassembler output for every single instruction.
+	This causes issues disassembling:
+		0x1000 lui $t0, 0x1234
+		0x1004 ori $t0, $t0, 0x5678
+		0x1008 nop
+	Into:
+		0x1000 li $t0, 0x12345678
+		0x1004 li $t0, 0x12346789
+		0x1008 nop
+	Where it should be:
+		0x1000 li $t0, 0x12345678
+		0x1008 nop
+
+	As a quick remedy, I'm disabling the macro generation.
+*/
+#if 0
 		// lui
-		if (MIPS_GET_OP(opInfo.encodedOpcode) == 0x0F && funcPos < funcEnd && funcPos != nextData)
+		if (MIPS_GET_OP(opInfo.encodedOpcode) == 0x0F && funcPos < funcEnd && funcPos != nextData.address.value)
 		{
 			u32 next = cpu->read32(funcPos);
 
@@ -674,7 +693,7 @@ void DisassemblyFunction::load()
 				}
 			}
 		}
-
+#endif
 		// just a normal opcode
 	}
 
@@ -695,12 +714,12 @@ void DisassemblyFunction::clear()
 	hash = 0;
 }
 
-bool DisassemblyOpcode::disassemble(u32 address, DisassemblyLineInfo& dest, bool insertSymbols)
+bool DisassemblyOpcode::disassemble(u32 address, DisassemblyLineInfo& dest, bool insertSymbols, bool simplify)
 {
 	char opcode[64],arguments[256];
 
-	std::string dis = cpu->disasm(address,insertSymbols);
-	parseDisasm(cpu->GetSymbolMap(),dis.c_str(),opcode,arguments,insertSymbols);
+	std::string dis = cpu->disasm(address,simplify);
+	parseDisasm(cpu->GetSymbolGuardian(),dis.c_str(),opcode,arguments,std::size(arguments),insertSymbols);
 	dest.type = DISTYPE_OPCODE;
 	dest.name = opcode;
 	dest.params = arguments;
@@ -765,7 +784,7 @@ void DisassemblyMacro::setMacroMemory(const std::string& _name, u32 _immediate,
 	numOpcodes = 2;
 }
 
-bool DisassemblyMacro::disassemble(u32 address, DisassemblyLineInfo& dest, bool insertSymbols)
+bool DisassemblyMacro::disassemble(u32 address, DisassemblyLineInfo& dest, bool insertSymbols, bool simplify)
 {
 	char buffer[64];
 	dest.type = DISTYPE_MACRO;
@@ -777,12 +796,12 @@ bool DisassemblyMacro::disassemble(u32 address, DisassemblyLineInfo& dest, bool
 	case MACRO_LI:
 		dest.name = name;
 
-		addressSymbol = cpu->GetSymbolMap().GetLabelString(immediate);
+		addressSymbol = cpu->GetSymbolGuardian().SymbolStartingAtAddress(immediate).name;
 		if (!addressSymbol.empty() && insertSymbols)
 		{
-			sprintf(buffer,"%s,%s",cpu->getRegisterName(0,rt),addressSymbol.c_str());
+			std::snprintf(buffer,std::size(buffer),"%s,%s",cpu->getRegisterName(0,rt),addressSymbol.c_str());
 		} else {
-			sprintf(buffer,"%s,0x%08X",cpu->getRegisterName(0,rt),immediate);
+			std::snprintf(buffer,std::size(buffer),"%s,0x%08X",cpu->getRegisterName(0,rt),immediate);
 		}
 
 		dest.params = buffer;
@@ -793,12 +812,12 @@ bool DisassemblyMacro::disassemble(u32 address, DisassemblyLineInfo& dest, bool
 	case MACRO_MEMORYIMM:
 		dest.name = name;
 
-		addressSymbol = cpu->GetSymbolMap().GetLabelString(immediate);
+		addressSymbol = cpu->GetSymbolGuardian().SymbolStartingAtAddress(immediate).name;
 		if (!addressSymbol.empty() && insertSymbols)
 		{
-			sprintf(buffer,"%s,%s",cpu->getRegisterName(0,rt),addressSymbol.c_str());
+			std::snprintf(buffer,std::size(buffer),"%s,%s",cpu->getRegisterName(0,rt),addressSymbol.c_str());
 		} else {
-			sprintf(buffer,"%s,0x%08X",cpu->getRegisterName(0,rt),immediate);
+			std::snprintf(buffer,std::size(buffer),"%s,0x%08X",cpu->getRegisterName(0,rt),immediate);
 		}
 
 		dest.params = buffer;
@@ -836,7 +855,7 @@ void DisassemblyData::recheck()
 	}
 }
 
-bool DisassemblyData::disassemble(u32 address, DisassemblyLineInfo& dest, bool insertSymbols)
+bool DisassemblyData::disassemble(u32 address, DisassemblyLineInfo& dest, bool insertSymbols, bool simplify)
 {
 	dest.type = DISTYPE_DATA;
 
@@ -899,7 +918,7 @@ void DisassemblyData::createLines()
 		bool inString = false;
 		while (pos < end)
 		{
-			u8 b = memRead8(pos++);
+			u8 b = r5900Debug.read8(pos++);
 			if (b >= 0x20 && b <= 0x7F)
 			{
 				if (currentLine.size()+1 >= maxChars)
@@ -923,9 +942,9 @@ void DisassemblyData::createLines()
 			} else {
 				char buffer[64];
 				if (pos == end && b == 0)
-					strcpy(buffer,"0");
+					StringUtil::Strlcpy(buffer, "0", std::size(buffer));
 				else
-					sprintf(buffer,"0x%02X",b);
+					std::snprintf(buffer, std::size(buffer), "0x%02X", b);
 
 				if (currentLine.size()+strlen(buffer) >= maxChars)
 				{
@@ -976,23 +995,23 @@ void DisassemblyData::createLines()
 			switch (type)
 			{
 			case DATATYPE_BYTE:
-				value = memRead8(pos);
-				sprintf(buffer,"0x%02X",value);
+				value = cpu->read8(pos);
+				std::snprintf(buffer,std::size(buffer),"0x%02X",value);
 				pos++;
 				break;
 			case DATATYPE_HALFWORD:
-				value = memRead16(pos);
-				sprintf(buffer,"0x%04X",value);
+				value = cpu->read16(pos);
+				std::snprintf(buffer,std::size(buffer),"0x%04X",value);
 				pos += 2;
 				break;
 			case DATATYPE_WORD:
 				{
-					value = memRead32(pos);
-					const std::string label = cpu->GetSymbolMap().GetLabelString(value);
+					value = cpu->read32(pos);
+					const std::string label = cpu->GetSymbolGuardian().SymbolStartingAtAddress(value).name;
 					if (!label.empty())
-						sprintf(buffer,"%s",label.c_str());
+						std::snprintf(buffer,std::size(buffer),"%s",label.c_str());
 					else
-						sprintf(buffer,"0x%08X",value);
+						std::snprintf(buffer,std::size(buffer),"0x%08X",value);
 					pos += 4;
 				}
 				break;
@@ -1034,7 +1053,7 @@ DisassemblyComment::DisassemblyComment(DebugInterface* _cpu, u32 _address, u32 _
 
 }
 
-bool DisassemblyComment::disassemble(u32 address, DisassemblyLineInfo& dest, bool insertSymbols)
+bool DisassemblyComment::disassemble(u32 address, DisassemblyLineInfo& dest, bool insertSymbols, bool simplify)
 {
 	dest.type = DISTYPE_OTHER;
 	dest.name = name;
