@@ -1,43 +1,36 @@
-/*  PCSX2 - PS2 Emulator for PCs
- *  Copyright (C) 2002-2021 PCSX2 Dev Team
- *
- *  PCSX2 is free software: you can redistribute it and/or modify it under the terms
- *  of the GNU Lesser General Public License as published by the Free Software Found-
- *  ation, either version 3 of the License, or (at your option) any later version.
- *
- *  PCSX2 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
- *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
- *  PURPOSE.  See the GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License along with PCSX2.
- *  If not, see <http://www.gnu.org/licenses/>.
- */
+// SPDX-FileCopyrightText: 2002-2025 PCSX2 Dev Team
+// SPDX-License-Identifier: GPL-3.0+
 
 #pragma once
 
 #include "common/HashCombine.h"
 #include "common/WindowInfo.h"
-#include "GSFastList.h"
-#include "GSTexture.h"
-#include "GSVertex.h"
+#include "GS/GS.h"
+#include "GS/Renderers/Common/GSFastList.h"
+#include "GS/Renderers/Common/GSTexture.h"
+#include "GS/Renderers/Vulkan/GSTextureVK.h"
+#include "GS/Renderers/Common/GSVertex.h"
 #include "GS/GSAlignedClass.h"
 #include "GS/GSExtra.h"
 #include <array>
 
-class HostDisplay;
-
 enum class ShaderConvert
 {
 	COPY = 0,
 	RGBA8_TO_16_BITS,
 	DATM_1,
 	DATM_0,
-	HDR_INIT,
-	HDR_RESOLVE,
+	DATM_1_RTA_CORRECTION,
+	DATM_0_RTA_CORRECTION,
+	COLCLIP_INIT,
+	COLCLIP_RESOLVE,
+	RTA_CORRECTION,
+	RTA_DECORRECTION,
 	TRANSPARENCY_FILTER,
 	FLOAT32_TO_16_BITS,
 	FLOAT32_TO_32_BITS,
 	FLOAT32_TO_RGBA8,
+	FLOAT32_TO_RGB8,
 	FLOAT16_TO_RGB5A1,
 	RGBA8_TO_FLOAT32,
 	RGBA8_TO_FLOAT24,
@@ -47,12 +40,35 @@ enum class ShaderConvert
 	RGBA8_TO_FLOAT24_BILN,
 	RGBA8_TO_FLOAT16_BILN,
 	RGB5A1_TO_FLOAT16_BILN,
+	FLOAT32_TO_FLOAT24,
 	DEPTH_COPY,
+	DOWNSAMPLE_COPY,
 	RGBA_TO_8I,
+	RGB5A1_TO_8I,
+	CLUT_4,
+	CLUT_8,
 	YUV,
 	Count
 };
 
+enum class SetDATM : u8
+{
+	DATM0 = 0U,
+	DATM1,
+	DATM0_RTA_CORRECTION,
+	DATM1_RTA_CORRECTION
+};
+
+enum class ShaderInterlace
+{
+	WEAVE = 0,
+	BOB = 1,
+	BLEND = 2,
+	MAD_BUFFER = 3,
+	MAD_RECONSTRUCT = 4,
+	Count
+};
+
 static inline bool HasDepthOutput(ShaderConvert shader)
 {
 	switch (shader)
@@ -65,6 +81,7 @@ static inline bool HasDepthOutput(ShaderConvert shader)
 		case ShaderConvert::RGBA8_TO_FLOAT24_BILN:
 		case ShaderConvert::RGBA8_TO_FLOAT16_BILN:
 		case ShaderConvert::RGB5A1_TO_FLOAT16_BILN:
+		case ShaderConvert::FLOAT32_TO_FLOAT24:
 		case ShaderConvert::DEPTH_COPY:
 			return true;
 		default:
@@ -78,6 +95,8 @@ static inline bool HasStencilOutput(ShaderConvert shader)
 	{
 		case ShaderConvert::DATM_0:
 		case ShaderConvert::DATM_1:
+		case ShaderConvert::DATM_0_RTA_CORRECTION:
+		case ShaderConvert::DATM_1_RTA_CORRECTION:
 			return true;
 		default:
 			return false;
@@ -112,6 +131,17 @@ static inline bool SupportsBilinear(ShaderConvert shader)
 	}
 }
 
+static inline u32 ShaderConvertWriteMask(ShaderConvert shader)
+{
+	switch (shader)
+	{
+		case ShaderConvert::FLOAT32_TO_RGB8:
+			return 0x7;
+		default:
+			return 0xf;
+	}
+}
+
 enum class PresentShader
 {
 	COPY = 0,
@@ -120,11 +150,14 @@ enum class PresentShader
 	TRIANGULAR_FILTER,
 	COMPLEX_FILTER,
 	LOTTES_FILTER,
+	SUPERSAMPLE_4xRGSS,
+	SUPERSAMPLE_AUTO,
 	Count
 };
 
 /// Get the name of a shader
 /// (Can't put methods on an enum class)
+int SetDATMShader(SetDATM datm);
 const char* shaderName(ShaderConvert value);
 const char* shaderName(PresentShader value);
 
@@ -139,11 +172,22 @@ enum ChannelFetch
 	ChannelFetch_GXBY  = 6,
 };
 
-#pragma pack(push, 1)
+enum class HWBlendType
+{
+	SRC_ONE_DST_FACTOR      = 1, // Use the dest color as blend factor, Cs is set to 1.
+	SRC_ALPHA_DST_FACTOR    = 2, // Use the dest color as blend factor, Cs is set to (Alpha - 1).
+	SRC_DOUBLE              = 3, // Double source color.
+	SRC_HALF_ONE_DST_FACTOR = 4, // Use the dest color as blend factor, Cs is set to 0.5, additionally divide As or Af by 2.
+	SRC_INV_DST_BLEND_HALF  = 5, // Halve the alpha then double the final result.
+	INV_SRC_DST_BLEND_HALF  = 6, // Halve the alpha then double the final result.
 
-class DisplayConstantBuffer
+	BMIX1_ALPHA_HIGH_ONE    = 1, // Blend formula is replaced when alpha is higher than 1.
+	BMIX1_SRC_HALF          = 2, // Impossible blend will always be wrong on hw, divide Cs by 2.
+	BMIX2_OVERFLOW          = 3, // Blending Cs might overflow, try to compensate.
+};
+
+struct alignas(16) DisplayConstantBuffer
 {
-public:
 	GSVector4 SourceRect; // +0,xyzw
 	GSVector4 TargetRect; // +16,xyzw
 	GSVector2 SourceSize; // +32,xy
@@ -175,46 +219,54 @@ public:
 		TimeAndPad = GSVector4(time);
 	}
 };
+static_assert(sizeof(DisplayConstantBuffer) == 96, "DisplayConstantBuffer is correct size");
 
-class MergeConstantBuffer
+struct alignas(16) MergeConstantBuffer
 {
-public:
 	GSVector4 BGColor;
 	u32 EMODA;
 	u32 EMODC;
-	u32 pad[2];
+	u32 DOFFSET;
+	float ScaleFactor;
 };
+static_assert(sizeof(MergeConstantBuffer) == 32, "MergeConstantBuffer is correct size");
 
-class InterlaceConstantBuffer
+struct alignas(16) InterlaceConstantBuffer
 {
-public:
 	GSVector4 ZrH; // data passed to the shader
-	InterlaceConstantBuffer() { memset(this, 0, sizeof(*this)); }
 };
-
-#pragma pack(pop)
+static_assert(sizeof(InterlaceConstantBuffer) == 16, "InterlaceConstantBuffer is correct size");
 
 enum HWBlendFlags
 {
-	// A couple of flag to determine the blending behavior
-	BLEND_CD        = 0x8,    // Output is Cd, hw blend can handle it
-	BLEND_MIX1      = 0x10,   // Mix of hw and sw, do Cs*F or Cs*As in shader
-	BLEND_MIX2      = 0x20,   // Mix of hw and sw, do Cs*(As + 1) or Cs*(F + 1) in shader
-	BLEND_MIX3      = 0x40,   // Mix of hw and sw, do Cs*(1 - As) or Cs*(1 - F) in shader
-	BLEND_A_MAX     = 0x80,   // Impossible blending uses coeff bigger than 1
-	BLEND_C_CLR1    = 0x100,  // Clear color blending (use directly the destination color as blending factor)
-	BLEND_C_CLR2_AF = 0x200,  // Clear color blending (use directly the destination color as blending factor)
-	BLEND_C_CLR2_AS = 0x400,  // Clear color blending (use directly the destination color as blending factor)
-	BLEND_C_CLR3    = 0x800,  // Multiply Cs by (255/128) to compensate for wrong Ad/255 value, should be Ad/128
-	BLEND_NO_REC    = 0x1000, // Doesn't require sampling of the RT as a texture
-	BLEND_ACCU      = 0x2000, // Allow to use a mix of SW and HW blending to keep the best of the 2 worlds
+	// Flags to determine blending behavior
+	BLEND_CD     = 0x1,    // Output is Cd, hw blend can handle it
+	BLEND_HW1    = 0x2,    // Clear color blending (use directly the destination color as blending factor)
+	BLEND_HW2    = 0x4,    // Clear color blending (use directly the destination color as blending factor)
+	BLEND_HW3    = 0x8,    // Multiply Cs by (255/128) to compensate for wrong Ad/255 value, should be Ad/128
+	BLEND_HW4    = 0x10,   // HW rendering is split in 2 passes
+	BLEND_HW5    = 0x20,   // HW rendering is split in 2 passes
+	BLEND_HW6    = 0x40,   // HW rendering is split in 2 passes
+	BLEND_HW7    = 0x80,   // HW rendering is split in 2 passes
+	BLEND_HW8    = 0x100,  // HW rendering is split in 2 passes
+	BLEND_HW9    = 0x200,  // HW rendering is split in 2 passes
+	BLEND_MIX1   = 0x400,  // Mix of hw and sw, do Cs*F or Cs*As in shader
+	BLEND_MIX2   = 0x800,  // Mix of hw and sw, do Cs*(As + 1) or Cs*(F + 1) in shader
+	BLEND_MIX3   = 0x1000, // Mix of hw and sw, do Cs*(1 - As) or Cs*(1 - F) in shader
+	BLEND_ACCU   = 0x2000, // Allow to use a mix of SW and HW blending to keep the best of the 2 worlds
+	BLEND_NO_REC = 0x4000, // Doesn't require sampling of the RT as a texture
+	BLEND_A_MAX  = 0x8000, // Impossible blending uses coeff bigger than 1
 };
 
-// Determines the HW blend function for DX11/OGL
+// Determines the HW blend function for the video backend
 struct HWBlend
 {
+	typedef u8 BlendOp; /*GSDevice::BlendOp*/
+	typedef u8 BlendFactor; /*GSDevice::BlendFactor*/
+
 	u16 flags;
-	u8 op, src, dst;
+	BlendOp op;
+	BlendFactor src, dst;
 };
 
 struct alignas(16) GSHWDrawConfig
@@ -225,13 +277,6 @@ struct alignas(16) GSHWDrawConfig
 		Line,
 		Triangle,
 	};
-	enum class GSTopology: u8
-	{
-		Point,
-		Line,
-		Triangle,
-		Sprite,
-	};
 	enum class VSExpand: u8
 	{
 		None,
@@ -240,22 +285,6 @@ struct alignas(16) GSHWDrawConfig
 		Sprite,
 	};
 #pragma pack(push, 1)
-	struct GSSelector
-	{
-		union
-		{
-			struct
-			{
-				GSTopology topology : 2;
-				bool expand : 1;
-				bool iip : 1;
-				bool forward_primid : 1;
-			};
-			u8 key;
-		};
-		GSSelector(): key(0) {}
-		GSSelector(u8 k): key(k) {}
-	};
 	struct VSSelector
 	{
 		union
@@ -265,7 +294,7 @@ struct alignas(16) GSHWDrawConfig
 				u8 fst : 1;
 				u8 tme : 1;
 				u8 iip : 1;
-				u8 point_size : 1;		///< Set when points need to be expanded without geometry shader.
+				u8 point_size : 1;		///< Set when points need to be expanded without VS expanding.
 				VSExpand expand : 2;
 				u8 _free : 2;
 			};
@@ -273,7 +302,11 @@ struct alignas(16) GSHWDrawConfig
 		};
 		VSSelector(): key(0) {}
 		VSSelector(u8 k): key(k) {}
+
+		/// Returns true if the fixed index buffer should be used.
+		__fi bool UseExpandIndexBuffer() const { return (expand == VSExpand::Point || expand == VSExpand::Sprite); }
 	};
+	static_assert(sizeof(VSSelector) == 1, "VSSelector is a single byte");
 #pragma pack(pop)
 #pragma pack(push, 4)
 	struct PSSelector
@@ -285,11 +318,10 @@ struct alignas(16) GSHWDrawConfig
 		{
 			struct
 			{
-				// *** Word 1
 				// Format
 				u32 aem_fmt   : 2;
 				u32 pal_fmt   : 2;
-				u32 dfmt      : 2; // 0 → 32-bit, 1 → 24-bit, 2 → 16-bit
+				u32 dst_fmt   : 2; // 0 → 32-bit, 1 → 24-bit, 2 → 16-bit
 				u32 depth_fmt : 2; // 0 → None, 1 → 32-bit, 2 → 16-bit, 3 → RGBA
 				// Alpha extension/Correction
 				u32 aem : 1;
@@ -301,43 +333,51 @@ struct alignas(16) GSHWDrawConfig
 				// Pixel test
 				u32 date : 3;
 				u32 atst : 3;
+				u32 afail : 2;
 				// Color sampling
 				u32 fst : 1; // Investigate to do it on the VS
 				u32 tfx : 3;
 				u32 tcc : 1;
 				u32 wms : 2;
 				u32 wmt : 2;
+				u32 adjs : 1;
+				u32 adjt : 1;
 				u32 ltf : 1;
 				// Shuffle and fbmask effect
 				u32 shuffle  : 1;
-				u32 read_ba  : 1;
+				u32 shuffle_same : 1;
+				u32 real16src: 1;
+				u32 process_ba : 2;
+				u32 process_rg : 2;
+				u32 shuffle_across : 1;
 				u32 write_rg : 1;
 				u32 fbmask   : 1;
 
-				//u32 _free1:0;
-
-				// *** Word 2
 				// Blend and Colclip
-				u32 blend_a     : 2;
-				u32 blend_b     : 2;
-				u32 blend_c     : 2;
-				u32 blend_d     : 2;
-				u32 fixed_one_a : 1;
-				u32 clr_hw      : 3;
-				u32 hdr         : 1;
-				u32 colclip     : 1;
-				u32 blend_mix   : 2;
-				u32 pabe        : 1;
-				u32 no_color    : 1; // disables color output entirely (depth only)
-				u32 no_color1   : 1; // disables second color output (when unnecessary)
-				u32 no_ablend   : 1; // output alpha blend in col0 (for no-DSB)
-				u32 only_alpha  : 1; // don't bother computing RGB
+				u32 blend_a        : 2;
+				u32 blend_b        : 2;
+				u32 blend_c        : 2;
+				u32 blend_d        : 2;
+				u32 fixed_one_a    : 1;
+				u32 blend_hw       : 3; /*HWBlendType*/
+				u32 a_masked       : 1;
+				u32 colclip_hw     : 1; // colclip (COLCLAMP off) emulation through HQ textures
+				u32 rta_correction : 1;
+				u32 rta_source_correction : 1;
+				u32 colclip        : 1; // COLCLAMP off (color blend outputs wrap around 0-255)
+				u32 blend_mix      : 2;
+				u32 round_inv      : 1; // Blending will invert the value, so rounding needs to go the other way
+				u32 pabe           : 1;
+				u32 no_color       : 1; // disables color output entirely (depth only)
+				u32 no_color1      : 1; // disables second color output (when unnecessary)
 
 				// Others ways to fetch the texture
 				u32 channel : 3;
+				u32 channel_fb : 1;
 
 				// Dithering
 				u32 dither : 2;
+				u32 dither_adjust : 1;
 
 				// Depth clamp
 				u32 zclamp : 1;
@@ -350,7 +390,7 @@ struct alignas(16) GSHWDrawConfig
 				u32 automatic_lod : 1;
 				u32 manual_lod : 1;
 				u32 point_sampler : 1;
-				u32 invalid_tex0 : 1; // Lupin the 3rd
+				u32 region_rect : 1;
 
 				// Scan mask
 				u32 scanmsk : 2;
@@ -371,8 +411,8 @@ struct alignas(16) GSHWDrawConfig
 		__fi bool IsFeedbackLoop() const
 		{
 			const u32 sw_blend_bits = blend_a | blend_b | blend_d;
-			const bool sw_blend_needs_rt = sw_blend_bits != 0 && ((sw_blend_bits | blend_c) & 1u);
-			return tex_is_fb || fbmask || (date > 0 && date != 3) || sw_blend_needs_rt;
+			const bool sw_blend_needs_rt = (sw_blend_bits != 0 && ((sw_blend_bits | blend_c) & 1u)) || ((a_masked & blend_c) != 0);
+			return channel_fb || tex_is_fb || fbmask || (date > 0 && date != 3) || sw_blend_needs_rt;
 		}
 
 		/// Disables color output from the pixel shader, this is done when all channels are masked.
@@ -391,6 +431,7 @@ struct alignas(16) GSHWDrawConfig
 			no_color = no_color1 = 1;
 		}
 	};
+	static_assert(sizeof(PSSelector) == 12, "PSSelector is 12 bytes");
 #pragma pack(pop)
 	struct PSSelectorHash
 	{
@@ -477,9 +518,9 @@ struct alignas(16) GSHWDrawConfig
 			};
 			u8 key;
 		};
-		DepthStencilSelector(): key(0) {}
-		DepthStencilSelector(u8 k): key(k) {}
-		static DepthStencilSelector NoDepth()
+		constexpr DepthStencilSelector(): key(0) {}
+		constexpr DepthStencilSelector(u8 k): key(k) {}
+		static constexpr DepthStencilSelector NoDepth()
 		{
 			DepthStencilSelector out;
 			out.ztst = ZTST_ALWAYS;
@@ -505,8 +546,8 @@ struct alignas(16) GSHWDrawConfig
 			};
 			u8 key;
 		};
-		ColorMaskSelector(): key(0xF) {}
-		ColorMaskSelector(u8 c): key(0) { wrgba = c; }
+		constexpr ColorMaskSelector(): key(0xF) {}
+		constexpr ColorMaskSelector(u8 c): key(0) { wrgba = c; }
 	};
 #pragma pack(pop)
 	struct alignas(16) VSConstantBuffer
@@ -552,17 +593,20 @@ struct alignas(16) GSHWDrawConfig
 		GSVector4 FogColor_AREF;
 		GSVector4 WH;
 		GSVector4 TA_MaxDepth_Af;
-		GSVector4i MskFix;
 		GSVector4i FbMask;
 
 		GSVector4 HalfTexel;
 		GSVector4 MinMax;
+		GSVector4 LODParams;
+		GSVector4 STRange;
 		GSVector4i ChannelShuffle;
 		GSVector2 TCOffsetHack;
 		GSVector2 STScale;
 
 		GSVector4 DitherMatrix[4];
 
+		GSVector4 ScaleFactor;
+
 		__fi PSConstantBuffer()
 		{
 			memset(this, 0, sizeof(*this));
@@ -593,23 +637,30 @@ struct alignas(16) GSHWDrawConfig
 			return true;
 		}
 	};
+	// For hardware rendering backends
 	struct BlendState
 	{
+		typedef u8 BlendOp; /*GSDevice::BlendOp*/
+		typedef u8 BlendFactor; /*GSDevice::BlendFactor*/
+
 		union
 		{
 			struct
 			{
-				u8 enable : 1;
-				u8 constant_enable : 1;
-				u8 op : 6;
-				u8 src_factor;
-				u8 dst_factor;
+				bool enable : 1;
+				bool constant_enable : 1;
+				BlendOp op : 6;
+				BlendFactor src_factor : 4;
+				BlendFactor dst_factor : 4;
+				BlendFactor src_factor_alpha : 4;
+				BlendFactor dst_factor_alpha : 4;
 				u8 constant;
 			};
 			u32 key;
 		};
-		BlendState(): key(0) {}
-		BlendState(bool enable_, u8 src_factor_, u8 dst_factor_, u8 op_, bool constant_enable_, u8 constant_)
+		constexpr BlendState(): key(0) {}
+		constexpr BlendState(bool enable_, BlendFactor src_factor_, BlendFactor dst_factor_, BlendOp op_,
+			BlendFactor src_alpha_factor_, BlendFactor dst_alpha_factor_, bool constant_enable_, u8 constant_)
 			: key(0)
 		{
 			enable = enable_;
@@ -617,8 +668,13 @@ struct alignas(16) GSHWDrawConfig
 			src_factor = src_factor_;
 			dst_factor = dst_factor_;
 			op = op_;
+			src_factor_alpha = src_alpha_factor_;
+			dst_factor_alpha = dst_alpha_factor_;
 			constant = constant_;
 		}
+
+		// Blending has no effect if RGB is masked.
+		bool IsEffective(ColorMaskSelector colormask) const;
 	};
 	enum class DestinationAlphaMode : u8
 	{
@@ -629,12 +685,21 @@ struct alignas(16) GSHWDrawConfig
 		Full,           ///< Full emulation (using barriers / ROV)
 	};
 
+	enum class ColClipMode : u8
+	{
+		NoModify = 0,
+		ConvertOnly = 1,
+		ResolveOnly = 2,
+		ConvertAndResolve = 3,
+		EarlyResolve = 4
+	};
+
 	GSTexture* rt;        ///< Render target
 	GSTexture* ds;        ///< Depth stencil
 	GSTexture* tex;       ///< Source texture
 	GSTexture* pal;       ///< Palette texture
 	const GSVertex* verts;///< Vertices to draw
-	const u32* indices;   ///< Indices to draw
+	const u16* indices;   ///< Indices to draw
 	u32 nverts;           ///< Number of vertices
 	u32 nindices;         ///< Number of indices
 	u32 indices_per_prim; ///< Number of indices that make up one primitive
@@ -644,7 +709,6 @@ struct alignas(16) GSHWDrawConfig
 	Topology topology;  ///< Draw topology
 
 	alignas(8) PSSelector ps;
-	GSSelector gs;
 	VSSelector vs;
 
 	BlendState blend;
@@ -656,15 +720,15 @@ struct alignas(16) GSHWDrawConfig
 	bool require_full_barrier; ///< Require texture barrier between all prims
 
 	DestinationAlphaMode destination_alpha;
-	bool datm : 1;
+	SetDATM datm : 2;
 	bool line_expand : 1;
-	bool separate_alpha_pass : 1;
-	bool second_separate_alpha_pass : 1;
 
 	struct AlphaPass
 	{
 		alignas(8) PSSelector ps;
-		bool enable;
+		bool enable : 1;
+		bool require_one_barrier : 1;
+		bool require_full_barrier : 1;
 		ColorMaskSelector colormask;
 		DepthStencilSelector depth;
 		float ps_aref;
@@ -673,38 +737,83 @@ struct alignas(16) GSHWDrawConfig
 
 	AlphaPass alpha_second_pass;
 
+	struct BlendMultiPass
+	{
+		BlendState blend;
+		u8 blend_hw; /*HWBlendType*/
+		u8 dither;
+		bool enable;
+	};
+	static_assert(sizeof(BlendMultiPass) == 8, "blend multi pass is 8 bytes");
+
+	BlendMultiPass blend_multi_pass;
+
 	VSConstantBuffer cb_vs;
 	PSConstantBuffer cb_ps;
+	
+	// These are here as they need to be preserved between draws, and the state clear only does up to the constant buffers.
+	ColClipMode colclip_mode;
+	GIFRegFRAME colclip_frame;
+	GSVector4i colclip_update_area; ///< Area in the framebuffer which colclip will modify;
 };
 
 class GSDevice : public GSAlignedClass<32>
 {
 public:
+	enum class PresentResult
+	{
+		OK,
+		FrameSkipped,
+		DeviceLost
+	};
+
+	enum class DebugMessageCategory
+	{
+		Cache,
+		Reg,
+		Debug,
+		Message,
+		Performance
+	};
+
 	// clang-format off
 	struct FeatureSupport
 	{
 		bool broken_point_sampler : 1; ///< Issue with AMD cards, see tfx shader for details
-		bool geometry_shader      : 1; ///< Supports geometry shader
 		bool vs_expand            : 1; ///< Supports expanding points/lines/sprites in the vertex shader
 		bool primitive_id         : 1; ///< Supports primitive ID for use with prim tracking destination alpha algorithm
 		bool texture_barrier      : 1; ///< Supports sampling rt and hopefully texture barrier
 		bool provoking_vertex_last: 1; ///< Supports using the last vertex in a primitive as the value for flat shading.
-		bool point_expand         : 1; ///< Supports point expansion in hardware without using geometry shaders.
-		bool line_expand          : 1; ///< Supports line expansion in hardware without using geometry shaders.
+		bool point_expand         : 1; ///< Supports point expansion in hardware.
+		bool line_expand          : 1; ///< Supports line expansion in hardware.
 		bool prefer_new_textures  : 1; ///< Allocate textures up to the pool size before reusing them, to avoid render pass restarts.
 		bool dxt_textures         : 1; ///< Supports DXTn texture compression, i.e. S3TC and BC1-3.
 		bool bptc_textures        : 1; ///< Supports BC6/7 texture compression.
 		bool framebuffer_fetch    : 1; ///< Can sample from the framebuffer without texture barriers.
-		bool dual_source_blend    : 1; ///< Can use alpha output as a blend factor.
-		bool clip_control         : 1; ///< Can use 0..1 depth range instead of -1..1.
 		bool stencil_buffer       : 1; ///< Supports stencil buffer, and can use for DATE.
 		bool cas_sharpening       : 1; ///< Supports sufficient functionality for contrast adaptive sharpening.
+		bool test_and_sample_depth: 1; ///< Supports concurrently binding the depth-stencil buffer for sampling and depth testing.
 		FeatureSupport()
 		{
 			memset(this, 0, sizeof(*this));
 		}
 	};
 
+	struct MultiStretchRect
+	{
+		GSVector4 src_rect;
+		GSVector4 dst_rect;
+		GSTexture* src;
+		bool linear;
+		GSHWDrawConfig::ColorMaskSelector wmask; // 0xf for all channels by default
+	};
+
+	struct TextureRecycleDeleter
+	{
+		void operator()(GSTexture* const tex);
+	};
+	using RecycledTexture = std::unique_ptr<GSTexture, TextureRecycleDeleter>;
+
 	enum BlendFactor : u8
 	{
 		// HW blend factors
@@ -720,16 +829,44 @@ public:
 	};
 	// clang-format on
 
+protected:
+	std::string m_name = "Unknown";
+	FeatureSupport m_features;
+	u32 m_max_texture_size = 0;
+
+	struct
+	{
+		u32 start, count;
+	} m_vertex = {};
+	struct
+	{
+		u32 start, count;
+	} m_index = {};
+
+	u32 m_frame = 0; // for ageing the pool
+
 private:
-	FastList<GSTexture*> m_pool;
+	std::array<FastList<GSTexture*>, 2> m_pool; // [texture, target]
+	u64 m_pool_memory_usage = 0;
+
 	static const std::array<HWBlend, 3*3*3*3> m_blendMap;
-	static const std::array<u8, 16> m_replaceDualSrcBlendMap;
 
 protected:
-	static constexpr int   NUM_INTERLACE_SHADERS = 5;
+	static constexpr int NUM_INTERLACE_SHADERS = 5;
 	static constexpr float MAD_SENSITIVITY = 0.08f;
-	static constexpr u32   MAX_POOLED_TEXTURES = 300;
-	static constexpr u32   NUM_CAS_CONSTANTS = 12; // 8 plus src offset x/y, 16 byte alignment
+	static constexpr u32 MAX_POOLED_TARGETS = 300;
+	static constexpr u32 MAX_TARGET_AGE = 20;
+	static constexpr u32 MAX_POOLED_TEXTURES = 300;
+	static constexpr u32 MAX_TEXTURE_AGE = 10;
+	static constexpr u32 NUM_CAS_CONSTANTS = 12; // 8 plus src offset x/y, 16 byte alignment
+	static constexpr u32 EXPAND_BUFFER_SIZE = sizeof(u16) * 16383 * 6;
+
+	WindowInfo m_window_info;
+	GSVSyncMode m_vsync_mode = GSVSyncMode::Disabled;
+	bool m_allow_present_throttle = false;
+	u64 m_last_frame_displayed_time = 0;
+
+	GSTexture* m_imgui_font = nullptr;
 
 	GSTexture* m_merge = nullptr;
 	GSTexture* m_weavebob = nullptr;
@@ -738,26 +875,17 @@ protected:
 	GSTexture* m_target_tmp = nullptr;
 	GSTexture* m_current = nullptr;
 	GSTexture* m_cas = nullptr;
+	GSTexture* m_colclip_rt = nullptr; ///< Temp hw colclip texture
 
-	struct
-	{
-		size_t stride, start, count, limit;
-	} m_vertex = {};
-	struct
-	{
-		size_t start, count, limit;
-	} m_index = {};
-	unsigned int m_frame = 0; // for ageing the pool
-	bool m_rbswapped = false;
-	FeatureSupport m_features;
+	bool AcquireWindow(bool recreate_window);
 
 	virtual GSTexture* CreateSurface(GSTexture::Type type, int width, int height, int levels, GSTexture::Format format) = 0;
-	GSTexture* FetchSurface(GSTexture::Type type, int width, int height, int levels, GSTexture::Format format, bool clear, bool prefer_reuse);
+	GSTexture* FetchSurface(GSTexture::Type type, int width, int height, int levels, GSTexture::Format format, bool clear, bool prefer_unused_texture);
 
-	virtual void DoMerge(GSTexture* sTex[3], GSVector4* sRect, GSTexture* dTex, GSVector4* dRect, const GSRegPMODE& PMODE, const GSRegEXTBUF& EXTBUF, const GSVector4& c) = 0;
-	virtual void DoInterlace(GSTexture* sTex, GSTexture* dTex, int shader, bool linear, float yoffset, int bufIdx) = 0;
-	virtual void DoFXAA(GSTexture* sTex, GSTexture* dTex) {}
-	virtual void DoShadeBoost(GSTexture* sTex, GSTexture* dTex, const float params[4]) {}
+	virtual void DoMerge(GSTexture* sTex[3], GSVector4* sRect, GSTexture* dTex, GSVector4* dRect, const GSRegPMODE& PMODE, const GSRegEXTBUF& EXTBUF, u32 c, const bool linear) = 0;
+	virtual void DoInterlace(GSTexture* sTex, const GSVector4& sRect, GSTexture* dTex, const GSVector4& dRect, ShaderInterlace shader, bool linear, const InterlaceConstantBuffer& cb) = 0;
+	virtual void DoFXAA(GSTexture* sTex, GSTexture* dTex) = 0;
+	virtual void DoShadeBoost(GSTexture* sTex, GSTexture* dTex, const float params[4]) = 0;
 
 	/// Resolves CAS shader includes for the specified source.
 	static bool GetCASShaderSource(std::string* source);
@@ -769,78 +897,146 @@ public:
 	GSDevice();
 	virtual ~GSDevice();
 
-	__fi unsigned int GetFrameNumber() const { return m_frame; }
+	/// Returns a string containing current adapter in use.
+	const std::string& GetName() const { return m_name; }
 
+	GSTexture* GetColorClipTexture() const { return m_colclip_rt; }
+		
+	void SetColorClipTexture(GSTexture* tex) { m_colclip_rt = tex; }
+
+	/// Returns a string representing the specified API.
+	static const char* RenderAPIToString(RenderAPI api);
+
+	/// Parses the configured fullscreen mode into its components (width * height @ refresh hz)
+	static bool GetRequestedExclusiveFullscreenMode(u32* width, u32* height, float* refresh_rate);
+
+	/// Converts a fullscreen mode to a string.
+	static std::string GetFullscreenModeString(u32 width, u32 height, float refresh_rate);
+
+	/// Generates a fixed index buffer for expanding points and sprites. Buffer is assumed to be at least EXPAND_BUFFER_SIZE in size.
+	static void GenerateExpansionIndexBuffer(void* buffer);
+
+	/// Reads the specified shader source file.
+	static std::optional<std::string> ReadShaderSource(const char* filename);
+
+	/// Returns the maximum number of mipmap levels for a given texture size.
+	static int GetMipmapLevelsForSize(int width, int height);
+
+	__fi u64 GetPoolMemoryUsage() const { return m_pool_memory_usage; }
+
+	__fi FeatureSupport Features() const { return m_features; }
+	__fi u32 GetMaxTextureSize() const { return m_max_texture_size; }
+
+	__fi const WindowInfo& GetWindowInfo() const { return m_window_info; }
+	__fi s32 GetWindowWidth() const { return static_cast<s32>(m_window_info.surface_width); }
+	__fi s32 GetWindowHeight() const { return static_cast<s32>(m_window_info.surface_height); }
+	__fi GSVector2i GetWindowSize() const { return GSVector2i(static_cast<s32>(m_window_info.surface_width), static_cast<s32>(m_window_info.surface_height)); }
+	__fi float GetWindowScale() const { return m_window_info.surface_scale; }
+	__fi GSVSyncMode GetVSyncMode() const { return m_vsync_mode; }
+	__fi bool IsPresentThrottleAllowed() const { return m_allow_present_throttle; }
+
+	__fi GSTexture* GetCurrent() const { return m_current; }
+	__fi GSTexture* GetMAD() const { return m_mad; }
+	
 	void Recycle(GSTexture* t);
 
-	enum
-	{
-		Windowed,
-		Fullscreen,
-		DontCare
-	};
+	/// Returns true if it's an OpenGL-based renderer.
+	bool UsesLowerLeftOrigin() const;
 
-	enum class DebugMessageCategory
-	{
-		Cache,
-		Reg,
-		Debug,
-		Message,
-		Performance
-	};
+	/// Recreates the font, call when the window scaling changes.
+	bool UpdateImGuiFontTexture();
 
-	virtual bool Create();
+	virtual bool Create(GSVSyncMode vsync_mode, bool allow_present_throttle);
 	virtual void Destroy();
 
-	virtual void ResetAPIState();
-	virtual void RestoreAPIState();
+	/// Returns the graphics API used by this device.
+	virtual RenderAPI GetRenderAPI() const = 0;
 
-	virtual void BeginScene() {}
-	virtual void EndScene();
+	/// Returns true if we have a window we're rendering into.
+	virtual bool HasSurface() const = 0;
 
-	virtual void ClearRenderTarget(GSTexture* t, const GSVector4& c) {}
-	virtual void ClearRenderTarget(GSTexture* t, u32 c) {}
-	virtual void InvalidateRenderTarget(GSTexture* t) {}
-	virtual void ClearDepth(GSTexture* t) {}
-	virtual void ClearStencil(GSTexture* t, u8 c) {}
+	/// Destroys the surface we're currently drawing to.
+	virtual void DestroySurface() = 0;
 
-	virtual void PushDebugGroup(const char* fmt, ...) {}
-	virtual void PopDebugGroup() {}
-	virtual void InsertDebugMessage(DebugMessageCategory category, const char* fmt, ...) {}
+	/// Switches to a new window/surface.
+	virtual bool UpdateWindow() = 0;
 
-	GSTexture* CreateRenderTarget(int w, int h, GSTexture::Format format, bool clear = true);
-	GSTexture* CreateDepthStencil(int w, int h, GSTexture::Format format, bool clear = true);
+	/// Call when the window size changes externally to recreate any resources.
+	virtual void ResizeWindow(s32 new_window_width, s32 new_window_height, float new_window_scale) = 0;
+
+	/// Returns true if exclusive fullscreen is supported.
+	virtual bool SupportsExclusiveFullscreen() const = 0;
+
+	/// Returns false if the window was completely occluded. If frame_skip is set, the frame won't be
+	/// displayed, but the GPU command queue will still be flushed.
+	virtual PresentResult BeginPresent(bool frame_skip) = 0;
+
+	/// Presents the frame to the display.
+	virtual void EndPresent() = 0;
+
+	/// Changes vsync mode for this display.
+	virtual void SetVSyncMode(GSVSyncMode mode, bool allow_present_throttle) = 0;
+
+	/// Returns a string of information about the graphics driver being used.
+	virtual std::string GetDriverInfo() const = 0;
+
+	/// Enables/disables GPU frame timing.
+	virtual bool SetGPUTimingEnabled(bool enabled) = 0;
+
+	/// Returns the amount of GPU time utilized since the last time this method was called.
+	virtual float GetAndResetAccumulatedGPUTime() = 0;
+
+	/// Returns true if not enough time has passed for present to not block.
+	bool ShouldSkipPresentingFrame();
+
+	/// Sleeps to the time the next frame can be displayed.
+	void ThrottlePresentation();
+
+	void ClearRenderTarget(GSTexture* t, u32 c);
+	void ClearDepth(GSTexture* t, float d);
+	void InvalidateRenderTarget(GSTexture* t);
+
+	virtual void PushDebugGroup(const char* fmt, ...) = 0;
+	virtual void PopDebugGroup() = 0;
+	virtual void InsertDebugMessage(DebugMessageCategory category, const char* fmt, ...) = 0;
+
+	GSTexture* CreateRenderTarget(int w, int h, GSTexture::Format format, bool clear = true, bool prefer_reuse = true);
+	GSTexture* CreateDepthStencil(int w, int h, GSTexture::Format format, bool clear = true, bool prefer_reuse = true);
 	GSTexture* CreateTexture(int w, int h, int mipmap_levels, GSTexture::Format format, bool prefer_reuse = false);
-	GSTexture* CreateOffscreen(int w, int h, GSTexture::Format format);
-	GSTexture::Format GetDefaultTextureFormat(GSTexture::Type type);
 
-	/// Download the region `rect` of `src` into `out_map`
-	/// `out_map` will be valid a call to `DownloadTextureComplete`
-	virtual bool DownloadTexture(GSTexture* src, const GSVector4i& rect, GSTexture::GSMap& out_map) { return false; }
+	virtual std::unique_ptr<GSDownloadTexture> CreateDownloadTexture(u32 width, u32 height, GSTexture::Format format) = 0;
 
-	/// Scale the region `sRect` of `src` to the size `dSize` using `ps_shader` and store the result in `out_map`
-	/// `out_map` will be valid a call to `DownloadTextureComplete`
-	virtual bool DownloadTextureConvert(GSTexture* src, const GSVector4& sRect, const GSVector2i& dSize, GSTexture::Format format, ShaderConvert ps_shader, GSTexture::GSMap& out_map, bool linear);
-
-	/// Must be called to free resources after calling `DownloadTexture` or `DownloadTextureConvert`
-	virtual void DownloadTextureComplete() {}
-
-	virtual void CopyRect(GSTexture* sTex, GSTexture* dTex, const GSVector4i& r, u32 destX, u32 destY) {}
-	virtual void StretchRect(GSTexture* sTex, const GSVector4& sRect, GSTexture* dTex, const GSVector4& dRect, ShaderConvert shader = ShaderConvert::COPY, bool linear = true) {}
-	virtual void StretchRect(GSTexture* sTex, const GSVector4& sRect, GSTexture* dTex, const GSVector4& dRect, bool red, bool green, bool blue, bool alpha) {}
+	virtual void CopyRect(GSTexture* sTex, GSTexture* dTex, const GSVector4i& r, u32 destX, u32 destY) = 0;
+	virtual void StretchRect(GSTexture* sTex, const GSVector4& sRect, GSTexture* dTex, const GSVector4& dRect, ShaderConvert shader = ShaderConvert::COPY, bool linear = true) = 0;
+	virtual void StretchRect(GSTexture* sTex, const GSVector4& sRect, GSTexture* dTex, const GSVector4& dRect, bool red, bool green, bool blue, bool alpha, ShaderConvert shader = ShaderConvert::COPY) = 0;
 
 	void StretchRect(GSTexture* sTex, GSTexture* dTex, const GSVector4& dRect, ShaderConvert shader = ShaderConvert::COPY, bool linear = true);
 
 	/// Performs a screen blit for display. If dTex is null, it assumes you are writing to the system framebuffer/swap chain.
-	virtual void PresentRect(GSTexture* sTex, const GSVector4& sRect, GSTexture* dTex, const GSVector4& dRect, PresentShader shader, float shaderTime, bool linear) {}
+	virtual void PresentRect(GSTexture* sTex, const GSVector4& sRect, GSTexture* dTex, const GSVector4& dRect, PresentShader shader, float shaderTime, bool linear) = 0;
 
-	virtual void RenderHW(GSHWDrawConfig& config) {}
+	/// Same as doing StretchRect for each item, except tries to batch together rectangles in as few draws as possible.
+	/// The provided list should be sorted by texture, the implementations only check if it's the same as the last.
+	virtual void DrawMultiStretchRects(const MultiStretchRect* rects, u32 num_rects, GSTexture* dTex, ShaderConvert shader = ShaderConvert::COPY);
 
-	__fi FeatureSupport Features() const { return m_features; }
-	__fi GSTexture* GetCurrent() const { return m_current; }
+	/// Sorts a MultiStretchRect list for optimal batching.
+	static void SortMultiStretchRects(MultiStretchRect* rects, u32 num_rects);
+
+	/// Updates a GPU CLUT texture from a source texture.
+	virtual void UpdateCLUTTexture(GSTexture* sTex, float sScale, u32 offsetX, u32 offsetY, GSTexture* dTex, u32 dOffset, u32 dSize) = 0;
+
+	/// Converts a colour format to an indexed format texture.
+	virtual void ConvertToIndexedTexture(GSTexture* sTex, float sScale, u32 offsetX, u32 offsetY, u32 SBW, u32 SPSM, GSTexture* dTex, u32 DBW, u32 DPSM) = 0;
+
+	/// Uses box downsampling to resize a texture.
+	virtual void FilteredDownsampleTexture(GSTexture* sTex, GSTexture* dTex, u32 downsample_factor, const GSVector2i& clamp_min, const GSVector4& dRect) = 0;
+
+	virtual void RenderHW(GSHWDrawConfig& config) = 0;
+
+	virtual void ClearSamplerCache() = 0;
 
 	void ClearCurrent();
-	void Merge(GSTexture* sTex[3], GSVector4* sRect, GSVector4* dRect, const GSVector2i& fs, const GSRegPMODE& PMODE, const GSRegEXTBUF& EXTBUF, const GSVector4& c);
+	void Merge(GSTexture* sTex[3], GSVector4* sRect, GSVector4* dRect, const GSVector2i& fs, const GSRegPMODE& PMODE, const GSRegEXTBUF& EXTBUF, u32 c);
 	void Interlace(const GSVector2i& ds, int field, int mode, float yoffset);
 	void FXAA();
 	void ShadeBoost();
@@ -848,24 +1044,14 @@ public:
 
 	void CAS(GSTexture*& tex, GSVector4i& src_rect, GSVector4& src_uv, const GSVector4& draw_rect, bool sharpen_only);
 
-	bool ResizeTexture(GSTexture** t, GSTexture::Type type, int w, int h, bool clear = true, bool prefer_reuse = false);
-	bool ResizeTexture(GSTexture** t, int w, int h, bool prefer_reuse = false);
-	bool ResizeTarget(GSTexture** t, int w, int h);
-	bool ResizeTarget(GSTexture** t);
-
-	bool IsRBSwapped() { return m_rbswapped; }
+	bool ResizeRenderTarget(GSTexture** t, int w, int h, bool preserve_contents, bool recycle);
 
 	void AgePool();
 	void PurgePool();
 
-	virtual void ClearSamplerCache();
-
-	virtual void PrintMemoryUsage();
-
 	__fi static constexpr bool IsDualSourceBlendFactor(u8 factor)
 	{
-		return (factor == SRC1_ALPHA || factor == INV_SRC1_ALPHA
-			/*|| factor == SRC1_COLOR || factor == INV_SRC1_COLOR*/); // not used
+		return (factor == SRC1_ALPHA || factor == INV_SRC1_ALPHA || factor == SRC1_COLOR || factor == INV_SRC1_COLOR);
 	}
 	__fi static constexpr bool IsConstantBlendFactor(u16 factor)
 	{
@@ -874,28 +1060,8 @@ public:
 
 	// Convert the GS blend equations to HW blend factors/ops
 	// Index is computed as ((((A * 3 + B) * 3) + C) * 3) + D. A, B, C, D taken from ALPHA register.
-	__ri static HWBlend GetBlend(u32 index, bool replace_dual_src)
-	{
-		HWBlend ret = m_blendMap[index];
-		if (replace_dual_src)
-		{
-			ret.src = m_replaceDualSrcBlendMap[ret.src];
-			ret.dst = m_replaceDualSrcBlendMap[ret.dst];
-		}
-		return ret;
-	}
+	__ri static HWBlend GetBlend(u32 index) { return m_blendMap[index]; }
 	__ri static u16 GetBlendFlags(u32 index) { return m_blendMap[index].flags; }
-	__fi static bool IsDualSourceBlend(u32 index)
-	{
-		return (IsDualSourceBlendFactor(m_blendMap[index].src) ||
-				IsDualSourceBlendFactor(m_blendMap[index].dst));
-	}
-
-	/// Alters the pipeline configuration for drawing the separate alpha pass.
-	static void SetHWDrawConfigForAlphaPass(GSHWDrawConfig::PSSelector* ps,
-		GSHWDrawConfig::ColorMaskSelector* cms,
-		GSHWDrawConfig::BlendState* bs,
-		GSHWDrawConfig::DepthStencilSelector* dss);
 };
 
 template <>
