@@ -1,36 +1,25 @@
-/*  PCSX2 - PS2 Emulator for PCs
- *  Copyright (C) 2002-2020  PCSX2 Dev Team
- *
- *  PCSX2 is free software: you can redistribute it and/or modify it under the terms
- *  of the GNU Lesser General Public License as published by the Free Software Found-
- *  ation, either version 3 of the License, or (at your option) any later version.
- *
- *  PCSX2 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
- *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
- *  PURPOSE.  See the GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License along with PCSX2.
- *  If not, see <http://www.gnu.org/licenses/>.
- */
+// SPDX-FileCopyrightText: 2002-2025 PCSX2 Dev Team
+// SPDX-License-Identifier: GPL-3.0+
 
-#include <stdexcept>
-#include <cstdlib>
-#include <string>
-#include <cerrno>
-#include <cassert>
+#include "Host.h"
+#include "StateWrapper.h"
+#include "USB/USB.h"
+#include "USB/deviceproxy.h"
+#include "USB/qemu-usb/USBinternal.h"
+#include "USB/qemu-usb/desc.h"
 
-#include "PrecompiledHeader.h"
+#include "common/Console.h"
 #include "common/SettingsInterface.h"
 #include "common/WindowInfo.h"
-#include "USB.h"
-#include "qemu-usb/USBinternal.h"
-#include "qemu-usb/desc.h"
-#include "deviceproxy.h"
 
-#include "HostSettings.h"
-#include "StateWrapper.h"
 #include "fmt/format.h"
 
+#include <cassert>
+#include <cerrno>
+#include <cstdlib>
+#include <stdexcept>
+#include <string>
+
 #define PSXCLK 36864000 /* 36.864 Mhz */
 
 namespace USB
@@ -122,10 +111,9 @@ void USB::UpdateDevice(u32 port)
 	s_usb_device_proxy[port]->UpdateSettings(s_usb_device[port], *Host::GetSettingsInterface());
 }
 
-s32 USBinit()
+void USBinit()
 {
 	RegisterDevice::Register();
-	return 0;
 }
 
 void USBshutdown()
@@ -262,15 +250,19 @@ void USB::DoDeviceState(USBDevice* dev, StateWrapper& sw)
 	sw.Do(&dev->setup_index);
 
 	sw.Do(&dev->configuration);
-	usb_desc_set_config(dev, dev->configuration);
+	if (sw.IsReading())
+		usb_desc_set_config(dev, dev->configuration);
 
 	int altsetting[USB_MAX_INTERFACES];
 	std::memcpy(altsetting, dev->altsetting, sizeof(altsetting));
 	sw.DoPODArray(altsetting, std::size(altsetting));
-	for (u32 i = 0; i < USB_MAX_INTERFACES; i++)
+	if (sw.IsReading())
 	{
-		dev->altsetting[i] = altsetting[i];
-		usb_desc_set_interface(dev, i, altsetting[i]);
+		for (u32 i = 0; i < USB_MAX_INTERFACES; i++)
+		{
+			dev->altsetting[i] = altsetting[i];
+			usb_desc_set_interface(dev, i, altsetting[i]);
+		}
 	}
 
 	DoEndpointState(&dev->ep_ctl, sw);
@@ -539,7 +531,7 @@ s64 usb_get_clock()
 	return s_usb_clocks;
 }
 
-s32 USB::DeviceTypeNameToIndex(const std::string_view& device)
+s32 USB::DeviceTypeNameToIndex(const std::string_view device)
 {
 	RegisterDevice& rd = RegisterDevice::instance();
 	return rd.Index(device);
@@ -549,58 +541,58 @@ const char* USB::DeviceTypeIndexToName(s32 device)
 {
 	RegisterDevice& rd = RegisterDevice::instance();
 	const DeviceProxy* proxy = (device != DEVTYPE_NONE) ? rd.Device(device) : nullptr;
-	return proxy ? proxy->TypeName() : "None";
+	return proxy ? proxy->TypeName() : TRANSLATE_NOOP("USB", "None");
 }
 
-std::vector<std::pair<std::string, std::string>> USB::GetDeviceTypes()
+std::vector<std::pair<const char*, const char*>> USB::GetDeviceTypes()
 {
 	RegisterDevice& rd = RegisterDevice::instance();
-	std::vector<std::pair<std::string, std::string>> ret;
+	std::vector<std::pair<const char*, const char*>> ret;
 	ret.reserve(rd.Map().size() + 1);
-	ret.emplace_back("None", "Not Connected");
+	ret.emplace_back("None", TRANSLATE_NOOP("USB", "Not Connected"));
 	for (const auto& it : rd.Map())
 		ret.emplace_back(it.second->TypeName(), it.second->Name());
 	return ret;
 }
 
-const char* USB::GetDeviceName(const std::string_view& device)
+const char* USB::GetDeviceName(const std::string_view device)
 {
 	const DeviceProxy* dev = RegisterDevice::instance().Device(device);
-	return dev ? dev->Name() : "Not Connected";
+	return dev ? dev->Name() : TRANSLATE_NOOP("USB", "Not Connected");
 }
 
-const char* USB::GetDeviceSubtypeName(const std::string_view& device, u32 subtype)
+const char* USB::GetDeviceSubtypeName(const std::string_view device, u32 subtype)
 {
 	const DeviceProxy* dev = RegisterDevice::instance().Device(device);
 	if (!dev)
 		return "Unknown";
 
-	const gsl::span<const char*> subtypes(dev->SubTypes());
+	const std::span<const char*> subtypes(dev->SubTypes());
 	if (subtypes.empty() || subtype >= subtypes.size())
 		return "";
 
 	return subtypes[subtype];
 }
 
-gsl::span<const char*> USB::GetDeviceSubtypes(const std::string_view& device)
+std::span<const char*> USB::GetDeviceSubtypes(const std::string_view device)
 {
 	const DeviceProxy* dev = RegisterDevice::instance().Device(device);
-	return dev ? dev->SubTypes() : gsl::span<const char*>();
+	return dev ? dev->SubTypes() : std::span<const char*>();
 }
 
-gsl::span<const InputBindingInfo> USB::GetDeviceBindings(const std::string_view& device, u32 subtype)
+std::span<const InputBindingInfo> USB::GetDeviceBindings(const std::string_view device, u32 subtype)
 {
 	const DeviceProxy* dev = RegisterDevice::instance().Device(device);
-	return dev ? dev->Bindings(subtype) : gsl::span<const InputBindingInfo>();
+	return dev ? dev->Bindings(subtype) : std::span<const InputBindingInfo>();
 }
 
-gsl::span<const SettingInfo> USB::GetDeviceSettings(const std::string_view& device, u32 subtype)
+std::span<const SettingInfo> USB::GetDeviceSettings(const std::string_view device, u32 subtype)
 {
 	const DeviceProxy* dev = RegisterDevice::instance().Device(device);
-	return dev ? dev->Settings(subtype) : gsl::span<const SettingInfo>();
+	return dev ? dev->Settings(subtype) : std::span<const SettingInfo>();
 }
 
-gsl::span<const InputBindingInfo> USB::GetDeviceBindings(u32 port)
+std::span<const InputBindingInfo> USB::GetDeviceBindings(u32 port)
 {
 	pxAssert(port < NUM_PORTS);
 	if (s_usb_device_proxy[port])
@@ -627,7 +619,7 @@ void USB::SetDeviceBindValue(u32 port, u32 bind_index, float value)
 	s_usb_device_proxy[port]->SetBindingValue(s_usb_device[port], bind_index, value);
 }
 
-void USB::InputDeviceConnected(const std::string_view& identifier)
+void USB::InputDeviceConnected(const std::string_view identifier)
 {
 	for (u32 i = 0; i < NUM_PORTS; i++)
 	{
@@ -636,7 +628,7 @@ void USB::InputDeviceConnected(const std::string_view& identifier)
 	}
 }
 
-void USB::InputDeviceDisconnected(const std::string_view& identifier)
+void USB::InputDeviceDisconnected(const std::string_view identifier)
 {
 	for (u32 i = 0; i < NUM_PORTS; i++)
 	{
@@ -655,21 +647,27 @@ void USB::SetConfigDevice(SettingsInterface& si, u32 port, const char* devname)
 	si.SetStringValue(GetConfigSection(port).c_str(), "Type", devname);
 }
 
-u32 USB::GetConfigSubType(const SettingsInterface& si, u32 port, const std::string_view& devname)
+u32 USB::GetConfigSubType(const SettingsInterface& si, u32 port, const std::string_view devname)
 {
 	return si.GetUIntValue(GetConfigSection(port).c_str(), fmt::format("{}_subtype", devname).c_str(), 0u);
 }
 
-void USB::SetConfigSubType(SettingsInterface& si, u32 port, const std::string_view& devname, u32 subtype)
+void USB::SetConfigSubType(SettingsInterface& si, u32 port, const std::string_view devname, u32 subtype)
 {
 	si.SetUIntValue(GetConfigSection(port).c_str(), fmt::format("{}_subtype", devname).c_str(), subtype);
 }
 
-std::string USB::GetConfigSubKey(const std::string_view& device, const std::string_view& bind_name)
+std::string USB::GetConfigSubKey(const std::string_view device, const std::string_view bind_name)
 {
 	return fmt::format("{}_{}", device, bind_name);
 }
 
+bool USB::ConfigKeyExists(SettingsInterface& si, u32 port, const char* devname, const char* key)
+{
+	const std::string real_key(fmt::format("{}_{}", devname, key));
+	return si.ContainsValue(GetConfigSection(port).c_str(), real_key.c_str());
+}
+
 bool USB::GetConfigBool(SettingsInterface& si, u32 port, const char* devname, const char* key, bool default_value)
 {
 	const std::string real_key(fmt::format("{}_{}", devname, key));
@@ -748,8 +746,8 @@ bool USB::MapDevice(SettingsInterface& si, u32 port, const std::vector<std::pair
 
 void USB::ClearPortBindings(SettingsInterface& si, u32 port)
 {
-	const std::string section(GetConfigSection(port));
-	const std::string type(GetConfigDevice(si, port));
+	const std::string section = GetConfigSection(port);
+	const std::string type = GetConfigDevice(si, port);
 	const u32 subtype = GetConfigSubType(si, port, type);
 	const DeviceProxy* dev = RegisterDevice::instance().Device(type);
 	if (!dev)
@@ -759,6 +757,47 @@ void USB::ClearPortBindings(SettingsInterface& si, u32 port)
 		si.DeleteValue(section.c_str(), GetConfigSubKey(type, bi.name).c_str());
 }
 
+void USB::CopyConfiguration(SettingsInterface* dest_si, const SettingsInterface& src_si,
+	bool copy_devices, bool copy_bindings)
+{
+	for (u32 port = 0; port < NUM_PORTS; port++)
+	{
+		const std::string section = GetConfigSection(port);
+		const std::string type = GetConfigDevice(src_si, port);
+		const u32 subtype = GetConfigSubType(src_si, port, type);
+		const DeviceProxy* dev = RegisterDevice::instance().Device(type);
+
+		if (copy_devices)
+		{
+			dest_si->CopyStringValue(src_si, section.c_str(), "Type");
+			if (dev)
+			{
+				dest_si->CopyUIntValue(src_si, section.c_str(), fmt::format("{}_subtype", type).c_str());
+
+				for (const SettingInfo& si : dev->Settings(subtype))
+					si.CopyValue(dest_si, src_si, section.c_str(), GetConfigSubKey(type, si.name).c_str());
+			}
+		}
+
+		if (copy_bindings && dev)
+		{
+			for (const InputBindingInfo& bi : dev->Bindings(subtype))
+				dest_si->CopyStringValue(src_si, section.c_str(), GetConfigSubKey(type, bi.name).c_str());
+		}
+	}
+}
+
+void USB::SetDefaultConfiguration(SettingsInterface* si)
+{
+	for (u32 port = 0; port < NUM_PORTS; port++)
+	{
+		const std::string section = GetConfigSection(port);
+
+		si->ClearSection(section.c_str());
+		si->SetStringValue(section.c_str(), "Type", "None");
+	}
+}
+
 void USB::CheckForConfigChanges(const Pcsx2Config& old_config)
 {
 	static_assert(Pcsx2Config::USBOptions::NUM_PORTS == NUM_PORTS);
