@@ -1,17 +1,5 @@
-/*  PCSX2 - PS2 Emulator for PCs
- *  Copyright (C) 2002-2022  PCSX2 Dev Team
- *
- *  PCSX2 is free software: you can redistribute it and/or modify it under the terms
- *  of the GNU Lesser General Public License as published by the Free Software Found-
- *  ation, either version 3 of the License, or (at your option) any later version.
- *
- *  PCSX2 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
- *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
- *  PURPOSE.  See the GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License along with PCSX2.
- *  If not, see <http://www.gnu.org/licenses/>.
- */
+// SPDX-FileCopyrightText: 2002-2025 PCSX2 Dev Team
+// SPDX-License-Identifier: GPL-3.0+
 
 #include "USB/shared/ringbuffer.h"
 #include "USB/USB.h"
@@ -31,7 +19,7 @@ namespace usb_mic
 		class CubebAudioDevice final : public AudioDevice
 		{
 		public:
-			CubebAudioDevice(u32 port, AudioDir dir, u32 channels, std::string devname, s32 latency);
+			CubebAudioDevice(AudioDir dir, u32 channels, std::string devname, s32 latency);
 			~CubebAudioDevice();
 
 			static std::vector<std::pair<std::string, std::string>> GetDeviceList(bool input);
@@ -42,7 +30,6 @@ namespace usb_mic
 			void SetResampling(int samplerate) override;
 			bool Start() override;
 			void Stop() override;
-			bool Compare(AudioDevice* compare) const override;
 
 		protected:
 			void ResetBuffers();
@@ -52,6 +39,7 @@ namespace usb_mic
 
 			u32 mSampleRate = 48000;
 			u32 mLatency = 50;
+			u32 mStreamLatency = 0;
 			cubeb* mContext;
 			cubeb_stream* mStream = nullptr;
 			std::string mDeviceName;
