@@ -1,26 +1,12 @@
-/*  PCSX2 - PS2 Emulator for PCs
- *  Copyright (C) 2002-2010  PCSX2 Dev Team
- *
- *  PCSX2 is free software: you can redistribute it and/or modify it under the terms
- *  of the GNU Lesser General Public License as published by the Free Software Found-
- *  ation, either version 3 of the License, or (at your option) any later version.
- *
- *  PCSX2 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
- *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
- *  PURPOSE.  See the GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License along with PCSX2.
- *  If not, see <http://www.gnu.org/licenses/>.
- */
+// SPDX-FileCopyrightText: 2002-2025 PCSX2 Dev Team
+// SPDX-License-Identifier: GPL-3.0+
 
-
-#include "PrecompiledHeader.h"
-
-#include "System.h"
-#include "iR5900.h"
+#include "Config.h"
+#include "R3000A.h"
 #include "Vif.h"
 #include "VU.h"
-#include "R3000A.h"
+#include "x86/iCore.h"
+#include "x86/iR5900.h"
 
 using namespace x86Emitter;
 
@@ -41,7 +27,7 @@ _x86regs x86regs[iREGCNT_GPR], s_saveX86regs[iREGCNT_GPR];
 // Clear allocation counter
 void _initXMMregs()
 {
-	memzero(xmmregs);
+	std::memset(xmmregs, 0, sizeof(xmmregs));
 	g_xmmAllocCounter = 0;
 }
 
@@ -906,11 +892,11 @@ int _allocIfUsedFPUtoXMM(int fpureg, int mode)
 void _recClearInst(EEINST* pinst)
 {
 	// we set everything as being live to begin with, since it needs to be written at the end of the block
-	memzero(*pinst);
-	memset8<EEINST_LIVE>(pinst->regs);
-	memset8<EEINST_LIVE>(pinst->fpuregs);
-	memset8<EEINST_LIVE>(pinst->vfregs);
-	memset8<EEINST_LIVE>(pinst->viregs);
+	std::memset(pinst, 0, sizeof(EEINST));
+	std::memset(pinst->regs, EEINST_LIVE, sizeof(pinst->regs));
+	std::memset(pinst->fpuregs, EEINST_LIVE, sizeof(pinst->fpuregs));
+	std::memset(pinst->vfregs, EEINST_LIVE, sizeof(pinst->vfregs));
+	std::memset(pinst->viregs, EEINST_LIVE, sizeof(pinst->viregs));
 }
 
 // returns nonzero value if reg has been written between [startpc, endpc-4]
