@@ -1,27 +1,15 @@
-/*  PCSX2 - PS2 Emulator for PCs
- *  Copyright (C) 2002-2010  PCSX2 Dev Team
- *
- *  PCSX2 is free software: you can redistribute it and/or modify it under the terms
- *  of the GNU Lesser General Public License as published by the Free Software Found-
- *  ation, either version 3 of the License, or (at your option) any later version.
- *
- *  PCSX2 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
- *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
- *  PURPOSE.  See the GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License along with PCSX2.
- *  If not, see <http://www.gnu.org/licenses/>.
- */
+// SPDX-FileCopyrightText: 2002-2025 PCSX2 Dev Team
+// SPDX-License-Identifier: GPL-3.0+
 
-
-#include "PrecompiledHeader.h"
-#include <time.h>
+#include <ctime>
 
 #include "iR3000A.h"
 #include "IopMem.h"
 #include "IopDma.h"
 #include "IopGte.h"
 
+#include "common/Console.h"
+
 using namespace x86Emitter;
 
 extern int g_psxWriteOk;
@@ -2345,7 +2333,7 @@ void rpsxpropCP2_basic(EEINST* prev, EEINST* pinst)
 			break;
 
 		default:
-			pxFailDev("iop invalid opcode in const propagation (rpsxpropCP2/BASIC)");
+			pxFail("iop invalid opcode in const propagation (rpsxpropCP2/BASIC)");
 			break;
 	}
 }
